package CodeGen;

/**
 * Insert the type's description here.
 * Creation date: (8/11/00 9:36:09 AM)
 * @author: 
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import system.*;
import java.util.*;
import java.io.*;
public class CheckComp {
	MessageGUI gui;
/**
 * CheckComp constructor comment.
 */
public CheckComp() {
	super();
}
/**
 * Insert the method's description here.
 * Creation date: (8/11/00 12:23:40 PM)
 * @param action CodeGen.Action
 * @param comp system.ATagentComponent
 */
private void addMethod(Action action, ATagentComponent comp) {
	ATagentComponentMethod m = new ATagentComponentMethod(0);
	m.setName(action.name);

	/* build variable string */
	if (action.vars.length > 0) {
		String v = "Object " + action.vars[0];
		for (int i = 1; i < action.vars.length; i++)
			v = v + ", Object " + action.vars[i];
		m.setParams(v);
	} else
		m.setParams("");

	/* set return type */
	m.setReturnType(action.ret);
	m.setVisible(true);
	m.setPre("");
	if (action.ret.equals("void"))
		m.setPost("");
	else
		m.setPost("return null;");
	m.setMethodView();
	comp.addMethod(m);
}
/**
 * Checks each agent for component completion.
 *
 * Creation date: (8/11/00 9:56:52 AM)
 * @param agent system.ATagent
 */
private void checkAgent(ATagent agent, ATsystem sys) {
	gui.write("Component Checking agent " + agent.getName() + ".");

	/* find "default" component */
	ATagentComponent comp = agent.getArchitecture().findComponentbyname(agent.getName());
	if (comp == null)
		gui.write("** Error - no default component for " + agent.getName() + ".");
	else {
		/* add main method */
		Action action = getAction("main()");
		if (!methodExists(action, comp)) {
			gui.write("\t #Method " + action.name + " NOT found in component - adding component!");
			addMethod(action, comp);
		}

		/* enumerate each conversation */
		Enumeration convsEnum = sys.returnConvs();
		while (convsEnum.hasMoreElements()) {
			ATconversation c = (ATconversation) convsEnum.nextElement();
			if (c.getInitiatorKey().equals(agent.getKey())) {
				checkConversation(c.getStateTable(1), comp);
			}
			if (c.getResponderKey().equals(agent.getKey())) {
				checkConversation(c.getStateTable(2), comp);
			}
		}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (8/11/00 10:37:55 AM)
 * @param conv system.ATstatetable
 * @param comp system.ATagentComponent
 */
private void checkConversation(ATstatetable conv, ATagentComponent comp) {
	gui.write("\tChecking conversation for " + conv.getParticipantName());

	// step through states
	Enumeration stateEnum = conv.returnStates();
	while (stateEnum.hasMoreElements()) {
		try {
			ATstate s = (ATstate) stateEnum.nextElement();
			gui.write("\t  State = " + s.getName());

			// check all actions
			Vector v = (Vector) s.getActions();
			Enumeration actions = v.elements();
			while (actions.hasMoreElements()) {
				String[] vars = {};
				String ret = "";
				String a = (String) actions.nextElement();
				if (!a.equals("")) {
					Action action = getAction(a);

					/*now check to see if action is in component */
/*					boolean found = false;
					Vector vm = (Vector) comp.getMethods();
					Enumeration methods = vm.elements();
					while (methods.hasMoreElements()) {
						ATagentComponentMethod m = (ATagentComponentMethod) methods.nextElement();
						if (m.getName().equals(action.name)) {
							found = true;
							gui.write("\t #Method " + action.name + " found in component!");
						}
					}
*/
					if (!methodExists(action, comp)){
						gui.write("\t #Method " + action.name + " NOT found in component - adding component!");
						addMethod(action, comp);
					}
				}
			}

			// skip start and end states
		} catch (ClassCastException e) {
			// skip start and end states ???
		}
	}
}
/**
 * genCheckComp
 *
 * This method should check to ensure that all activities defined in the
 * conversations have method definitions in the default agent component.
 * If the methods do not exist, it should create them.  If they already
 * exist, the method should leave them alone.
 *
 * This should simplify code generation and enforce assumptions on the
 * definition internal agent design.
 *
 * Creation date: (8/11/00 9:38:40 AM)
 * @param sys system.ATsystem
 */
public void genComponents(ATsystem sys) {
	gui = new MessageGUI("agentTool Component Generation ");
	
	/* check components for each agent */
	Enumeration agentsEnum = sys.returnAgents();
	while (agentsEnum.hasMoreElements())
		checkAgent((ATagent) agentsEnum.nextElement(), sys);
 		
	gui.write("Component Generation Complete.");
}
/**
 * Insert the method's description here.
 * Creation date: (8/11/00 11:25:03 AM)
 * @return CodeGen.Action
 * @param a java.lang.String
 */
private Action getAction(String a) {
	String name = "";
	String[] vars = {};
	String ret = "";
	String[] variables = {};

	/* compute values */
	if (!a.equals("")) {
		vars = parseTuple(a.substring(a.indexOf('=') + 1));
		name = vars[0];

		/* get return value */
		if (a.indexOf('=') > 0) {
			ret = a.substring(0, a.indexOf('='));
		} else
			ret = "void";

		/* copy parameters into variable array */
		if (vars.length > 0) {
			variables = new String[vars.length - 1];
			System.arraycopy(vars, 1, variables, 0, vars.length - 1);
		}

		/* return what we found */
		return new Action(name, variables, ret);
	} else
		return null;
}
/**
 * Check to see if action is in component.
 * Creation date: (8/16/00 9:03:02 AM)
 * @return boolean
 * @param a CodeGen.Action
 * @param comp system.ATagentComponent
 */
private boolean methodExists(Action action, ATagentComponent comp) {
	boolean found = false;
	Vector vm = (Vector) comp.getMethods();
	Enumeration methods = vm.elements();
	while (methods.hasMoreElements()) {
		ATagentComponentMethod m = (ATagentComponentMethod) methods.nextElement();
		if (m.getName().equals(action.name)) {
			found = true;
			gui.write("\t #Method " + action.name + " found in component!");
		}
	}
	return found;
}
/**
 * This method was created in VisualAge.
 *
 * This method parses tuple notation such as <x, y, z>
 */
private String[] parseTuple(String s) {
	String delimiters = "\n\t\r<,>()= ";
	StringTokenizer tokens = new StringTokenizer(s, delimiters);
	if (tokens.countTokens() > 0) {
		String x[] = new String[tokens.countTokens()];
		int i = 0;
		while (tokens.hasMoreTokens())
			x[i++] = tokens.nextToken();

		/* return the vector of elements */
		return x;
	} else
		return new String[0];
}
}
