package Transforms;

/**
 * This class represents an action in a state or on a transition
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import javax.swing.JOptionPane;
 
public class Action {
	private Vector			lhs = null;			// SEQUENCE of strings that represent the lhs of an assignment
	private Vector			rhs = null;			// SEQUENCE of strings that represent the rhs of an assignment
	private FunctionCall	function = null;	// function call in rhs of assignment
/**
 * Insert the method's description here.
 * Creation date: (11/5/00 12:52:17 AM)
 */
public Action() {
}
/**
 * Insert the method's description here.
 * Creation date: (11/5/00 12:37:50 AM)
 */
public Action(Vector l, Vector r) {
	lhs = l;
	rhs = r;
}
/**
 * Insert the method's description here.
 * Creation date: (11/5/00 12:36:53 AM)
 */
public Action(Vector l, FunctionCall f) {
	lhs = l;
	function = f;
}
/**
 * Insert the method's description here.
 * Creation date: (11/5/00 12:35:53 AM)
 */
 
public Action(FunctionCall f) {
	function = f;
}
/**
 * Creates a new Action that is a copy of this one
 *
 * Creation date: (11/16/00 9:02:29 AM)
 */
public Action copy() {
	if(lhs != null){
		if(rhs != null){
			return new Action((Vector)lhs.clone(), (Vector)rhs.clone());
		}
		else if(function != null){
			return new Action((Vector)lhs.clone(), function.copy());
		}
		else
			return null;
	}
	else{
		if(function != null){
			return new Action(function.copy());
		}
		else
			return null;
	}
	
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 7:43:13 PM)
 * @return Transforms.FunctionCall
 */
public FunctionCall getFunction() {
	return function;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 7:43:14 PM)
 * @return java.util.Vector
 */
public java.util.Vector getLhs() {
	return lhs;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 7:43:14 PM)
 * @return java.util.Vector
 */
public java.util.Vector getRhs() {
	return rhs;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 7:43:13 PM)
 * @param newFunction Transforms.FunctionCall
 */
public void setFunction(FunctionCall newFunction) {
	function = newFunction;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 7:43:14 PM)
 * @param newLhs java.util.Vector
 */
public void setLhs(java.util.Vector newLhs) {
	lhs = newLhs;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 7:43:14 PM)
 * @param newRhs java.util.Vector
 */
public void setRhs(java.util.Vector newRhs) {
	rhs = newRhs;
}
/**
 * Used for debugging purposes
 *
 * Creation date: (11/5/00 12:53:29 AM)
 */
public void show() {
	String param;
	
	//System.out.println("** Action **");
	if(lhs != null){
		// print out the lhs
		if(lhs.size() == 1){
			param = (String)lhs.elementAt(0);
			System.out.print("\t" + param + " = ");
		}
		else{
			System.out.print("\t<");
			for(int i = 0; i < lhs.size(); i++){
				param = (String)lhs.elementAt(i);
				System.out.print(param);
				if((i+1) < lhs.size()){
					System.out.print(", ");
				}
			}
			System.out.print("> = ");
		}
	}
	else{
		System.out.print("\t");
	}

	if(rhs != null){
		// print out the rhs
		if(rhs.size() == 1){
			param = (String)rhs.elementAt(0);
			System.out.println(param);
		}
		else{
			System.out.print("<");
			for(int i = 0; i < rhs.size(); i++){
				param = (String)rhs.elementAt(i);
				System.out.print(param);
				if((i+1) < rhs.size()){
					System.out.print(", ");
				}
			}
			System.out.println(">");
		}
	}

	if(function != null){
		function.show();
	}
}
/**
 * Returns the Action in the form of a String
 *
 * Creation date: (11/7/00 12:06:14 PM)
 */
public String toString() {
	String action = "";
	String param;
	
	//System.out.println("** Action **");
	if(lhs != null){
		// print out the lhs
		if(lhs.size() == 1){
			param = (String)lhs.elementAt(0);
			action = action + param + " = ";
		}
		else{
			action = action + "<";
			for(int i = 0; i < lhs.size(); i++){
				param = (String)lhs.elementAt(i);
				action = action + param;
				if((i+1) < lhs.size()){
					action = action + ", ";
				}
			}
			action = action + "> = ";
		}
	}

	if(rhs != null){
		// print out the rhs
		if(rhs.size() == 1){
			param = (String)rhs.elementAt(0);
			action = action + param;
		}
		else{
			action = action + "<";
			for(int i = 0; i < rhs.size(); i++){
				param = (String)rhs.elementAt(i);
				action = action + param;
				if((i+1) < rhs.size()){
					action = action + ", ";
				}
			}
			action = action + ">";
		}
	}

	if(function != null){
		action = action + function.toString();
	}

	return action;
}
/**
 * This method is used to translate a string into an action
 *
 * Creation date: (11/5/00 12:05:00 AM)
 */
public static Action translateIn(String atAction) {
	StringTokenizer 	st = new StringTokenizer(atAction, "=");
	StringTokenizer		st2;
	String 				l, r;
	int 				tokens;
	Action				action = new Action();
	
	tokens = st.countTokens();

	if(tokens == 0)
		return null;
	else if(tokens == 1){
		// process as a function call
		r = st.nextToken();
		FunctionCall f = FunctionCall.translateIn(r);

		// Now create the action
		action = new Action(f);
	}
	else if(tokens == 2){
		// process 1st as lhs
		l = st.nextToken();
		st2 = new StringTokenizer(l, "<,> ");
		Vector lhs = new Vector();
		while(st2.hasMoreTokens()){
			lhs.add(st2.nextToken());
		}

		// process 2nd as rhs
		r = st.nextToken();
		st2 = new StringTokenizer(r, "(");
		tokens = st2.countTokens();
		if(tokens == 1){
			// rhs is a tuple
			st2 = new StringTokenizer(r, "<,> ");
			Vector rhs = new Vector();
			while(st2.hasMoreTokens()){
				rhs.add(st2.nextToken());
			}

			// Now create the action
			action = new Action(lhs, rhs);
		}
		else{
			// rhs is a function call
			FunctionCall f = FunctionCall.translateIn(r);

			// Now create the action
			action = new Action(lhs, f);
		}			
	}
	//action.show();
	return action;
}
}
