package Transforms;

/**
 * This class represents the Component Mobility Transformation
 * (CMT10) in the Mobile Design transformation process.
 * This transform adds transitions to handle the process of a
 * component receiving a move required message from the agent component.
 *
 * Creation date: (12/28/00 12:07:00 AM)
 * @author: Athie Self
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.*;

public class CMTransform10 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
	Vector statesSelected = new Vector();
	Component c = null;
/**
 * CMTransform10 constructor comment.
 */

public CMTransform10(TSystem t, Component comp, Vector ss) {
	super();

	sys = t.copy();
	currentSys = t;
	beforeSys = t.copy();
	c = comp;
	statesSelected = (Vector)ss.clone();
	setRationale("");
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method will only return true if the target is a TSystem 
 * and that system has roles and tasks.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)){
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * the params attribute is not used
 */

public void execute(Object params) {	
								
	Vector			transitions = null;
	Transition		t;
	boolean			boolTransition = false;
	
	ATagentComponent atComponent = (ATagentComponent)c.getAtComponent();

	StateTable st = c.getStateTable();
	ATstatetable atStateTable = atComponent.getStateTable();	
	ATtableElement atRestore = atStateTable.findStatebyname("restore");
	ATtableElement atMoveReceived = atStateTable.findStatebyname("moveReceived");
	
	// Go through all of the selected states and 
	// add the appropriate transitions from the 
	// restore state to a selected state and from
	// a selected state to the moveReceived state.
	for(int i = 0; i < statesSelected.size(); i++) {

		State s = (State)statesSelected.elementAt(i);
		ATtableElement atState = s.getElement();
		
		// add transitions to the state table
		ATtransition atTransition1 = new ATtransition(sys.version,sys.atSys.getNextKey());
		atTransition1.setRecMessage("moveReq");
		atTransition1.setCoordinates(atState,atMoveReceived);
		atStateTable.addTransition(atTransition1);

		// check to see if the state selected already has a transition
		// from the restore state to it.  If yes, do not add the
		// transition.  If no, then add the transition.
		transitions = st.getTransitions();
		
		for(int j = 0; j < transitions.size(); j++) {
			
			t = (Transition)transitions.elementAt(j);
			
			if ((t.getFromName().equals("restore"))
				&& (t.getToName().equals(s.getName()))) {
				boolTransition = true;
			}
		}

		if (!boolTransition) {
			ATtransition atTransition2 = new ATtransition(sys.version,sys.atSys.getNextKey());
			atTransition2.setGuardCondition("state=="+atState.getName());
			atTransition2.setCoordinates(atRestore,atState);
			atStateTable.addTransition(atTransition2);					
		}
			
		atComponent.setStateTable(atStateTable);
	}				
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
