package Transforms;

/**
 * This class represents the Component Mobility Transformation
 * (CMT10) in the Mobile Design transformation process.
 * This transform adds transitions to handle the process of a
 * component receiving a move required message from the agent component.
 *
 * Creation date: (12/28/00 12:07:00 AM)
 * @author: Athie Self
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import javax.swing.JOptionPane;
import system.*;

public class CMTransform10ComponentSelection extends Thread {
	TSystem sys = null;
	Vector candidateStates		= new Vector();
	Vector vecSelectedStates	= new Vector();
/**
 * CMTransform10ComponentSelection constructor comment.
 */

public CMTransform10ComponentSelection(TSystem t) {
	sys = t;
	start();
}
/**
 * This method contains the flow of events throughout the phase
 *
 * Creation date: (12/6/00 12:27:39 PM)
 */
public synchronized void determineComponentsForCMT11() {

	Vector agents = sys.agents;
	boolean boolHasTransientReactiveMobileComponent = false;
	int intMobileComponentCount = 0;
		
	for(int i = 0; i < agents.size(); i++) {
	
		Agent ag = (Agent)agents.elementAt(i);
		String strAgentName = new String(ag.getName());
		
		if(isMobility_Specified(ag)) {
			
			ATagent atAgent = ag.getAtAgent();
			ATarchitecture atArch = atAgent.getArchitecture();

			Vector components = ag.getComponents();

			for(int j = 0; j < components.size(); j++) {

				Component c = (Component)components.elementAt(j);

				if(!(c.getName().equals("AgentComponent"))) {

					ATagentComponent atComponent = (ATagentComponent)c.getAtComponent();

					StateTable st = c.getStateTable();
					ATstatetable atStateTable = atComponent.getStateTable();	
					ATtableElement atMoveReceived = atStateTable.findStatebyname("moveReceived");
					
					if (atMoveReceived != null) {
						
						selectStates(c, strAgentName);
						
						CMTransform10 cmt10 = new CMTransform10(sys,c,vecSelectedStates);
					
						vecSelectedStates.clear();
					}
				}
			}
		}
	}	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param a Transforms.Agent
 */

public boolean isMobility_Specified(Agent a) {

	Vector actions    = new Vector();
	Vector states     = new Vector();
	
	Vector components = a.getComponents();
	
	if(components != null) {

		for(int j = 0; j < components.size(); j++) {

			Component c = (Component)components.elementAt(j);

			if (!c.getName().equals("AgentComponent")) {

				StateTable st = c.getStateTable();

				if(st != null) {
					states = st.getStates();
				}

				if(states != null) {					

					for(int k = 0; k < states.size(); k++) {

						State s = (State)states.elementAt(k);

						actions = s.getActions();

						if(actions != null) {

							for(int l = 0; l < actions.size(); l++) {

								Action act = (Action)actions.elementAt(l);

								FunctionCall fc = (FunctionCall)act.getFunction();

								if(fc!=null) {

									if(fc.getName().equals("move")) {
										return true;
									}
								}	
							}		
						}	
					}
				}
			}
		}
	}
	
	return false;	
}
/**
 * This method contains the flow of events throughout the phase
 *
 * Creation date: (12/6/00 12:27:39 PM)
 */
public synchronized void run() {

	determineComponentsForCMT11();
						
}
/**
 * Creation date: (12/6/00 12:27:39 PM)
 */

public synchronized void selectStates(Component comp, String strAgN) {

	StateTable st = comp.getStateTable();
	Vector states = st.getStates();
	
	// Go through all of the states and determine which
	// ones are candidates for receive a moveReq message.
	candidateStates.clear();					

	for(int k = 0; k < states.size(); k++) {
			
		State s = (State)states.elementAt(k);
		String stateName = new String(s.getName());
		
		if((!stateName.equals("restore"))
			&& (!stateName.equals("EndState"))
			&& (!stateName.equals("StartState"))
			&& (!stateName.equals("moveCalled"))
			&& (!stateName.equals("moveReceived"))
			&& (!stateName.startsWith("Null"))) {
				
			candidateStates.add(s);
			
		}
	}
	
	String msg = "Every component belonging to a Mobile Agent\n";
	msg += "must be able to receive moveReq messages.\n";
	msg += "Select the appropriate states for component:\n\n";
	msg += comp.getName()+"\n\n";
	msg += "that is part of agent:\n\n";
	msg += strAgN;
	Object[] options = {"OK"};
	JOptionPane.showOptionDialog(null, msg, "", 
		JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE,
		null, options, options[0]);

	// prompt the user to pick states
	CMStateWindow cmSW = new CMStateWindow(this);					
	cmSW.show();
	while(!cmSW.isDone()){
		try{
			sleep(100);
		}catch(Exception e){}
	}
}
}
