package Transforms;

/**
 * This class represents the Component Mobility Transformation
 * (CMT8) in the Mobile Design transformation process.
 * This transform adds a null transition from the state containing
 * a move activity to the moveCalled state that was created in CMT4.
 *
 * Creation date: (12/28/00 12:07:00 AM)
 * @author: Athie Self
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.*;

public class CMTransform8 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * CMTransform8 constructor comment.
 */

public CMTransform8(TSystem t) {
	super();

	sys = t.copy();
	currentSys = t;
	beforeSys = t.copy();
	setRationale("");
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method will only return true if the target is a TSystem 
 * and that system has roles and tasks.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)){
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * the params attribute is not used
 */

public void execute(Object params) {	
			
	Vector agents = sys.agents;
	
	for(int i = 0; i < agents.size(); i++) {
	
		Agent ag = (Agent)agents.elementAt(i);

		if(isMobility_Specified(ag)) {
			
			ATagent atAgent = ag.getAtAgent();
			ATarchitecture atArch = atAgent.getArchitecture();

			Vector components = ag.getComponents();

			for(int j = 0; j < components.size(); j++) {

				Component c = (Component)components.elementAt(j);
				ATagentComponent atComponent = atArch.findComponentbyname(c.getName());

				if(!(c.getName().equals("AgentComponent"))) {

					if(isMobility_Specified_Component(c)) {

						StateTable st = (StateTable)c.getStateTable();

						Vector states = st.getStates();

						for (int k = 0; k < states.size(); k++) {

							State s = (State)states.elementAt(k);

							if (isMobility_Specified_State(s)) {
								
								ATstatetable atStateTable = atComponent.getStateTable();
								ATtableElement atMoveCalledState = atStateTable.findStatebyname("moveCalled");
								ATtableElement atMoveActionState = (ATtableElement)s.getElement();
								
								// add transition to the state table
								ATtransition atTransition = new ATtransition(sys.version,sys.atSys.getNextKey());
								atTransition.setCoordinates(atMoveActionState,atMoveCalledState);
								atStateTable.addTransition(atTransition);
								
								atComponent.setStateTable(atStateTable);									
							}	
						}
					}
				}
			}
		}
	}
		
	// now make the altered system the current system
	// can be undone by the unDo method
	currentSys = sys;
	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param a Transforms.Agent
 */

public boolean isMobility_Specified(Agent a) {

	Vector actions    = new Vector();
	Vector states     = new Vector();
	
	Vector components = a.getComponents();
	
	if(components != null) {

		for(int j = 0; j < components.size(); j++) {

			Component c = (Component)components.elementAt(j);

			if (!c.getName().equals("AgentComponent")) {

				StateTable st = c.getStateTable();

				if(st != null) {
					states = st.getStates();
				}

				if(states != null) {					

					for(int k = 0; k < states.size(); k++) {

						State s = (State)states.elementAt(k);

						actions = s.getActions();

						if(actions != null) {

							for(int l = 0; l < actions.size(); l++) {

								Action act = (Action)actions.elementAt(l);

								FunctionCall fc = (FunctionCall)act.getFunction();

								if(fc!=null) {

									if(fc.getName().equals("move")) {
										return true;
									}
								}	
							}		
						}	
					}
				}
			}
		}
	}
	
	return false;	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param c Transforms.Component
 */

public boolean isMobility_Specified_Component(Component c) {

	Vector			actions = new Vector();
	Vector			states = new Vector();
	StateTable		st;
	Action			act;
	State			s = null;
	FunctionCall	fc;

	st = c.getStateTable();
	if(st != null) {
		states = st.getStates();
	}
	if(states != null) {					
		for(int k = 0; k < states.size(); k++) {
			s = (State)states.elementAt(k);
			actions = s.getActions();
			if(actions != null) {
				for(int l = 0; l < actions.size(); l++) {
					act = (Action)actions.elementAt(l);
					fc = act.getFunction();
					if(fc!=null) {
						if(fc.getName().equals("move")) {
							return true;
						}
					}	
				}		
			}	
		}
	}
	
	return false;	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param s Transforms.State
 */

public boolean isMobility_Specified_State (State s) {

	Vector vecActions = s.getActions();

	if(vecActions != null) {
	
		for(int i = 0; i < vecActions.size(); i++) {
		
			Action act = (Action)vecActions.elementAt(i);

			FunctionCall fc = (FunctionCall)act.getFunction();
			
			if(fc != null) {
			
				if(fc.getName().equals("move")) {
				
					return true;
					
				}
			}	
		}		
	}	
	
	return false;	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
