package Transforms;

/**
 * This class is used to represent externally sent events in task and component diagrams
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
 
public class SendEvent {
	private Event		event = null;			// event being sent
	private String		recipient = null;		// string representing the recipient of the event
	private Vector		protocols = null;		// Set of protocol that it belongs to
	private String		convName = null;
	private Vector		conversations = null;	// set of conversations the send event belongs to
	//private Vector		convMatches = null;		// set of ReceiveEvents that match in other components
	private Vector		eventMatches = null;
	private Vector		transMatches = null;
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 11:29:22 AM)
 */
public SendEvent(Event e, String r) {
	event = e;
	recipient = r;
}
/**
 * This method creates a new SendEvent that is logically equivalent to this
 *
 * Creation date: (11/16/00 9:24:31 AM)
 */
public SendEvent copy() {
	SendEvent se = new SendEvent(event.copy(), String.valueOf(recipient));
	if(protocols != null)
		se.setProtocols((Vector)protocols.clone());
	if(conversations != null)
		se.setConversations((Vector)conversations.clone());
	return se;
}
/**
 * This method determines if the two SendEvents are logically equivalent
 *
 * Creation date: (11/10/00 9:42:48 AM)
 */
public boolean equals(SendEvent se) {
	if(!event.equals(se.event))
		return false;
	if(!recipient.equals(se.recipient))
		return false;
	return true;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @return java.util.Vector
 */
public java.util.Vector getConversations() {
	return conversations;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @return java.lang.String
 */
public java.lang.String getConvName() {
	return convName;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @return Transforms.Event
 */
public Event getEvent() {
	return event;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:41:39 AM)
 * @return java.util.Vector
 */
public java.util.Vector getEventMatches() {
	return eventMatches;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @return java.util.Vector
 */
public java.util.Vector getProtocols() {
	return protocols;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @return java.lang.String
 */
public java.lang.String getRecipient() {
	return recipient;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:41:39 AM)
 * @return java.util.Vector
 */
public java.util.Vector getTransMatches() {
	return transMatches;
}
/**
 * This method is used to determine if there is a conversation with the
 * given name in the transition's conversations set
 *
 * Creation date: (11/29/00 1:30:54 PM)
 */
public boolean hasConvName(String cname) {
	if(conversations != null)
	for(int i = 0; i < conversations.size(); i++){
		Conversation c = (Conversation)conversations.elementAt(i);
		String cname2 = c.getName();
		if(cname2.equals(cname))
		return true;
	}

	// default
	return false;
}
/**
 * This method determines if the two SendEvents have exactly the same protocol set
 *
 * Creation date: (11/30/00 12:58:23 PM)
 */
public boolean sameProtocols(SendEvent se) {
	Vector protocols2 = se.getProtocols();
	
	if((protocols != null) && (protocols2 != null)){
		// they have to be the same size
		if(protocols.size() != protocols2.size())
			return false;
		// se has to also have all of the protocols
		for(int j = 0; j < protocols.size(); j++){
			Protocol p = (Protocol)protocols.elementAt(j);
			if(!protocols2.contains(p))
				return false;
		}
	}
	// if only one is null then they don't match
	else if((protocols == null) || (protocols2 != null))
		return false;
	else if((protocols != null) || (protocols2 == null))
		return false;
		
	return true;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @param newConversations java.util.Vector
 */
public void setConversations(java.util.Vector newConversations) {
	conversations = newConversations;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @param newConvName java.lang.String
 */
public void setConvName(java.lang.String newConvName) {
	convName = newConvName;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @param newEvent Transforms.Event
 */
public void setEvent(Event newEvent) {
	event = newEvent;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:41:39 AM)
 * @param newEventMatches java.util.Vector
 */
public void setEventMatches(java.util.Vector newEventMatches) {
	eventMatches = newEventMatches;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @param newProtocols java.util.Vector
 */
public void setProtocols(java.util.Vector newProtocols) {
	protocols = newProtocols;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:24:32 AM)
 * @param newRecipient java.lang.String
 */
public void setRecipient(java.lang.String newRecipient) {
	recipient = newRecipient;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:41:39 AM)
 * @param newTransMatches java.util.Vector
 */
public void setTransMatches(java.util.Vector newTransMatches) {
	transMatches = newTransMatches;
}
/**
 * Show method used for debugging
 *
 * Creation date: (11/6/00 1:38:16 PM)
 */
public void show() {
	System.out.print("send(");
	event.show();
	System.out.print(", " + recipient + ")");
	if(protocols != null){
		System.out.print("\t(Protocols: ");
		for(int i = 0; i < protocols.size(); i++){
			Protocol p = (Protocol)protocols.elementAt(i);
			System.out.print(p.getName());
			if(p.getInitComp() != null){
				System.out.print("[" + p.getInitComp().getName() + ", " + p.getRespComp().getName() + "]");
			}
			if(i+1 < protocols.size())
				System.out.print(" , ");
		}
		System.out.print(")");
	}
	if(conversations != null){
		System.out.print("\t(Conversations: ");
		for(int i = 0; i < conversations.size(); i++){
			Conversation conv = (Conversation)conversations.elementAt(i);
			String cname = conv.getName();
			System.out.print(cname);
			if(i+1 < conversations.size())
				System.out.print(" , ");
		}
		System.out.print(")");
	}
	
	System.out.println();
}
/**
 * Returns a string representation of the SendEvent
 *
 * Creation date: (11/6/00 1:38:16 PM)
 */
public String toString() {
	return ("send(" + event.toString() + ", " + recipient + ")");
}
}
