package Transforms;

/**
 * This class represents the first phase of the transformation system.
 * During this phase protocols will be determined for external events
 * and agent components are created for the roles that the agents play.
 *
 * Creation date: (11/10/00 11:16:23 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import system.ATTask;
import system.ATtransition;
import java.util.Vector;
import javax.swing.JOptionPane;

public class Stage1 extends Thread {
	TSystem sys = null;
	boolean debug = false;
/**
 * Phase1 constructor comment.
 */
public Stage1(TSystem s) {
	sys = s;
	sys.phaseOneStarted = true;
	start();
}
/**
 * This method builds and returns a Vector of protocols that may
 * or may not belong to the ReceiveEvent re
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
public Vector ambiguous(Task t, Transition trans, ReceiveEvent re){
	Vector protocols = sys.protocols;
	Vector reProtocols = re.getProtocols();

	Protocol p;
	Vector transitions2;
	Task t2;
	StateTable st2;
	Transition trans2;
	Vector sendEvents2;
	SendEvent se2;

	Vector ambigProts = null;
	boolean ambig = false;

	if(protocols != null)
	for(int i = 0; i < protocols.size(); i++){
		p = (Protocol)protocols.elementAt(i);
		if( (reProtocols == null) || ((reProtocols != null) && !reProtocols.contains(p)) ){

			t2 = null;
			if(p.getInitiator().equals(t))
			t2 = p.getResponder();
			else  if(p.getResponder().equals(t))
			t2 = p.getInitiator();

			if(t2 != null){			
				st2 = t2.getStateTable();
				transitions2 = st2.getTransitions();
				if(transitions2 != null)
				for(int j = 0; j < transitions2.size(); j++){
					trans2 = (Transition)transitions2.elementAt(j);
					sendEvents2 = trans2.getSendEvents();
					if(sendEvents2 != null)
					for(int k = 0; k < sendEvents2.size(); k++){
						se2 = (SendEvent)sendEvents2.elementAt(k);
						if(se2.getEvent().corresponds(re.getEvent())){
							ambig = true;
							if(ambigProts == null)
								ambigProts = new Vector();
							if(!ambigProts.contains(p))
								ambigProts.add(p);
						}
					}
				}
			}

		}
	}
		
	return ambigProts;
}
/**
 * This method builds and returns a Vector of protocols that may
 * or may not belong to the SendEvent se
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
public Vector ambiguous(Task t, Transition trans, SendEvent se){
	Vector protocols = sys.protocols;
	Vector seProtocols = se.getProtocols();

	Protocol p;
	Vector transitions2;
	Task t2;
	StateTable st2;
	Transition trans2;
	ReceiveEvent re2;

	Vector ambigProts = null;
	boolean ambig = false;

	if(protocols != null)
	for(int i = 0; i < protocols.size(); i++){
		p = (Protocol)protocols.elementAt(i);
		if( (seProtocols == null) || ((seProtocols != null) && !seProtocols.contains(p)) ){

			t2 = null;
			if(p.getInitiator().equals(t))
			t2 = p.getResponder();
			else  if(p.getResponder().equals(t))
			t2 = p.getInitiator();

			if(t2 != null){			
				st2 = t2.getStateTable();
				transitions2 = st2.getTransitions();
				if(transitions2 != null)
				for(int j = 0; j < transitions2.size(); j++){
					trans2 = (Transition)transitions2.elementAt(j);
					re2 = trans2.getReceiveEvent();
					if((re2 != null) && (re2.getEvent().corresponds(se.getEvent()))){
						ambig = true;
						if(ambigProts == null)
						ambigProts = new Vector();
						if(!ambigProts.contains(p))
						ambigProts.add(p);
					}
				}
			}

		}
	}
		
	return ambigProts;
}
/**
 * This method will ensure that there are no events that belong to both
 * internal and external protocols
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
void checkProtocolCorrectness(){
	Vector 			agents = sys.agents;
	Agent 			a;
	Vector 			components;
	Component 		c;
	StateTable 		st;
	Vector 			transitions;
	Transition 		t;
	Vector 			sendEvents;
	SendEvent 		se;
	ReceiveEvent 	re;
	Vector 			protocols;
	Protocol 		p;

	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				re = t.getReceiveEvent();
				if(re != null){
					protocols = re.getProtocols();
					if((protocols != null) && inconsistent(protocols))
						repair(re, a, c, st, t);					
				}
				sendEvents = t.getSendEvents();
				if(sendEvents != null)
				for(int l = 0; l < sendEvents.size(); l++){
					se = (SendEvent)sendEvents.elementAt(l);
					protocols = se.getProtocols();
					if((protocols != null) && inconsistent(protocols))
						repair(se, a, c, st, t);
				}
			}
		}
	}
}
/**
 * This method searches for protocols that the user may decide are internal
 * and then calls getProtocolDecision() for that decision
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
void determineProtocols(){
	Vector protocols = sys.protocols;
	Protocol p;
	Vector agents = sys.agents;
	Agent a;
	Vector components;
	Component c;
	boolean informed = false;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < protocols.size(); j++){
			p = (Protocol)protocols.elementAt(j);
			if((p.getMode() == Protocol.external)
				&& (p.getInitComp() != null)
				&& (p.getRespComp() != null)
				&& (components.contains(p.getInitComp()))
				&& (components.contains(p.getRespComp()))
				&& !p.isDecided()){
					// ask user if the protocol should remain external
					getProtocolDecision(a, p);
					p.setDecided(true);
				}
		}
	}
		
}
/**
 * This method searches for transitions that have events with ambiguity as to
 * which protocols they belong to.  For every ambiguous event, the user is asked
 * to decide what protocols it belongs to.
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
void eventsToProtocols(){
	Vector roles = sys.roles;
	Vector tasks = null;
	Vector protocols = sys.protocols;
	Vector transitions = null;
	Role r = null;
	Task t = null;
	StateTable st = null;
	Transition trans = null;
	Protocol p = null;
	Vector sendEvents = null;
	SendEvent se = null;
	ReceiveEvent re = null;
	Vector ambigProts = null;
	ProtocolWindow pw = null;
	boolean informed = false;

	if(roles != null)
	for(int i = 0; i < roles.size(); i++){
		r = (Role)roles.elementAt(i);
		tasks = r.getTasks();
		if(tasks != null)
		for(int j = 0; j < tasks.size(); j++){
			t = (Task)tasks.elementAt(j);
			st = t.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				trans = (Transition)transitions.elementAt(k);
				re = trans.getReceiveEvent();
				if(re != null){
					// check to see if there are any ambiguous protocols
					// for the ReceiveEvent
					ambigProts = ambiguous(t, trans, re);
					if(ambigProts != null){
						if((ambigProts.size() == 1) 
							&& ((re.getProtocols() == null) || (re.getProtocols().size() == 0)))
							re.setProtocols((Vector)ambigProts.clone());
						else{
							if(!informed){
								informed = true;
								String msg = "It was impossible to determine for some\n";
								msg += "external events which protocols they belong to.\n\n";
								msg += "For each of these events you will be asked to\n";
								msg += "determine the appropriate set of protocols."; 
								Object[] options = {"OK"};
								JOptionPane.showOptionDialog(null, msg, "", 
									JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE,
									null, options, options[0]);
							}
							// prompt the user to determine the correct protocols
							// get the possible protocols (those with matching events)
							// show the transition
							// display the dialog
							ATTask atTask = t.getAtTask();
							ATtransition atTransition = trans.getAtTransition();
							sys.editor.showTaskTransition(atTask, atTransition);
							pw = new ProtocolWindow(ambigProts, re);
							pw.show();
							while(!pw.isDone()){
								try{
									sleep(10);
								}catch(Exception e){}
							}
						}
					}
				}
				sendEvents = trans.getSendEvents();
				if(sendEvents != null)
				for(int l = 0; l < sendEvents.size(); l++){
					se = (SendEvent)sendEvents.elementAt(l);
					// check to see if there are any ambiguous protocols
					// for the SendEvent
					ambigProts = ambiguous(t, trans, se);
					if(ambigProts != null){
						if((ambigProts.size() == 1) 
							&& ((se.getProtocols() == null) || (se.getProtocols().size() == 0)))
							se.setProtocols((Vector)ambigProts.clone());
						else{
							if(!informed){
								informed = true;
								String msg = "It was impossible to determine for some\n";
								msg += "external events which protocols they belong to.\n\n";
								msg += "For each of these events you will be asked to\n";
								msg += "determine the appropriate set of protocols."; 
								Object[] options = {"OK"};
								JOptionPane.showOptionDialog(null, msg, "", 
									JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE,
									null, options, options[0]);
							}
							// prompt the user to determine the correct protocols
							// get the possible protocols (those with matching events)
							// show the transition
							// display the dialog
							ATTask atTask = t.getAtTask();
							ATtransition atTransition = trans.getAtTransition();
							sys.editor.showTaskTransition(atTask, atTransition);
							pw = new ProtocolWindow(ambigProts, se);
							pw.show();
							while(!pw.isDone()){
								try{
									sleep(10);
								}catch(Exception e){}
							}
						}
					}
				}
			}
		}
	}
}
/**
 * This method asks the user to decide if a protocol between only components
 * of a single agent are internal or external.
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
void getProtocolDecision(Agent a, Protocol p){
	Role r, initRole = null, respRole = null;
	Task t;
	Vector tasks;
	Vector roles = sys.roles;
	Task init = p.getInitiator();
	Task resp = p.getResponder();

	for(int i = 0; i < roles.size(); i++){
		r = (Role)roles.elementAt(i);
		tasks = r.getTasks();
		if(tasks != null)
		for(int j = 0; j < tasks.size(); j++){
			t = (Task)tasks.elementAt(j);
			if(t.equals(init))
			initRole = r;
			if(t.equals(resp))
			respRole = r;
		}
	}

	if(initRole != respRole){
		String title = "Choose Protocol Mode";
		String msg = "Agent: " + a.getName() + " is playing both\n";
		msg += "Role: " + initRole.getName() + " and Role: " + respRole.getName() + ".\n";
		msg += "Choose the mode of Protocol: " + p.getName() + " in the design.\n\n";
		msg += "If the protocol is internal, all events that belong to\n";
		msg += "the protocol will become internal events.  \n";
		msg += "Otherwise, they will remain the same.";
		Object[] options = {"Internal", "External"};
		int choice;
		choice = JOptionPane.showOptionDialog(null, msg, title, 
			JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE, 
			null, options, options[1]);

		if(choice == 0)
		p.setMode(Protocol.internal);
		
	}
		
		
}
/**
 * This method determines if there is both an internal and external
 * protocol in the given vector
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
boolean inconsistent(Vector protocols){
	boolean external = false;
	boolean internal = false;

	if(protocols == null)
	return true;

	for(int i = 0; i < protocols.size(); i++){
		Protocol p = (Protocol)protocols.elementAt(i);
		if(p.getMode() == Protocol.internal)
		internal = true;
		if(p.getMode() == Protocol.external)
		external = true;
	}

	if((external == true) && (internal == true))
		return true;

	else
		return false;
}
/**
 * This method determines if there are any errors in the way protocols have been
 * defined
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
boolean protocolErrors(){
Vector 			agents = sys.agents;
	Agent 			a;
	Vector 			components;
	Component 		c;
	StateTable 		st;
	Vector 			transitions;
	Transition 		t;
	Vector 			sendEvents;
	SendEvent 		se;
	ReceiveEvent 	re;
	Vector 			protocols;
	Protocol 		p;

	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				re = t.getReceiveEvent();
				if(re != null){
					protocols = re.getProtocols();
					if((protocols != null) && inconsistent(protocols))
						return true;					
				}
				sendEvents = t.getSendEvents();
				if(sendEvents != null)
				for(int l = 0; l < sendEvents.size(); l++){
					se = (SendEvent)sendEvents.elementAt(l);
					protocols = se.getProtocols();
					if((protocols != null) && inconsistent(protocols))
						return true;
				}
			}
		}
	}

	return false;
}
/**
 * This method displays a message to the user that they have an event that
 * belongs to both internal and external protocols.  It then sets all of the
 * protocols to external.  The user will be asked again to determine the mode
 * of those protocols
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
void repair(ReceiveEvent re, Agent a, Component c, StateTable st, Transition t){

	// need to first highlight the transition in the
	// component state diagram

	sys.editor.showComponentTransition(a.getAtAgent(), c.getAtComponent(), st.getAtStateTable(), t.getAtTransition());
	
	// display a dialog detailing the error
	String msg = "The highlighted ReceiveEvent belongs to both\n";
	msg += "External as well as Internal Protocols.\n";
	msg += "This is an error, and you will be asked\n";
	msg += "to redefine the mode of these protocols.";
	Object[] options = {"OK"};
	JOptionPane.showOptionDialog(null, msg, "Error", 
		JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
		null, options, options[0]);

	// set all of re's protocols to external
	Vector protocols = re.getProtocols();
	for(int i = 0; i < protocols.size(); i++){
		Protocol p = (Protocol)protocols.elementAt(i);
		p.setMode(Protocol.external);
		p.setDecided(false);
	}

}
/**
 * This method displays a message to the user that they have an event that
 * belongs to both internal and external protocols.  It then sets all of the
 * protocols to external.  The user will be asked again to determine the mode
 * of those protocols
 *
 * Creation date: (11/24/00 9:35:15 AM)
 */
void repair(SendEvent se, Agent a, Component c, StateTable st, Transition t){

	// need to first highlight the transition in the
	// component state diagram
	
	sys.editor.showComponentTransition(a.getAtAgent(), c.getAtComponent(), st.getAtStateTable(), t.getAtTransition());
	
	// display a dialog detailing the error
	String msg = "The highlighted SendEvent belongs to both\n";
	msg += "External as well as Internal Protocols.\n";
	msg += "This is an error, and you will be asked\n";
	msg += "to redefine the mode of these protocols.";
	Object[] options = {"OK"};
	JOptionPane.showOptionDialog(null, msg, "Error", 
		JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
		null, options, options[0]);

	// set all of re's protocols to external
	Vector protocols = se.getProtocols();
	for(int i = 0; i < protocols.size(); i++){
		Protocol p = (Protocol)protocols.elementAt(i);
		p.setMode(Protocol.external);
		p.setDecided(false);
	}

}
/**
 * This method contains the flow of events throughout the phase
 *
 * Creation date: (12/6/00 12:27:39 PM)
 */
public synchronized void run() {
	// label the protocols that external events belong to
	// (t1 - t3)
	Transform1 t1 = new Transform1(sys);
	Transform2 t2 = new Transform2(sys);
	Transform3 t3 = new Transform3(sys);

	// prompt the user to assign protocols
	// to ambiguous external events
	eventsToProtocols();
	
	// create the agent components (t4)
	Transform4 t4 = new Transform4(sys);

	// create the protocols in the design phase (t5)
	// and match up the external events in the components
	// to the copied protocols (t6 & t7)
	Transform5 t5 = new Transform5(sys);
	Transform6 t6 = new Transform6(sys);
	Transform7 t7 = new Transform7(sys);
	
	// prompt the user to determine if protocols
	// between combined roles are internal / external
	determineProtocols();

	// check to make sure that no external event belongs to
	// both an internal and external event
	while(protocolErrors()){
		checkProtocolCorrectness();
		determineProtocols();
	}
	
	
	// change external events into internal events
	// if their protocol has been changed to internal
	Transform8 t8 = new Transform8(sys);
	
	if(debug){
		System.out.println();
		System.out.println("After phase 1: ");
		sys.show();
	}
	
	sys.phaseOneDone = true;
}
}
