package Transforms;

/**
 * This class represents the second phase of the transformation process.
 * During this phase component state diagrams are annotated as to where
 * conversations begin and end.  In this phase how the different halves
 * of the conversations match up are determined.  Components are also
 * prepared for the removal of the conversations.
 *
 * Creation date: (11/10/00 11:16:23 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import system.ATconversation;
import java.util.Vector;
import javax.swing.JOptionPane;

public class Stage2 extends Thread{
	TSystem sys = null;
	ConversationWindow cw = null;
	boolean debug = false;
	Vector asked;
/**
 * Phase1 constructor comment.
 */
public Stage2(TSystem s) {
	sys = s;
	sys.phaseTwoStarted = true;
	start();
	
	
}
/**
 * Insert the method's description here.
 * Creation date: (1/19/01 8:55:20 AM)
 */
boolean alreadyAsked(SendEvent se, ReceiveEvent re) {
	if(asked != null)
	for(int i = 0; i < asked.size(); i++){
		SendReceivePair srp = (SendReceivePair)asked.elementAt(i);
		SendEvent s = srp.se;
		ReceiveEvent r = srp.re;
		if((re == r) && (se == s))
			return true;
	}
		
	return false;
}
/**
 * This method will determin for the given transition and ReceiveEvent
 * all transitions that might be a match as the beginning of a conversation
 *
 * Creation date: (12/1/00 7:47:58 AM)
 */
public Vector ambiguous(Component c, Transition trans, ReceiveEvent re){
	Vector protocols = sys.protocols;
	Vector reProtocols = re.getProtocols();

	Protocol p;
	Vector transitions2;
	Component c2;
	StateTable st2;
	Transition trans2;
	Vector sendEvents2;
	SendEvent se2;

	Vector ambigTransitions = null;
	boolean ambig = false;

	if(protocols != null)
	for(int i = 0; i < protocols.size(); i++){
		p = (Protocol)protocols.elementAt(i);
		if((reProtocols != null) 
			&& reProtocols.contains(p)){

			c2 = null;
			if(p.getInitComp() == c)
				c2 = p.getRespComp();
			else  if(p.getRespComp() == c)
				c2 = p.getInitComp();

			if(c2 != null){			
				st2 = c2.getStateTable();
				transitions2 = st2.getTransitions();
				if(transitions2 != null)
				for(int j = 0; j < transitions2.size(); j++){
					trans2 = (Transition)transitions2.elementAt(j);
					sendEvents2 = trans2.getSendEvents();
					if(sendEvents2 != null)
					for(int k = 0; k < sendEvents2.size(); k++){
						se2 = (SendEvent)sendEvents2.elementAt(k);
						if(((trans.getTransMatches() == null)
							|| !trans.getTransMatches().contains(trans2))
						&& (se2.getEvent().corresponds(re.getEvent()))){
							ambig = true;
							AgentCompTrans obj = new AgentCompTrans(c2.getParent(), c2, st2, trans2);
							obj.setSe(se2);
							if(ambigTransitions == null)
								ambigTransitions = new Vector();
							if(!ambigTransitions.contains(obj))
								ambigTransitions.add(obj);
						}
					}
				}
			}

		}
	}
		
	return ambigTransitions;
}
/**
 * This method will determin for the given transition and SendEvent
 * all transitions that might be a match as the beginning of a conversation
 *
 * Creation date: (12/1/00 7:47:58 AM)
 */
public Vector ambiguous(Component c, Transition trans, SendEvent se){
	Vector protocols = sys.protocols;
	Vector seProtocols = se.getProtocols();

	Protocol p;
	Vector transitions2;
	Component c2;
	StateTable st2;
	Transition trans2;
	ReceiveEvent re2;

	Vector ambigTransitions = null;
	boolean ambig = false;

	if(seProtocols != null)
	for(int i = 0; i < seProtocols.size(); i++){
		p = (Protocol)seProtocols.elementAt(i);
		
		c2 = null;
		if(p.getInitComp() == c)
			c2 = p.getRespComp();
		else  if(p.getRespComp() == c)
			c2 = p.getInitComp();

		if(c2 != null){			
			st2 = c2.getStateTable();
			transitions2 = st2.getTransitions();
			if(transitions2 != null)
			for(int j = 0; j < transitions2.size(); j++){
				trans2 = (Transition)transitions2.elementAt(j);
				re2 = trans2.getReceiveEvent();
				if(((trans.getTransMatches() == null) ||
					!trans.getTransMatches().contains(trans2))
					&& (re2 != null) 
					&& (re2.getEvent().corresponds(se.getEvent()))){
						ambig = true;
						AgentCompTrans obj = new AgentCompTrans(c2.getParent(), c2, st2, trans2);
						obj.setRe(re2);
						if(ambigTransitions == null)
							ambigTransitions = new Vector();
						if(!ambigTransitions.contains(obj))
							ambigTransitions.add(obj);
					}
			}
		}
	}
		
	return ambigTransitions;
}
/**
 * This method will check to see if the transition already has a start label.
 * If not, then it will receive the new start label.
 *
 * t is the transition already with the start label
 * t2 is the transition being checked
 *
 * Creation date: (12/1/00 7:47:58 AM)
 */
void checkStart(Transition t, Transition t2, SendEvent se) {
	if(t.getReceiveEvent() != null){
		// t2 has a sendEvent that matches up
		if(t2.getProtocols() != null){
			// only t2 needs to get the convName
			t2.setStart(true);
			t2.setConvNames(new Vector());
			t2.getConvNames().add((String)t.getConvNames().elementAt(0));
		}
		else{
			// se also needs to get the convName
			t2.setStart(true);
			if(t2.getConvNames() == null)
				t2.setConvNames(new Vector());
			t2.getConvNames().add((String)t.getConvNames().elementAt(0));
			se.setConvName((String)t.getConvNames().elementAt(0));
		}
	}
	else{
		// t2 has the receiveEvent.  Only t2 needs to get the convName of either se or t
		String convName;
		if(t.getConvNames().size() == 1)
			convName = (String)t.getConvNames().elementAt(0);
		else
			convName = se.getConvName();
		t2.setStart(true);
		t2.setConvNames(new Vector());
		t2.getConvNames().add(convName);
	}
}
/**
 * This method will create conversations between agents for each unique match
 * in the components.  The conversations are empty and will be filled in during
 * phase3.
 *
 * Creation date: (12/1/00 7:47:58 AM)
 */
void createConversations() {
	Vector agents = sys.agents;
	Vector conversations = sys.conversations;
	Vector components;
	Vector transitions;
	Agent a;
	Component c;
	StateTable st;
	Transition t;
	Conversation conv = null;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if(t.isStart() 
					&& (t.getReceiveEvent() != null) 
					&& (t.getConvNames() != null)
					&& t.isMatched()){
					AgentCompTrans obj = new AgentCompTrans(a, c, st, t);
					String cname = (String)t.getConvNames().elementAt(0);
					Vector matches = getMatches(obj, cname);
					for(int l = 0; l < matches.size(); l++){
						obj = (AgentCompTrans)matches.elementAt(l);
						Agent a2 = obj.getAgent();
						Component c2 = obj.getComponent();
						Transition t2 = obj.getTransition();
						String newConvName = sys.getNewConversationName(cname);
						
						// create the conversation
						conv = new Conversation(newConvName);
						ATconversation atConv = new ATconversation(sys.version, sys.atSys.getNextKey());
						atConv.setName(newConvName);
						conv.setAtConversation(atConv);
						sys.conversations.add(conv);
						sys.atSys.addConvs(atConv);
						
						// agent a is the responder
						conv.setResp(a);
						// agent a2 is the initiator
						conv.setInit(a2);

						// set the coordinates of the atConversation
						conv.getAtConversation().setCoordinates(conv.getInit().getAtAgent(), conv.getResp().getAtAgent());
												
						// add the conversation to the transitions
						if(t.getConversations() == null)
							t.setConversations(new Vector());
						if(t2.getConversations() == null)
							t2.setConversations(new Vector());
						t.getConversations().add(conv);
						t2.getConversations().add(conv);	
						if(t2.getProtocols() == null){
							// need to set the conversations for sendEvents w/ cname
							Vector sendEvents = t2.getSendEvents();
							for(int m = 0; m < sendEvents.size(); m++){
								SendEvent se = (SendEvent)sendEvents.elementAt(m);
								if((se.getConvName() != null) && se.getConvName().equals(cname)){
								// add the conversation to the sendEvent
								if(se.getConversations() == null)
									se.setConversations(new Vector());
								se.getConversations().add(conv);
								}
							}
						}
					}
				}
			}
		}
	}
	
}
/**
 * This method determines all transitions that match the transition
 * with the given convName in the given AgentCompTrans object
 *
 * Creation date: (12/1/00 7:47:58 AM)
 */
Vector getMatches(AgentCompTrans o, String cname) {
	Vector matches = new Vector();
	AgentCompTrans newObj = null;
	Vector agents = sys.agents;
	Vector components;
	Vector transitions;
	Agent a;
	Component c;
	StateTable st;
	Transition t;
	Transition t1 = o.getTransition();

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if((c != o.getComponent()) && (transitions != null))
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if((t1.getTransMatches() != null) 
					&& t1.getTransMatches().contains(t) 
					&& (t.getReceiveEvent() == null) 
					&& t.isStart() 
					&& t.hasConvName(cname) 
					&& t.isMatched()){
						newObj = new AgentCompTrans(a, c, st, t);
						matches.add(newObj);
				}
			}
		}
	}

	
	return matches;
}
/**
 * This method iterates through all of the transitions.  If a transition is
 * labeled the start of a conversation it is checked to see if it corresponds
 * to any ambiguous transitions.
 *
 * Creation date: (12/1/00 7:47:58 AM)
 */
void matchConversations() {
	Vector agents = sys.agents;
	Vector components;
	Vector transitions;
	Agent a;
	Component c;
	Transition t;

	asked = new Vector();
	
	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			StateTable st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				// run the garbage collector 
				System.gc();
				if(t.isStart())
					matchUpAmbiguous(t, st, c, a);
			}
		}
	}
}
/**
 * This method will take a transition that is labeled start.
 * Any transition where it is ambiguous if the events on the transitions
 * correspond are displayed to the user to determine.  If the user determines
 * that they match, the transitions are matched up and given the same convName.
 *
 * Creation date: (12/1/00 7:47:58 AM)
 */
void matchUpAmbiguous(Transition t, StateTable st, Component c, Agent a) {
	Vector ambigTransitions;
	AgentCompTrans obj;
	Agent a2;
	Component c2;
	StateTable st2;
	Transition t2;
	ReceiveEvent re;
	SendEvent se;
	SendReceivePair srp;
	
	re = t.getReceiveEvent();
	if(re != null){
		ambigTransitions = ambiguous(c, t, re);
		if(ambigTransitions != null)
		for(int j = 0; j < ambigTransitions.size(); j++){
			int value = 0;
			obj = (AgentCompTrans)ambigTransitions.elementAt(j);
			a2 = obj.getAgent();
			c2 = obj.getComponent();
			st2 = obj.getStateTable();
			t2 = obj.getTransition();
			se = obj.getSe();

			if(!alreadyAsked(se, re)){
				if(cw == null)
					cw = new ConversationWindow(sys.version);
				cw.loadWindow(a2, a, c2, c, t2, t, se, re, sys);
				value = cw.getValue();
				//cw.dispose();
				// run garbage collector
				System.gc();
				
				srp = new SendReceivePair(se, re);
				asked.add(srp);
				
				if(value == 1){
				
					// first set the trans to start if not already
					checkStart(t, t2, se);
					// now match up all of the conversation names
					String newConvName = sys.getConvName();
					if(se.getEventMatches() == null)
						se.setEventMatches(new Vector());
					if(re.getEventMatches() == null)
						re.setEventMatches(new Vector());
					if(se.getTransMatches() == null)
						se.setTransMatches(new Vector());
					if(re.getTransMatches() == null)
						re.setTransMatches(new Vector());
					se.getEventMatches().add(re);
					re.getEventMatches().add(se);
					re.getTransMatches().add(t2);
					se.getTransMatches().add(t);			

					// need to match up the names with the other
					String cname = se.getConvName();
					// if the send event does not have a convName,
					// then the transition should only have one convName
					// that can be used
					if(cname == null)
					cname = (String)t.getConvNames().elementAt(0);

					// the transition with the receive event will only have
					// one convName that needs to be replaced
					String cname2 = (String)t2.getConvNames().elementAt(0);
								
					// update everything to the new ConvName
					// this should happen EVERY time
					Vector updated = new Vector();
					updated.add(t);
					updated.add(t2);
					se.setConvName(newConvName);
					re.setConvName(newConvName);
					t.updateConvName(cname, newConvName);
					updateMatches(t, cname, newConvName, updated);
					t2.updateConvName(cname2, newConvName);
					updateMatches(t2, cname2, newConvName, updated);
					t.setMatched(true);
					t2.setMatched(true);
				}
			}
		}
	}
	else{
	Vector sendEvents = t.getSendEvents();
	if(sendEvents != null)
	for(int i = 0; i < sendEvents.size(); i++){
		se = (SendEvent)sendEvents.elementAt(i);
		ambigTransitions = ambiguous(c, t, se);
		if(ambigTransitions != null)
		for(int j = 0; j < ambigTransitions.size(); j++){
			int value = 0;
			obj = (AgentCompTrans)ambigTransitions.elementAt(j);
			a2 = obj.getAgent();
			c2 = obj.getComponent();
			st2 = obj.getStateTable();
			t2 = obj.getTransition();
			re = obj.getRe();
			if(!alreadyAsked(se, re)){
				if(cw == null)
					cw = new ConversationWindow(sys.version);
				cw.loadWindow(a, a2, c, c2, t, t2, se, re, sys);
				value = cw.getValue();
				// run garbage collector
				System.gc();
				
				srp = new SendReceivePair(se, re);
				asked.add(srp);
			
				if(value == 1){
				
					// first set the trans to start if not already
					checkStart(t, t2, se);

					// now match up all of the conversation names 
					String newConvName = sys.getConvName();
					if(se.getEventMatches() == null)
						se.setEventMatches(new Vector());
					if(re.getEventMatches() == null)
						re.setEventMatches(new Vector());
					if(se.getTransMatches() == null)
						se.setTransMatches(new Vector());
					if(re.getTransMatches() == null)
						re.setTransMatches(new Vector());
					se.getEventMatches().add(re);
					re.getEventMatches().add(se);
					re.getTransMatches().add(t);
					se.getTransMatches().add(t2);			

					// need to match up the names with the other
					String cname = se.getConvName();
					// if the send event does not have a convName,
					// then the transition should only have one convName
					// that can be used
					if(cname == null)
					cname = (String)t.getConvNames().elementAt(0);

					// the transition with the receive event will only have
					// one convName that needs to be replaced
					String cname2 = (String)t2.getConvNames().elementAt(0);
								
					// update everything to the new ConvName
					// this should happen EVERY time
					Vector updated = new Vector();
					updated.add(t);
					updated.add(t2);
					se.setConvName(newConvName);
					re.setConvName(newConvName);
					t.updateConvName(cname, newConvName);
					updateMatches(t, cname, newConvName, updated);
					t2.updateConvName(cname2, newConvName);
					updateMatches(t2, cname2, newConvName, updated);
					t.setMatched(true);
					t2.setMatched(true);
				}
			}
		}
	}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (1/19/01 8:55:20 AM)
 */
void newMethod() {
}
/**
 * Insert the method's description here.
 * Creation date: (1/7/01 3:02:45 PM)
 */
void removeExtraEnds() {
	Vector agents = sys.agents;
	Agent a;
	Vector components;
	Component c;
	StateTable st;
	Vector transitions;
	Transition t;
	
	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if(t.isEnd() && (t.getConversations() == null))
					t.setEnd(false);
			}
		}
	}
}
/**
 * This method contains the flow of events throughout the phase
 *
 * Creation date: (12/1/00 7:47:58 AM)
 */
public synchronized void run() {
	// wait for phase1 to complete
	while(!sys.phaseOneDone){
		try{
			sleep(100);
		}catch(Exception e){}
	}	

	// split up transitions with a receiveEvent and events
	// that do not correspond
	Transform10	t10 = new Transform10(sys);

	// split up transitions with both internal and external events
	Transform11	t11 = new Transform11(sys);

	// determine the protocols for transitions
	Transform12 t12 = new Transform12(sys);

	// determine which transitions are the start of a conversation
	Transform19	t19 = new Transform19(sys);

	// determine which transitions are the end of a conversation
	Transform25	t25 = new Transform25(sys);

	// ensure that state containing mobility remains in the component
	Transform28a t28a = new Transform28a(sys);		

	// try to automatically determine how some conversations match up
	Transform29 t29 = new Transform29(sys);
	
	// this is where the conversation halves are matched up
	// Conversations are created between the agents
	// Within each component, "duplicate" conversation halves MUST have
	// the same convName
	matchConversations();
	
	if(cw != null)
		cw.dispose();
	System.gc();
	
	// reapply Transform 24 to capture any new end labels
	t25 = new Transform25(sys);
	
	// split up any transitions with a receive event and send event to different
	// conversations
	Transform31 t31 = new Transform31(sys);

	// create the conversations for the convNames
	createConversations();
	
	// propagate the conversations from start transitions
	Transform32 t32 = new Transform32(sys);

	// remove any extra end labels
	removeExtraEnds();
	
	if(debug){
		System.out.println();
		System.out.println("After phase 2: ");
		sys.show();
	}

	sys.phaseTwoDone = true;
}
/**
 * This method updates conversation names of all matching transitions
 * with the new conversation name, removing the old one.  It is recursive
 * and must maintain and pass a vector of transitions this has already
 * been called on.
 * 
 * Creation date: (12/6/00 7:11:34 PM)
 */
void updateMatches(Transition t, String oldName, String newName, Vector updated) {
	Vector matches = t.getTransMatches();
	if(matches != null)
	for(int i = 0; i < matches.size(); i++){
		Transition t2 = (Transition)matches.elementAt(i);
		t2.updateConvName(oldName, newName);
		ReceiveEvent re = t2.getReceiveEvent();
		if((re != null) && (re.getConvName() != null) && re.getConvName().equals(oldName))
			re.setConvName(newName);
		Vector sendEvents = t2.getSendEvents();
		if(sendEvents != null)
		for(int j = 0; j < sendEvents.size(); j++){
			SendEvent se = (SendEvent)sendEvents.elementAt(j);
			String cname = se.getConvName();
			if((cname != null) && (cname.equals(oldName)))
				se.setConvName(newName);
		}

		if(!updated.contains(t2)){
			updated.add(t2);
			updateMatches(t2, oldName, newName, updated);
		}
	}
}
}
