package Transforms;

/**
 * This class represents transformation 10 in the transformation process.
 * The result of this transform is that any transitions with an external
 * receive event and external send events that don't belong to the same protocols
 * will be split up.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.ATtableElement;
import system.ATstate;
import system.ATstateGRFX;
import system.ATstartState;
import system.ATstartStateGRFX;
import system.ATendState;
import system.ATendStateGRFX;
import system.ATstatetable;
import system.ATtransition;

public class Transform10 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment. 
 */
public Transform10(TSystem t) {
	super();

	sys = t;
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method determines if all of the sent events correspond to the receive event
 *
 * Creation date: (11/11/00 12:32:42 PM)
 */
public boolean corresponds(ReceiveEvent re, Vector sends, Vector sendEvents) {
	Vector	reProtocols = re.getProtocols();
	
	if(reProtocols == null){
		System.out.println();
		System.out.println("Error in Transform 10");
		System.out.println("ReceiveEvent: " + re + " does not have any protocols");
		return true;
	}
	
	if(sends != null)
		return false;
	
	if(sendEvents != null)
	for(int i = 0; i < sendEvents.size(); i++){
		SendEvent se = (SendEvent)sendEvents.elementAt(i);
		// if there is a SendEvent to a list, then it needs to be split up
		// the sendEvent to a list goes on the second transition
		if(isList(se.getRecipient()))
			return false;
			
		Vector seProtocols = se.getProtocols();
		if(seProtocols == null){
			System.out.println();
			System.out.println("Error in Transform 10");
			System.out.println("SendEvent: " + se + " does not have any protocols");
			return true;
		}
		else{
			if(reProtocols.size() != seProtocols.size()){
				// if the size of the sets are not the same,
				// then the sets cannot be the same
				return false;
			}
			
			for(int j = 0; j < reProtocols.size(); j++){
				Protocol p = (Protocol)reProtocols.elementAt(j);
				if(!seProtocols.contains(p)){
					// p was in one, but not the other
					// therefore, the sets are not the same
					return false;
				}
			}
		}
	}
	
	// default - all events have the same protocols
	return true;
}
/**
 * This method will split up all transitions in component state tables
 * that have a receiveEvent and other send events that have different protocols
 *
 * the params attribute is not used
 */
public void execute(Object params) {
	Vector			agents = sys.agents;
	Vector			components = null;
	Vector			transitions = null;
	Agent			a;
	Component 		c;
	ReceiveEvent	re;
	StateTable		st;
	Transition		t;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			if(st != null)
				transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if((t.getReceiveEvent() != null) 
					&& !corresponds(t.getReceiveEvent(), t.getSends(), t.getSendEvents())){
						// split up transitions that have a receive event
						// and other send events that do not correspond
						splitUp(st, t);
						k--;
					}
			}
		}	
	}

}
/**
 * This method is used to determine if a recipient is to a list
 *
 * Creation date: (11/11/00 2:46:50 PM)
 */
public boolean isList(String s) {
	char first = s.charAt(0);
	char last = s.charAt(s.length()-1);
	if((first == '<') && (last == '>'))
		return true;
	else
		return false;
}
/**
 * This method will place the new null state visually between the original
 * from and to states
 *
 * Creation date: (11/22/00 7:13:47 PM)
 */
void placeBetween(State s, Transition t) {
	State from = t.getFrom();
	State to = t.getTo();
	ATtableElement fromElement = from.getElement();
	ATtableElement toElement = to.getElement();
	ATtableElement atFrom, atTo;
	int newX;
	int newY;
	int[] fromCoords = new int[4];
	int[] toCoords = new int[4];
	
	if(fromElement instanceof ATstate){
		fromCoords = ((ATstate)fromElement).getGrfx().getCoords();
	}
	if(fromElement instanceof ATstartState){
		fromCoords = ((ATstartState)fromElement).getGrfx().getCoords();
	}
	if(fromElement instanceof ATendState){
		fromCoords = ((ATendState)fromElement).getGrfx().getCoords();
	}

	if(toElement instanceof ATstate){
		toCoords = ((ATstate)toElement).getGrfx().getCoords();
	}
	if(toElement instanceof ATstartState){
		toCoords = ((ATstartState)toElement).getGrfx().getCoords();
	}
	if(toElement instanceof ATendState){
		toCoords = ((ATendState)toElement).getGrfx().getCoords();
	}

	// determine newX
	newX = (toCoords[0] + fromCoords[0]) / 2;
	
	// determind newY
	newY = (toCoords[1] + fromCoords[1]) / 2;
	
	ATstate newState = new ATstate(fromElement.getVersion(), sys.atSys.getNextKey());
	newState.setName(s.getName());
	newState.setCoordinates(newX, newY, 50, 50);
	s.setElement(newState);
	
}
/**
 * This method determines if the two vectors of protocols are the same
 *
 * Creation date: (11/21/00 1:36:20 PM)
 */
public boolean protocolsMatch(Vector seProtocols, Vector reProtocols) {

	if((seProtocols != null) && (reProtocols != null)){
		if(seProtocols.size() != reProtocols.size())
			return false;
		for(int i = 0; i < seProtocols.size(); i++){
			Protocol p = (Protocol)seProtocols.elementAt(i);
			if(!reProtocols.contains(p))
				return false;
		}
	}
	
	return true;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * This method will split up the transition in the given state table
 *
 * Creation date: (11/11/00 12:43:06 PM)
 */
public void splitUp(StateTable st, Transition t) {
	ATstatetable	atStateTable = st.getAtStateTable();
	ReceiveEvent 	re = t.getReceiveEvent();
	State 			s;
	Transition 		t1, t2;
	String			stateName;
	Vector			states = st.getStates();
	Vector			transitions = st.getTransitions();
	boolean			added = false;
	ATtableElement	s1, s2, s3;

	
	for(int n = 0; added == false; n++){
		stateName = "Null" + String.valueOf(n);
		if(!st.hasState(stateName)){
			added = true;
			// need to create the corresponding objects that
			// go in the AT system
			s = new State(stateName, sys);
			placeBetween(s, t);
			states.add(s);
			// add s's ATtableElement to the ATstatetable
			atStateTable.addState(s.getElement());
			
			t1 = new Transition(sys);
			t2 = new Transition(sys);
			
			t1.setReceiveEvent(t.getReceiveEvent());
			t1.setGuard(t.getGuard());
			t1.setActions(t.getActions());
			t1.setFrom(t.getFrom());
			t1.setTo(s);
			
			t2.setSends(t.getSends());
			t2.setFrom(s);
			t2.setTo(t.getTo());
			
			// determine the sendEvents that correspond
			// i.e. they have the same protocols (but are not lists)
			Vector reProtocols = re.getProtocols();
			Vector sendEvents = t.getSendEvents();
			if(sendEvents != null)
			for(int i = 0; i < sendEvents.size(); i++){
				SendEvent se = (SendEvent)sendEvents.elementAt(i);
				Vector seProtocols = se.getProtocols();
				// add corresponding send events to t1
				if(protocolsMatch(seProtocols, reProtocols) && !isList(se.getRecipient())){
					if(t1.getSendEvents() == null)
						t1.setSendEvents(new Vector());
					t1.getSendEvents().add(se);
				}
				// add non-corresponding send events to t2
				else{
					if(t2.getSendEvents() == null)
						t2.setSendEvents(new Vector());
					t2.getSendEvents().add(se);
				}
			}

			// create the ATtransitions for t1 and t2
			ATtransition atT1 = new ATtransition(sys.version, sys.atSys.getNextKey());
			ATtransition atT2 = new ATtransition(sys.version, sys.atSys.getNextKey());
			t1.setAtTransition(atT1);
			t2.setAtTransition(atT2);

			// set the ATtableElements used to set the coordinates
			// of the ATtransitions
			s1 = t1.getFrom().getElement();
			s2 = t1.getTo().getElement();
			s3 = t2.getTo().getElement();

			// fill in the attributes for atT1
			atT1.setRecMessage(t1.getRecMessage());
			atT1.setGuardCondition(t1.getGuard());
			atT1.setTransMessage(t1.getTransMessage());
			// don't currently add actions - shouldn't be any
			atT1.setCurrentState(s1);
			atT1.setNextState(s2);
			atT1.setCoordinates(s1, s2);
			
			// fill in the attributes for atT2
			atT2.setRecMessage(t2.getRecMessage());
			atT2.setGuardCondition(t2.getGuard());
			atT2.setTransMessage(t2.getTransMessage());
			// don't currently add actions - shouldn't be any
			atT2.setCurrentState(s2);
			atT2.setNextState(s3);
			atT2.setCoordinates(s2, s3);
			
			// remove t from the transitions vector
			transitions.remove(t);
			transitions.add(t1);
			transitions.add(t2);
			// remove t's atTransition from the ATstatetable
			// and add t1's and t2's
			atStateTable.removeTransition(t.getAtTransition());
			atStateTable.addTransition(t1.getAtTransition());
			atStateTable.addTransition(t2.getAtTransition());
		}
	}
}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
