package Transforms;

/**
 * This class represents transformation 2 in the transformation process
 * It will determine the protocols for external events in some situations.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;

public class Transform2 extends Transform {	
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform2(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method will only return true if the target is a TSystem and that system has
 * roles, tasks, and protocols defined.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will now check all external events and try
 * determine the protocol they belong to.
 *
 * The params attribute is not used.
 */
public void execute(Object params) {
	// get the protocols from the system
	Vector			protocols = sys.protocols;
	Task			t, t2;
	Protocol		p;

	if(debug){
		System.out.println();
		System.out.println("*** Transformation 2 ***");
	}
	
	for(int i = 0; i < protocols.size(); i++){
		p = (Protocol)protocols.elementAt(i);
		t = p.getInitiator();
		t2 = p.getResponder();
		findMatch(sys, p, t, t2);

		t = p.getResponder();
		t2 = p.getInitiator();
		findMatch(sys, p, t, t2);
	}
}
/**
 * This method will now check all external events and try
 * determine the protocol they belong to.
 *
 * The params attribute is not used.
 */
public void findMatch(TSystem sys, Protocol p, Task t, Task t2) {
	StateTable 		st = t.getStateTable();
	StateTable		st2 = t2.getStateTable();
	Transition		trans, trans2;
	SendEvent		se;
	ReceiveEvent	re;
	Vector transitions = st.getTransitions();
	Vector transitions2 = st2.getTransitions();

	
	// check all send events in the transitions vector
	// for task t's state table
	for(int j = 0; j < transitions.size(); j++){
		trans = (Transition)transitions.elementAt(j);
		
		// get the send events for this transition
		Vector sendEvents = trans.getSendEvents();
		
		// check each individual send event
		if(sendEvents != null)
		for(int k = 0; k < sendEvents.size(); k++){
			se = (SendEvent)sendEvents.elementAt(k);
			// check the send event against all receiveEvents
			// for transitions in task t2's state table
			for(int l = 0; l < transitions2.size(); l++){
				trans2 = (Transition)transitions2.elementAt(l);
				re = trans2.getReceiveEvent();
				if((re != null) 
					&& (re.getSender() != null) 
					&& re.getEvent().corresponds(se.getEvent())
					&& onlySend(st, se)
					&& isUnique(sys, p, t2, re)){
						if(se.getProtocols() == null)
							se.setProtocols(new Vector());
						if(!se.getProtocols().contains(p))
							se.getProtocols().add(p);
						if(re.getProtocols() == null)
							re.setProtocols(new Vector());
						if(!re.getProtocols().contains(p))
							re.getProtocols().add(p);
						if(debug){
							System.out.println();
							System.out.println("Assigning protocol " + p.getName() + " to:");
							System.out.println(se.toString() + " in Task: " + t.getName());
							System.out.println(re.toString() + " in Task: " + t2.getName());
						}
					}
			}
		}
	}
}
/**
 * This method will determine if there are no other protocols the receive event
 * could possibly belong to
 *
 * Creation date: (11/9/00 3:44:46 PM)
 */
public boolean isUnique(TSystem sys, Protocol p, Task t2, ReceiveEvent re) {
	Vector		protocols = sys.protocols;
	Protocol 	p2;
	Task		t3;
	
	for(int i = 0; i < protocols.size(); i++){
		p2 = (Protocol)protocols.elementAt(i);
		if((p2 != p) && ((p2.getInitiator() == t2) || (p2.getResponder() == t2))){
			if(p2.getInitiator() == t2)
				t3 = p2.getResponder();
			else
				t3 = p2.getInitiator();
			StateTable st3 = t3.getStateTable();
			Vector transitions = st3.getTransitions();
			if(transitions != null)
			for(int j = 0; j < transitions.size(); j++){
				Transition trans3 = (Transition)transitions.elementAt(j);
				Vector sendEvents = trans3.getSendEvents();
				if(sendEvents != null)
				for(int k = 0; k < sendEvents.size(); k++){
					SendEvent se = (SendEvent)sendEvents.elementAt(k);
					if(se.getEvent().corresponds(re.getEvent()))
						return false;
				}
			}	
		}
	}
	
	return true;
}
/**
 * This method will determine if the send event is the only send event in the
 * state table with the same send event
 *
 * Creation date: (11/15/00 10:51:28 AM)
 */
public boolean onlySend(StateTable st, SendEvent se) {
	Vector		transitions = st.getTransitions();
	Transition 	t;
	Vector		sendEvents;
	SendEvent	se2;
	
	for(int i = 0; i < transitions.size(); i++){
		t = (Transition)transitions.elementAt(i);
		sendEvents = t.getSendEvents();
		if(sendEvents != null)
		for(int j = 0; j < sendEvents.size(); j++){
			se2 = (SendEvent)sendEvents.elementAt(j);
			if((se2 != se) && (se2.getEvent().corresponds(se.getEvent())))
				return false;
		}	
	}
	return true;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * This method will set the system back to the 
 * system before the transformation took place.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
