package Transforms;

/**
 * This class represents transformations 24-27 in the transformation process.
 * The result of this transform is that all appropriate transitions will receive
 * the end label.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;

public class Transform25 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform25(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method determines if there is a transition leaving t's to state that is a
 * start transition
 *
 * Creation date: (11/14/00 10:04:14 AM)
 */
public boolean beforeStart(StateTable st, Transition t) {
	Vector transitions = st.getTransitions();
	Transition t2;

	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(t2.isStart() && (t2.getFrom() == t.getTo()))
			return true;
	}

	// default
	return false;
}
/**
 * This method will add end labels to transitions in component state tables
 *
 * the params attribute is not used 
 */
public void execute(Object params) {	
	Vector			agents = sys.agents;
	Vector			components = null;
	Vector			transitions = null;
	Agent			a;
	Component 		c;
	Transition		t;
	StateTable		st;
	boolean			allMatch;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			if(st != null)
				transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if(t.getProtocols() != null){
					State s = t.getTo();
					if(s.getName().equals("EndState") // (Transformation 27)
						|| foundNew(st, t) // (Transformation 24)
						|| beforeStart(st, t) // (Transformation 26) 
						|| nullIn(t, st)
						|| nullOut(t, st)){
							t.setEnd(true);
					}
				}
				else if((t.getProtocols() == null) && (t.getSendEvents() != null)){
						// (Transformation 25)
						// either there are multiple SendEvents to different
						// recipients or there is a SendEvent to a list
						t.setEnd(true);
					}
			}
		}	
	}
}
/**
 * This method determines if there is a transition leaving t's to state
 * that has a protocol not found in t's protocol set
 *
 * Creation date: (11/13/00 1:00:15 PM)
 */
public boolean foundNew(StateTable st, Transition t) {
	Vector transitions = st.getTransitions();
	Transition t2;
	Vector protocols = t.getProtocols();
	Protocol p;
	boolean found;

	if(protocols != null)
	for(int i = 0; i < protocols.size(); i++){
		p = (Protocol)protocols.elementAt(i);
		found = false;
		for(int j = 0; j < transitions.size(); j++){
			t2 = (Transition)transitions.elementAt(j);
			if((t2 != t) && (t.getTo() == t2.getFrom())){
				Vector prots = t2.getProtocols();
				if((prots != null) && prots.contains(p))
					found = true;
			}
		}
		if(!found)
			return true;
	}
	
	// default
	return false;
}
/**
 * This method will determine if there is a transition into the from state
 * of transition t with null protocols
 *
 * Creation date: (12/28/00 1:03:40 PM)
 */
public boolean nullIn(Transition t, StateTable st) {
	Vector	protocols = t.getProtocols();
	Vector	transitions = st.getTransitions();
	Transition t2;

	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(t2.getTo() == t.getTo()){
			Vector protocolsIn = t2.getProtocols();
			if(protocolsIn == null){
					return true;
			}
		}
	}
	
	return false;
}
/**
 * This method will determine if there is a transition into the from state
 * of transition t with null protocols
 *
 * Creation date: (12/28/00 1:03:40 PM)
 */
public boolean nullOut(Transition t, StateTable st) {
	Vector	protocols = t.getProtocols();
	Vector	transitions = st.getTransitions();
	Transition t2;

	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(t2.getFrom() == t.getTo()){
			Vector protocolsIn = t2.getProtocols();
			if(protocolsIn == null){
					return true;
			}
		}
	}
	
	return false;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
