package Transforms;

/**
 * This class represents transformation 31 in the transformation process.
 * The result of this transform is that any transitions with a ReceiveEvent
 * and more than one convName will be split up.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.ATtableElement;
import system.ATstate;
import system.ATstateGRFX;
import system.ATstartState;
import system.ATstartStateGRFX;
import system.ATendState;
import system.ATendStateGRFX;
import system.ATstatetable;
import system.ATtransition;

public class Transform31 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform31(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will split up all transitions in component state tables
 * that have a receiveEvent and other send events that have different protocols
 *
 * the params attribute is not used
 */
public void execute(Object params) {
	Vector			agents = sys.agents;
	Vector			components = null;
	Vector			transitions = null;
	Agent			a;
	Component 		c;
	ReceiveEvent	re;
	StateTable		st;
	Transition		t;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			if(st != null)
				transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if((t.isStart())
					&& (t.getReceiveEvent() != null) 
					&& (t.getConvNames() != null)
					&& (t.getConvNames().size() > 1)){
						// split up transitions that have a receive event
						// and more than one convName
						splitUp(st, t);
						k--;
					}
			}
		}	
	}
}
/**
 * This method will place the new null state visually between the original
 * from and to states
 *
 * Creation date: (11/22/00 7:13:47 PM)
 */
void placeBetween(State s, Transition t) {
	State from = t.getFrom();
	State to = t.getTo();
	ATtableElement fromElement = from.getElement();
	ATtableElement toElement = to.getElement();
	ATtableElement atFrom, atTo;
	int newX;
	int newY;
	int[] fromCoords = new int[4];
	int[] toCoords = new int[4];
	
	if(fromElement instanceof ATstate){
		fromCoords = ((ATstate)fromElement).getGrfx().getCoords();
	}
	if(fromElement instanceof ATstartState){
		fromCoords = ((ATstartState)fromElement).getGrfx().getCoords();
	}
	if(fromElement instanceof ATendState){
		fromCoords = ((ATendState)fromElement).getGrfx().getCoords();
	}

	if(toElement instanceof ATstate){
		toCoords = ((ATstate)toElement).getGrfx().getCoords();
	}
	if(toElement instanceof ATstartState){
		toCoords = ((ATstartState)toElement).getGrfx().getCoords();
	}
	if(toElement instanceof ATendState){
		toCoords = ((ATendState)toElement).getGrfx().getCoords();
	}

	// determine newX
	newX = (toCoords[0] + fromCoords[0]) / 2;
	
	// determind newY
	newY = (toCoords[1] + fromCoords[1]) / 2;
	
	ATstate newState = new ATstate(sys.version, sys.atSys.getNextKey());
	newState.setName(s.getName());
	newState.setCoordinates(newX, newY, 50, 50);
	s.setElement(newState);
	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * This method will split up the transition in the given state table
 *
 * Creation date: (11/11/00 12:43:06 PM)
 */
public void splitUp(StateTable st, Transition t) {
	ATstatetable	atStateTable = st.getAtStateTable();
	ReceiveEvent 	re = t.getReceiveEvent();
	SendEvent		se = (SendEvent)t.getSendEvents().elementAt(0);
	State 			s;
	Transition 		t1, t2;
	String			stateName;
	Vector			states = st.getStates();
	Vector			transitions = st.getTransitions();
	boolean			added = false;
	ATtableElement	s1, s2, s3;

	
	for(int n = 0; added == false; n++){
		stateName = "Null" + String.valueOf(n);
		if(!st.hasState(stateName)){
			added = true;
			// need to create the corresponding objects that
			// go in the AT system
			s = new State(stateName, sys);
			placeBetween(s, t);
			states.add(s);
			// add s's ATtableElement to the ATstatetable
			atStateTable.addState(s.getElement());
			
			t1 = new Transition(sys);
			t2 = new Transition(sys);
			
			t1.setReceiveEvent(t.getReceiveEvent());
			t1.setGuard(t.getGuard());
			t1.setActions(t.getActions());
			t1.setFrom(t.getFrom());
			t1.setTo(s);
			t1.setProtocols(t.getProtocols());
			Vector convNames = new Vector();
			convNames.add(re.getConvName());
			t1.setConvNames(convNames);
			t1.setMatched(true);
			updateMatches(t1.getReceiveEvent(), t, t1);

			t2.setSendEvents(t.getSendEvents());
			t2.setFrom(s);
			t2.setTo(t.getTo());
			t2.setProtocols(t.getProtocols());
			convNames = new Vector();
			convNames.add(se.getConvName());
			t2.setConvNames(convNames);
			t2.setMatched(true);
			updateMatches(se, t, t2);

			// create the ATtransitions for t1 and t2
			ATtransition atT1 = new ATtransition(sys.version, sys.atSys.getNextKey());
			ATtransition atT2 = new ATtransition(sys.version, sys.atSys.getNextKey());
			t1.setAtTransition(atT1);
			t2.setAtTransition(atT2);

			// set the starts and end
			t1.setStart(true);
			t1.setEnd(true);
			t2.setStart(true);
			if(t.isEnd())
				t2.setEnd(true);

			// set the ATtableElements used to set the coordinates
			// of the ATtransitions
			s1 = t1.getFrom().getElement();
			s2 = t1.getTo().getElement();
			s3 = t2.getTo().getElement();

			// fill in the attributes for atT1
			atT1.setRecMessage(t1.getRecMessage());
			atT1.setGuardCondition(t1.getGuard());
			atT1.setTransMessage(t1.getTransMessage());
			// don't currently add actions - shouldn't be any
			atT1.setCurrentState(s1);
			atT1.setNextState(s2);
			atT1.setCoordinates(s1, s2);
			
			// fill in the attributes for atT2
			atT2.setRecMessage(t2.getRecMessage());
			atT2.setGuardCondition(t2.getGuard());
			atT2.setTransMessage(t2.getTransMessage());
			// don't currently add actions - shouldn't be any
			atT2.setCurrentState(s2);
			atT2.setNextState(s3);
			atT2.setCoordinates(s2, s3);
			
			// remove t from the transitions vector
			transitions.remove(t);
			transitions.add(t1);
			transitions.add(t2);
			// remove t's atTransition from the ATstatetable
			// and add t1's and t2's
			atStateTable.removeTransition(t.getAtTransition());
			atStateTable.addTransition(t1.getAtTransition());
			atStateTable.addTransition(t2.getAtTransition());
		}
	}
}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
/**
 * This method will update all matches to se to point to the new transition and SendEvent
 *
 * Creation date: (11/22/00 7:13:47 PM)
 */
void updateMatches(ReceiveEvent re, Transition oldTrans, Transition newTrans) {
	Vector matches = re.getEventMatches();
	if(matches != null)
	for(int i = 0; i < matches.size(); i++){
		SendEvent se = (SendEvent)matches.elementAt(i);
		int index = se.getTransMatches().indexOf(oldTrans);
		if(index != -1)
			se.getTransMatches().setElementAt(newTrans, index);
	}	
}
/**
 * This method will update all matches to se to point to the new transition and SendEvent
 *
 * Creation date: (11/22/00 7:13:47 PM)
 */
void updateMatches(SendEvent se, Transition oldTrans, Transition newTrans) {
	Vector matches = se.getEventMatches();
	if(matches != null)
	for(int i = 0; i < matches.size(); i++){
		ReceiveEvent re = (ReceiveEvent)matches.elementAt(i);
		int index = re.getTransMatches().indexOf(oldTrans);
		if(index != -1)
			re.getTransMatches().setElementAt(newTrans, index);
	}	
}
}
