package Transforms;

/**
 * This class represents transformation 32 in the transformation process.
 * The result of this transform is that the conversations set will be propogated
 * from the start transitions to the states and transitions that belong to those
 * conversations.  The propogation will stop with the end transitions.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;

public class Transform32 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform32(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will find start transitions and call the method to propogate its
 * conversations set.
 *
 * the params attribute is not used
 */
public void execute(Object params) {	
	Vector 		agents = sys.agents;
	Agent		a;
	Vector		components;
	Component 	c;
	StateTable	st;
	Vector		transitions;
	Transition	t;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if(t.isStart() && !t.isEnd()){
					propagate(t, st);					
				}
			}
		}
	}
}
/**
 * This method will propogate the conversations set to all states and transitions
 * that belong to them.
 *
 * Creation date: (11/26/00 2:32:32 PM)
 */
void propagate(Transition t, StateTable st) {
	Vector	conversations = t.getConversations();
	Vector	transitions = st.getTransitions();
	State s = t.getTo();

	if(conversations != null){
		if(s.getConversations() == null)
			s.setConversations(new Vector());
		// add each conversation to the 'to' state
		for(int i = 0; i < conversations.size(); i++){
			Conversation conv = (Conversation)conversations.elementAt(i);
			if(!s.getConversations().contains(conv))
				s.getConversations().add(conv);
		}
		// add each conversation to every transition leaving the 'to' state
		for(int i = 0; i < transitions.size(); i++){
			Transition t2 = (Transition)transitions.elementAt(i);
			if(t2.getFrom() == t.getTo()){
				if(t2.getConversations() == null)
					t2.setConversations(new Vector());
				for(int j = 0; j < conversations.size(); j++){
					Conversation conv = (Conversation)conversations.elementAt(j);
					if(!t2.getConversations().contains(conv))
						t2.getConversations().add(conv);
				}
			}
		}
	}
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
