package Transforms;

/**
 * This class represents transformation 46 in the transformation process.
 * The result of the transform is that transitions will be made for responder conversations.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import java.util.StringTokenizer;
import system.ATtransition;

public class Transform46 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform46(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * Insert the method's description here.
 * Creation date: (1/19/01 7:10:46 AM)
 */
boolean containsParameter(Vector parameters, Parameter p) {
	if(parameters != null)
	for(int i = 0; i < parameters.size(); i++){
		Parameter param = (Parameter)parameters.elementAt(i);
		if(param.getName().equals(p.getName()))
			return true;
	}
	return false;
}
/**
 * This method will add transitions that start responder conversations
 *
 * the params attribute is not used
 */
public void execute(Object params) {	
	Vector 			agents = sys.agents;
	Agent 			a;
	Vector			components;
	Component		c;
	StateTable		st;
	Vector			transitions;
	Transition		t;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if(t.isStart() && (t.getProtocols() != null) && (t.getReceiveEvent() != null)){
					State from = t.getFrom();
					State to = getEndState(t, st);
					Transition newTrans = new Transition(sys);
					newTrans.setGuard(new String(t.getGuard()));
					t.setGuard(null);
					t.getAtTransition().setGuardCondition("");
					newTrans.setFrom(from);
					newTrans.setTo(to);
					newTrans.setReceiveEvent(t.getReceiveEvent());
					// determine the parameters necessary for the function 
					Vector	parameters = getNeededParameters(t, st);
					// create a function with the name of the conversation being started
					String convID = "conv";
	
					if((t.getConversations() != null) && (t.getConversations().size() == 1)){
						Conversation conv = (Conversation)t.getConversations().elementAt(0);
						convID = conv.getName();
					}
					// since there's more than one conversation, just call it by the convName
					if((t.getConversations() != null) && (t.getConversations().size() > 1)){
						convID = (String)t.getConvNames().elementAt(0);
						convID = convID + "-" + getConvNumbers(t.getConversations());
					}
					FunctionCall f = new FunctionCall(convID, parameters);
					// create the action for the new transition
					Action newAction = new Action(f);
					Vector newActions = new Vector();
					newActions.add(newAction);
					newTrans.setActions(newActions);
					// add the transition to the stateTable					
					st.getTransitions().add(newTrans);

					// now create the corresponding ATtransition
					ATtransition newATtransition = new ATtransition(sys.version, sys.atSys.getNextKey());
					newATtransition.setCurrentState(from.getElement());
					newATtransition.setNextState(to.getElement());
					newATtransition.setCoordinates(from.getElement(), to.getElement());
					newATtransition.setGuardCondition(newTrans.getGuard());
					newATtransition.setRecMessage(t.getRecMessage());
					st.getAtStateTable().addTransition(newATtransition);
					newTrans.setAtTransition(newATtransition);
					newTrans.addATActions();
				}
			}
		}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (1/15/01 9:05:26 AM)
 */
String getConvNumbers(Vector conversations) {
	String numbers = "";
	if(conversations != null)
	for(int i = 0; i < conversations.size(); i++){
		Conversation c = (Conversation)conversations.elementAt(i);
		String name = c.getName();
		int index = name.lastIndexOf("-");
		if(index != -1){
			index++;
			String num = name.substring(index);
			numbers = numbers + num;
			if(i < conversations.size()-1)
				numbers = numbers + "_";
		}
	}
	return numbers;
}
/**
 * This method will find and return the end state for the given start state
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
State getEndState(Transition t, StateTable st) {
	Vector transitions = st.getTransitions();
	Transition t2;
	State endState;
	Vector convs = t.getConversations();
	
	if(transitions != null)
	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(t2.isEnd() && t2.hasAllConversations(convs)){
			endState = t2.getTo();
			return endState;
		}
	}
	
	
	
	return null;
}
/**
 * This method will determine the parameters needed for the action to start the conversation
 * for the given transition.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
Vector getNeededParameters(Transition t, StateTable st) {
	Vector transitions = st.getTransitions();
	Vector states = st.getStates();
	Vector parameters = new Vector();
	Vector actions = null;
	
	// first parameter is the recipient of one of the sendEvents
	ReceiveEvent re = (ReceiveEvent)t.getReceiveEvent();
	parameters.add(new Parameter(re.getSender()));

	// look at all transitions that belong to the convs
	if(transitions != null)
	for(int i = 0; i < transitions.size(); i++){
		Transition t2 = (Transition)transitions.elementAt(i);
		if(t2.hasAllConversations(t.getConversations()) && (t2.getSendEvents() != null)){
			// check all sendevents on the necessary transitions
			Vector sendEvents = t2.getSendEvents();
			for(int j = 0; j < sendEvents.size(); j++){
				SendEvent se2 = (SendEvent)sendEvents.elementAt(j);
				Event e = se2.getEvent();
				Vector params = e.getParameters();
				// check each parameter to see if it's needed
				if(params != null)
				for(int k = 0; k < params.size(); k++){
					Parameter p = (Parameter)params.elementAt(k);
					if(!p.isPrepended() 
						&& !isAssigned(p, t2, st, t2.getConversations(), null)
						&& !containsParameter(parameters, p))
							parameters.add(p);
				}
			}

			// check the actions
			actions = t2.getActions();
			if(actions != null)
			for(int j = 0; j < actions.size(); j++){
				Action a = (Action)actions.elementAt(j);
				Vector rhs = a.getRhs();
				FunctionCall f = a.getFunction();

				// Just need to check the right hand side
				if(rhs != null){
					for(int k = 0; k < rhs.size(); k++){
						String r = (String)rhs.elementAt(k);
						Parameter p = new Parameter(r);
						if(!p.isPrepended() 
							&& !isAssigned(p, a, t2, st, t2.getConversations(), null)
							&& !containsParameter(parameters, p))
								parameters.add(p);
					}
				}
				if(f != null){
					Vector params = f.getParameters();
					if(params != null)
					for(int k = 0; k < params.size(); k++){
						Parameter p = (Parameter)params.elementAt(k);
						if(!p.isPrepended() 
							&& !isAssigned(p, a, t2, st, t2.getConversations(), null)
							&& !containsParameter(parameters, p))
								parameters.add(p);
					}
				}
			}
		}
	}

	// look at all the states that belong to the convs
	if(states != null)
	for(int i = 0; i < states.size(); i++){
		State s = (State)states.elementAt(i);
		if(s.hasAllConversations(t.getConversations()) && (s.getActions() != null)){
			// check the actions
			actions = s.getActions();
			if(actions != null)
			for(int j = 0; j < actions.size(); j++){
				Action a = (Action)actions.elementAt(j);
				Vector rhs = a.getRhs();
				FunctionCall f = a.getFunction();

				// Just need to check the right hand side
				if(rhs != null){
					for(int k = 0; k < rhs.size(); k++){
						String r = (String)rhs.elementAt(k);
						Parameter p = new Parameter(r);
						if(!p.isPrepended() 
							&& !isAssigned(p, a, s, st, s.getConversations())
							&& !containsParameter(parameters, p))
								parameters.add(p);
					}
				}
				if(f != null){
					Vector params = f.getParameters();
					if(params != null)
					for(int k = 0; k < params.size(); k++){
						Parameter p = (Parameter)params.elementAt(k);
						if(!p.isPrepended() 
							&& !isAssigned(p, a, s, st, s.getConversations())
							&& !containsParameter(parameters, p))
								parameters.add(p);
					}
				}
			}
		}
	}
	
	return parameters;
}
/**
 * This method will check to see if the parameter is used somewhere in the guard
 * condition.  Since guard conditions are loosely defined as a string, the method
 * simply checks to see if it is a token in the string.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
public static boolean inGuard(Parameter p, String guard) {
	String param = p.getName();
	StringTokenizer st = new StringTokenizer(guard, " ,=<>()");
	while(st.hasMoreTokens()){
		String token = st.nextToken();
		if(token.equals(param))
			return true;
	}
	
	// default
	return false;
}
/**
 * This method will determine if the parameter given was assigned within
 * the conversation before it was sent in the transition
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean isAssigned(Parameter param, Action act, State s, StateTable st, Vector convs) {
	String pname = param.getName();
	Vector	actions;
	Action  a;
	Vector	transitions = st.getTransitions();
	Transition t2;
	
	// check the actions in the from state to see if they
	// make an assignment to the parameter prior to the

	actions = s.getActions();
	if(actions != null){
		int index = actions.indexOf(act);
		
		if(index == -1)
			System.out.println("Error in isAssigned: action not found");
			
		for(int i = 0; i < index; i++){
			a = (Action)actions.elementAt(i);
			Vector lhs = a.getLhs();
			if(lhs != null)
			for(int j = 0; j < lhs.size(); j++){
				String p = (String)lhs.elementAt(j);
				if(p.equals(pname))
					return true;
			}
		}
	}
	
	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(t2.hasAllConversations(convs) && (t2.getTo() == s)){
			if(isAssigned(param, t2, st, convs, null)){
				return true;
			}
		}
	}	
	
	return false;
}
/**
 * This method will determine if the parameter given was assigned within
 * the conversation before it was sent in the transition
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean isAssigned(Parameter param, Action act, Transition t, StateTable st, Vector convs, Vector checked) {
	String pname = param.getName();
	State 	s = t.getFrom();
	Action 	a;
	Vector	actions;
	Vector	transitions = st.getTransitions();
	Transition t2;
	boolean	sent = false;
	boolean received = false;

	// check to see if there was a received event with the parameter
	ReceiveEvent re = t.getReceiveEvent();
	if((re != null) && (re.getEvent().hasParameter(param))){
		return true;
	}
	
	// check the actions in the transition to see if they
	// make an assignment to the parameter
	actions = t.getActions();
	if(actions != null){
		int index = actions.indexOf(act);
		
		if(index == -1)
			System.out.println("Error in isAssigned: action not found");
			
		for(int i = 0; i < index; i++){
			a = (Action)actions.elementAt(i);
			Vector lhs = a.getLhs();
			if(lhs != null)
			for(int j = 0; j < lhs.size(); j++){
				String p = (String)lhs.elementAt(j);
				if(p.equals(pname))
					return true;
			}
		}
	}
	
	// check the actions in the from state to see if they
	// make an assignment to the parameter
	// (only if the state has all of the conversations)
	actions = s.getActions();
	if(s.hasAllConversations(convs) && (actions != null))
	for(int i = 0; i < actions.size(); i++){
		a = (Action)actions.elementAt(i);
		Vector lhs = a.getLhs();
		if(lhs != null)
		for(int j = 0; j < lhs.size(); j++){
			String p = (String)lhs.elementAt(j);
			if(p.equals(pname))
				return true;
		}
	}

	if(checked == null)
		checked = new Vector();
	checked.add(t);
	
	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(!checked.contains(t2) && t2.hasAllConversations(convs) && (t2.getTo() == t.getFrom())){
			if(isAssigned(param, t2, st, convs, checked)){
				return true;
			}
		}
	}	
	
	return false;
}
/**
 * This method will determine if the parameter given was assigned within
 * the conversation before it was sent in the transition
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean isAssigned(Parameter param, Transition t, StateTable st, Vector convs, Vector checked) {
	String pname = param.getName();
	State 	s = t.getFrom();
	Action 	a;
	Vector	actions;
	Vector	transitions = st.getTransitions();
	Transition t2;
	boolean	sent = false;
	boolean received = false;

	// check to see if there was a received event with the parameter
	ReceiveEvent re = t.getReceiveEvent();
	if((re != null) && (re.getEvent().hasParameter(param))){
		return true;
	}
	
	// check the actions in the transition to see if they
	// make an assignment to the parameter
	actions = t.getActions();
	if(actions != null)
	for(int i = 0; i < actions.size(); i++){
		a = (Action)actions.elementAt(i);
		Vector lhs = a.getLhs();
		if(lhs != null)
		for(int j = 0; j < lhs.size(); j++){
			String p = (String)lhs.elementAt(j);
			if(p.equals(pname))
				return true;
		}
	}
	
	// check the actions in the from state to see if they
	// make an assignment to the parameter
	// (only if the state has all of the conversations)
	actions = s.getActions();
	if(s.hasAllConversations(convs) && (actions != null))
	for(int i = 0; i < actions.size(); i++){
		a = (Action)actions.elementAt(i);
		Vector lhs = a.getLhs();
		if(lhs != null)
		for(int j = 0; j < lhs.size(); j++){
			String p = (String)lhs.elementAt(j);
			if(p.equals(pname))
				return true;
		}
	}

	if(checked == null)
		checked = new Vector();
	checked.add(t);
	
	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(!checked.contains(t2) && t2.hasAllConversations(convs) && (t2.getTo() == t.getFrom())){
			if(isAssigned(param, t2, st, convs, checked)){
				return true;
			}
		}
	}	
	
	return false;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
/**
 * This method checks to see if the parameter is used somewhere in the action.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean usedInAction(Parameter p, Action a) {
	Vector lhs = a.getLhs();
	Vector rhs = a.getRhs();
	FunctionCall f = a.getFunction();
	
	
	if(lhs != null){
		for(int i = 0; i < lhs.size(); i++){
			String s = (String)lhs.elementAt(i);
			if(s.equals(p.getName()) 
				|| s.equals("parent." + p.getName())
				|| ("parent." + s).equals(p.getName()))
				return true;
		}
	}
	if(rhs != null){
		for(int i = 0; i < rhs.size(); i++){
			String s = (String)rhs.elementAt(i);
			if(s.equals(p.getName()) 
				|| s.equals("parent." + p.getName())
				|| ("parent." + s).equals(p.getName()))
				return true;
		}
	}
	if(f != null){
		Vector parameters = f.getParameters();
		if(parameters != null)
		for(int i = 0; i < parameters.size(); i++){
			Parameter p2 = (Parameter)parameters.elementAt(i);
			if(p2.getName().equals(p.getName()) 
				|| p2.getName().equals("parent." + p.getName())
				|| ("parent." + p2.getName()).equals(p.getName()))
				return true;
		}
	}
		
	// default
	return false;
}
/**
 * This method checks to see if the parameter is used somewhere in the transition.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean usedInTransition(Parameter p, Transition t) {
	Event receive = t.getReceive();
	if(receive != null){
		if(receive.hasParameter(p))
		return true;
	}
	if(t.getReceiveEvent() != null){
		ReceiveEvent re = t.getReceiveEvent();
		Event e = re.getEvent();
		if(e.hasParameter(p))
		return true;
	}
	if((t.getGuard() != null) && inGuard(p, t.getGuard())){
		return true;		
	}
	Vector sends = t.getSends();
	if(sends != null){
		for(int x = 0; x < sends.size(); x++){
			Event e = (Event)sends.elementAt(x);
			if(e.hasParameter(p))
			return true;
		}
	}
	Vector sendEvents = t.getSendEvents();
	if(sendEvents != null){
		for(int x = 0; x < sendEvents.size(); x++){
			SendEvent se = (SendEvent)sendEvents.elementAt(x);
			Event e = se.getEvent();
			if(e.hasParameter(p))
			return true;
		}
		
	}
	Vector actions = t.getActions();
	if(actions != null){
		for(int i = 0; i < actions.size(); i++){
			Action a = (Action)actions.elementAt(i);
			if(usedInAction(p, a))
				return true;
		}
	}
		
	// default
	return false;
}
}
