package Transforms;

/**
 * This class represents transformation 48 in the transformation process.
 * The result of the transform is that states in component state tables that
 * belong to conversations will be placed in the conversation halves and removed
 * from the component.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.ATtableElement;
import system.ATstatetable;

public class Transform48 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform48(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will determine which conversation half of a conversation the
 * component has.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
ConversationHalf determineConvHalf(Conversation conv, Component c) {
	if(c.getConvs().contains(conv.getInitiator()))
		return conv.getInitiator();
	else if(c.getConvs().contains(conv.getResponder()))
		return conv.getResponder();
	else{
		System.out.println("Error in T41");
		return null;
	}
}
/**
 * This method will add states to the appropriate conversation halves and remove them
 * from the component state tables.
 *
 * the params attribute is not used
 */
public void execute(Object params) {	
	Vector 			agents = sys.agents;
	Agent 			a;
	Vector			components;
	Component		c;
	StateTable		st;
	Vector			states;
	State			s;
	Conversation 	conv;
	ConversationHalf ch;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			states = st.getStates();
			if(states != null)
			for(int k = 0; k < states.size(); k++){
				s = (State)states.elementAt(k);
				if((s.getConversations() != null) && !s.getName().equals("StartState") && !s.getName().equals("EndState")){
					Vector conversations = s.getConversations();
					for(int l = 0; l < conversations.size(); l++){
						conv = (Conversation)conversations.elementAt(l);
						ch = determineConvHalf(conv, c);
						StateTable chStateTable = ch.getStateTable();
						// copy the state
						State s2 = s.copy();
						// add the state to the state table
						chStateTable.getStates().add(s2);
						// add s2's ATtableElement to the ATstatetable
						ATtableElement atState = s2.getElement();
						ATstatetable atStateTable = chStateTable.getAtStateTable();
						atStateTable.addState(atState);
					}
					// now remove the state from the component's state table
					// first remove the state from the atStateTable
					ATstatetable atStateTable = st.getAtStateTable();
					ATtableElement element = s.getElement();
					atStateTable.removeState(element);
					st.getStates().remove(s);	
					k--;	
				}				
			}
		}
	}	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
