package Transforms;

/**
 * This class represents transformations 51 and 52 in the transformation process.
 * The result of the transform is that external events in conversation halves are
 * changed into internal events.  
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.ATtransition;
import system.ATtableElement;
import system.ATstatetable;

public class Transform51 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform51(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will change events in conversation halves from external events into
 * internal events.
 *
 * the params attribute is not used
 */
public void execute(Object params) {	
	Vector 			agents = sys.agents;
	Agent 			a;
	Vector			components;
	Component		c;
	Vector			convs;
	StateTable		st;
	Vector			transitions;
	Transition		t;
	Conversation 	conv;
	ConversationHalf ch;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			convs = c.getConvs();
			if(convs != null)
			for(int k = 0; k < convs.size(); k++){
				ch = (ConversationHalf)convs.elementAt(k);
				st = ch.getStateTable();
				transitions = st.getTransitions();
				if(transitions != null)
				for(int l = 0; l < transitions.size(); l++){
					t = (Transition)transitions.elementAt(l);
					if(t.getReceiveEvent() != null){
						t.setReceive(t.getReceiveEvent().getEvent());
						t.setReceiveEvent(null);
						t.getAtTransition().setRecMessage(t.getRecMessage());
					}	
					if(t.getSendEvents() != null){
						Vector sendEvents = (Vector)t.getSendEvents().clone();
						for(int m = 0; m < sendEvents.size(); m++){
							SendEvent se = (SendEvent)sendEvents.elementAt(m);
							if(t.getSends() == null)
								t.setSends(new Vector());
							t.getSends().add(se.getEvent());
							t.getSendEvents().remove(se);
						}
						t.getAtTransition().setTransMessage(t.getTransMessage());
					}
				}
			}
		}
	}	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
