package Transforms;

/**
 * This class is used to represent a transition in a state table
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import java.util.StringTokenizer;
import system.ATtransition;
 
public class Transition {
	private ATtransition	atTransition = null;
	private Event			receive = null;			
	private ReceiveEvent	receiveEvent = null;
	private String			guard = null;
	private Vector			actions = null;			// SEQUENCE of actions
	private Vector			sends = null;			// Set of Events to be sent
	private Vector			sendEvents = null;		// Set of SendEvents to be sent
	private State			from = null;
	private String			fromName = null;
	private State			to = null;
	private String			toName = null;
	private boolean 		start = false;
	private boolean			end = false;
	private Vector			convNames = null;		// SET of strings - only on start - the conversations that are started on the trans
	private Vector			conversations = null;	// SET of conversations that are started from the transition
	private Vector			protocols = null;		// SET of protocols that ALL of the external events belong to
	private boolean			determinedProtocols = false;
	//private Vector			convMatches = null;		// SET of transitions in other components that match in the conversation
	private boolean			matched = false;
	private Vector			transMatches = null;
	private TSystem			sys = null;
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 11:22:19 AM)
 */
public Transition(ATtransition t, TSystem tSys) {
	atTransition = t;
	sys = tSys;
}
/**
 * Insert the method's description here.
 * Creation date: (11/11/00 12:58:08 PM)
 */
public Transition(TSystem tSys) {
	sys = tSys;
}
/**
 * This method will add its actions to its ATtransitions actions
 *
 * Creation date: (11/29/00 1:30:54 PM)
 */
public void addATActions() {
	if(actions != null){
		Vector acts = getActionsAsStrings();
		for(int i = 0; i < acts.size(); i++){
			String a = (String)acts.elementAt(i);
			atTransition.addAction(a);
		}
		atTransition.setAction(acts);
	}	
}
/**
 * This method will add the given conversation to its conversations set
 *
 * Creation date: (11/13/00 1:04:27 PM)
 */
public void addConversation(Conversation conv) {
	if(conversations == null)
		conversations = new Vector();
	conversations.add(conv);
}
/**
 * This method will create and return a Transition that is
 * logically equivalent to this one
 *
 * Creation date: (11/15/00 5:53:34 PM)
 */
public Transition copy() {
	Transition t = new Transition(sys);
	if(receive != null)
		t.receive = receive.copy();
	if(receiveEvent != null)
		t.setReceiveEvent(receiveEvent.copy());
	if(guard != null)	
		t.guard = String.valueOf(guard);
	if((actions != null) && (actions.size() > 0)){
		Vector actions2 = new Vector();
		for(int i = 0; i < actions.size(); i++){
			Action a = (Action)actions.elementAt(i);
			actions2.add(a.copy());
		}
		t.actions = actions2;
	}

	if((sends != null) && (sends.size() > 0)){
		Vector sends2 = new Vector();
		for(int i = 0; i < sends.size(); i++){
			Event e = (Event)sends.elementAt(i);
			sends2.add(e.copy());
		}
		t.setSends(sends2);
	}

	if((sendEvents != null) && (sendEvents.size() > 0)){
		Vector sendEvents2 = new Vector();
		for(int i = 0; i < sendEvents.size(); i++){
			SendEvent se = (SendEvent)sendEvents.elementAt(i);
			sendEvents2.add(se.copy());
		}
		t.setSendEvents(sendEvents2);
	}
	t.setFrom(from);
	t.setTo(to);
	t.setFromName(fromName);
	t.setToName(toName);
	t.setProtocols(protocols);

	// now create the ATtransition
	ATtransition oldT = atTransition;
	ATtransition newT = new ATtransition(sys.version, sys.atSys.getNextKey());
	String rec = String.valueOf(oldT.getRecMessage());
	newT.setRecMessage(rec);
	String gd = String.valueOf(oldT.getGuardCondition());
	newT.setGuardCondition(gd);
	String trans = String.valueOf(oldT.getTransMessage());
	newT.setTransMessage(trans);
	Vector oldActions = oldT.getActions();
	Vector newActions = (Vector)oldActions.clone();
	newT.setAction(newActions);

	t.setAtTransition(newT);
	
	return t;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return java.util.Vector
 */
public java.util.Vector getActions() {
	return actions;
}
/**
 * This method will return a vector of strings that represents its actions
 *
 * Creation date: (11/22/00 8:39:31 PM)
 */
public Vector getActionsAsStrings() {
	// Vector of strings representing the actions
	Vector acts = new Vector();		
	if(actions != null)
	for(int i = 0; i < actions.size(); i++){
		Action a = (Action)actions.elementAt(i);
		String s = a.toString();
		acts.add(s);
	}
	
	return acts;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return system.ATtransition
 */
public system.ATtransition getAtTransition() {
	return atTransition;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return java.util.Vector
 */
public java.util.Vector getConversations() {
	return conversations;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return java.util.Vector
 */
public java.util.Vector getConvNames() {
	return convNames;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return Transforms.State
 */
public State getFrom() {
	return from;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return java.lang.String
 */
public java.lang.String getFromName() {
	return fromName;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return java.lang.String
 */
public java.lang.String getGuard() {
	return guard;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return java.util.Vector
 */
public java.util.Vector getProtocols() {
	return protocols;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return Transforms.Event
 */
public Event getReceive() {
	return receive;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return Transforms.ReceiveEvent
 */
public ReceiveEvent getReceiveEvent() {
	return receiveEvent;
}
/**
 * This method will return a string that represents the received message
 *
 * Creation date: (11/22/00 8:39:31 PM)
 */
public String getRecMessage() {
	if(receive != null)
		return receive.toString();
	if(receiveEvent != null)
		return receiveEvent.toString();
	return "";
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return java.util.Vector
 */
public java.util.Vector getSendEvents() {
	return sendEvents;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return java.util.Vector
 */
public java.util.Vector getSends() {
	return sends;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return Transforms.State
 */
public State getTo() {
	return to;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return java.lang.String
 */
public java.lang.String getToName() {
	return toName;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:41:07 AM)
 * @return java.util.Vector
 */
public java.util.Vector getTransMatches() {
	transMatches = new Vector();
	if((receiveEvent != null) && (receiveEvent.getTransMatches() != null))
		transMatches.addAll(receiveEvent.getTransMatches());
	if(sendEvents != null)
	for(int i = 0; i < sendEvents.size(); i++){
		SendEvent se = (SendEvent)sendEvents.elementAt(i);
		if(se.getTransMatches() != null)
			transMatches.addAll(se.getTransMatches());
	}
		
	if(transMatches.size() == 0)
		return null;
	else
		return transMatches;
}
/**
 * This method will return a string that represents all of the transmissions on the transition
 *
 * Creation date: (11/22/00 8:39:31 PM)
 */
public String getTransMessage() {
	String t = "";
	
	if((sends != null) && (sends.size() > 0)){
		for(int i = 0; i < sends.size(); i++){
			Event e = (Event)sends.elementAt(i);
			t += e.toString();
			if(i+1 < sends.size())
				t += "; ";
		}
	}
	
	if((sendEvents != null) && (sendEvents.size() > 0)){
		if((sends != null) && (sends.size() > 0))
			t+= "; ";
		for(int i = 0; i < sendEvents.size(); i++){
			SendEvent s = (SendEvent)sendEvents.elementAt(i);
			t += s.toString();
			if(i+1 < sendEvents.size())
				t += "; ";
		}
	}

	return t;
}
/**
 * This method is used to determine if the transition has all of the conversations in the
 * given vector
 *
 * Creation date: (11/22/00 8:39:31 PM)
 */
public boolean hasAllConversations(Vector convs) {
	if(convs == null)
	return true;

	if(conversations == null)
	return false;

	// loop through and test each conversation in conversations
	for(int i = 0; i < convs.size(); i++){
		Conversation conv = (Conversation)convs.elementAt(i);
		if(!conversations.contains(conv))
			return false;
	}
	
	// default
	return true;	
}
/**
 * This method is used to determine if the given conversation is in the
 * transition's conversations set
 *
 * Creation date: (11/29/00 1:30:54 PM)
 */
public boolean hasConversation(Conversation conv) {
	if((conversations != null) && conversations.contains(conv))
		return true;

	// default
	return false;
}
/**
 * This method is used to determine if there is a conversation with the
 * given name in the transition's conversations set
 *
 * Creation date: (11/29/00 1:30:54 PM)
 */
public boolean hasConvName(String cname) {
	if(convNames != null)
	for(int i = 0; i < convNames.size(); i++){
		String cname2 = (String)convNames.elementAt(i);
		if(cname2.equals(cname))
		return true;
	}

	// default
	return false;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return boolean
 */
public boolean isDeterminedProtocols() {
	return determinedProtocols;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return boolean
 */
public boolean isEnd() {
	return end;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return boolean
 */
public boolean isMatched() {
	return matched;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @return boolean
 */
public boolean isStart() {
	return start;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newActions java.util.Vector
 */
public void setActions(java.util.Vector newActions) {
	actions = newActions;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newAtTransition system.ATtransition
 */
public void setAtTransition(system.ATtransition newAtTransition) {
	atTransition = newAtTransition;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newConversations java.util.Vector
 */
public void setConversations(java.util.Vector newConversations) {
	conversations = newConversations;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newConvNames java.util.Vector
 */
public void setConvNames(java.util.Vector newConvNames) {
	convNames = newConvNames;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newDeterminedProtocols boolean
 */
public void setDeterminedProtocols(boolean newDeterminedProtocols) {
	determinedProtocols = newDeterminedProtocols;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newEnd boolean
 */
public void setEnd(boolean newEnd) {
	end = newEnd;

	if(atTransition != null)
		atTransition.setEnd(newEnd);
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newFrom Transforms.State
 */
public void setFrom(State newFrom) {
	from = newFrom;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newFromName java.lang.String
 */
public void setFromName(java.lang.String newFromName) {
	fromName = newFromName;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newGuard java.lang.String
 */
public void setGuard(java.lang.String newGuard) {
	guard = newGuard;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newMatched boolean
 */
public void setMatched(boolean newMatched) {
	matched = newMatched;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newProtocols java.util.Vector
 */
public void setProtocols(java.util.Vector newProtocols) {
	protocols = newProtocols;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newReceive Transforms.Event
 */
public void setReceive(Event newReceive) {
	receive = newReceive;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newReceiveEvent Transforms.ReceiveEvent
 */
public void setReceiveEvent(ReceiveEvent newReceiveEvent) {
	receiveEvent = newReceiveEvent;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newSendEvents java.util.Vector
 */
public void setSendEvents(java.util.Vector newSendEvents) {
	sendEvents = newSendEvents;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newSends java.util.Vector
 */
public void setSends(java.util.Vector newSends) {
	sends = newSends;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newStart boolean
 */
public void setStart(boolean newStart) {
	start = newStart;

	if(atTransition != null)
		atTransition.setStart(newStart);
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newTo Transforms.State
 */
public void setTo(State newTo) {
	to = newTo;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:47:31 AM)
 * @param newToName java.lang.String
 */
public void setToName(java.lang.String newToName) {
	toName = newToName;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:41:07 AM)
 * @param newTransMatches java.util.Vector
 */
public void setTransMatches(java.util.Vector newTransMatches) {
	transMatches = newTransMatches;
}
/**
 * Show method used for debugging
 *
 * Creation date: (11/6/00 1:19:48 PM)
 */
public void show() {
	System.out.println("Transition: " + toString());

	if(receive != null){
		System.out.print("receive: ");		
		receive.show();
		System.out.println();
	}
	if(receiveEvent != null){
		System.out.print("receiveEvent: ");
		if(receiveEvent != null)
			receiveEvent.show();
		System.out.println();
	}
	if((guard != null) && (!guard.equals("")))
		System.out.println("guard: " + guard);
	if((actions != null) && (actions.size() != 0)){
		System.out.println("Actions: ");
		for(int i = 0; i < actions.size(); i++){
			Action a = (Action)actions.elementAt(i);
			a.show();
		}
	}
	if(sends != null){
		System.out.print("sends: ");
		for(int i = 0; i < sends.size(); i++){
			Event e = (Event)sends.elementAt(i);
			e.show();
			if(i+1 < sends.size())
				System.out.print("; ");
		}
		System.out.println();
	}
	if(sendEvents != null){
		System.out.println("sendEvents: ");
		for(int i = 0; i < sendEvents.size(); i++){
			SendEvent s = (SendEvent)sendEvents.elementAt(i);
			System.out.print("\t");
			s.show();
		}
	}
	if(protocols != null){
		System.out.print("protocols: ");
		for(int i = 0; i < protocols.size(); i++){
			Protocol p = (Protocol)protocols.elementAt(i);
			System.out.print(p.getName());
			if(i+1 < protocols.size())
				System.out.print(", ");
		}
		System.out.println();
	}
	else{
		System.out.println("protocols: {}");
	}
	if(start)
		System.out.println("start = true");
	if(end)
		System.out.println("end = true");
	
	if(convNames != null){
		System.out.print("Conversation Names: ");
		for(int i = 0; i < convNames.size(); i++){
			System.out.print((String)convNames.elementAt(i));
			if(i+1 < convNames.size())
				System.out.print(", ");
		}
		System.out.println();
	}
	
	if(conversations != null){
		System.out.print("Conversations: ");
		for(int i = 0; i < conversations.size(); i++){
			Conversation conv = (Conversation)conversations.elementAt(i);
			System.out.print(conv.getName());
			if(i+1 < conversations.size())
				System.out.print(", ");
		}
		System.out.println();
	}
	
}
/**
 * Returns a string representation of the transition
 * Creation date: (11/6/00 1:19:48 PM)
 */
public String toString() {
	String t = "";
	if(receive != null)
		t += receive.toString();
	if(receiveEvent != null)
		t += receiveEvent.toString();
	if((guard != null) && !guard.equals("")){
		t += "[" + guard + "]";
	}
	if((actions != null) && (actions.size() != 0)){
		t += " /";
		for(int i = 0; i < actions.size(); i++){
			Action a = (Action)actions.elementAt(i);
			t += a.toString();
			if(i+1 < actions.size())
				t += "; ";
		}
	}
	if((sends != null) || (sendEvents != null))
		t += " ^";

	if(sends != null){
		for(int i = 0; i < sends.size(); i++){
			Event e = (Event)sends.elementAt(i);
			t += e.toString();
			if(i+1 < sends.size())
				t += "; ";
		}
	}
	
	if(sendEvents != null){
		if(sends != null)
			t+= "; ";
		for(int i = 0; i < sendEvents.size(); i++){
			SendEvent s = (SendEvent)sendEvents.elementAt(i);
			t += s.toString();
			if(i+1 < sendEvents.size())
				t += "; ";
		}
	}
	t += "\n";
	
	t += "from: " + from.getName() + "\n"; 
	t += "to: " + to.getName() + "";
	
	return t;	
}
/**
 * This method will create and return a Transition that corresponds
 * to the given ATtransition
 *
 * Creation date: (11/3/00 4:01:42 PM)
 */
public static Transition translateIn(ATtransition atTransition, TSystem tSys) {
	Transition	transition = new Transition(atTransition, tSys);
	String		current, next, receive, send, guard;
	Vector		actions;
	StringTokenizer st, st2;
	String 		s, performative, param, sender = null, recipient = null;
	Vector		parameters;
	Event		event;

	current = atTransition.getCurrentState().getName();
	next = atTransition.getNextState().getName();
	receive = atTransition.getRecMessage();
	send = atTransition.getTransMessage();
	guard = atTransition.getGuardCondition();
	actions = atTransition.getActions();

	// parse receive
	if(receive != null){
		st = new StringTokenizer(receive, "(), ");
		if(st.countTokens() > 0){
			s = st.nextToken();
			if(s.equals("receive")){
				// there is an external receiveEvent
				performative = st.nextToken();
				parameters = new Vector();
				while(st.hasMoreTokens()){
					param = st.nextToken();
					if(st.hasMoreTokens()){
						parameters.add(new Parameter(param));
					}
					else{
						sender = param;
					}
				}
				event = new Event(performative, parameters);
				transition.setReceiveEvent(new ReceiveEvent(event, sender));
			}
			else{
				// there is an internal receive Event
				performative = s;
				parameters = new Vector();
				while(st.hasMoreTokens()){
					param = st.nextToken();
					parameters.add(new Parameter(param));
				}
				event = new Event(performative, parameters);
				transition.receive = event;
			}
		}
	}

	// set guard condition
	transition.guard = guard;
	
	// parse send
	if(send != null){
		// transmissions are separated by semicolon
		st = new StringTokenizer(send, ";");
		while(st.hasMoreTokens()){
			// parse individual transmissions
			s = st.nextToken();
			st2 = new StringTokenizer(s, "(), ");
			if(st2.countTokens() > 0){
				s = st2.nextToken();
				if(s.equals("send")){
					// there is an external SendEvent
					performative = st2.nextToken();
					parameters = new Vector();
					while(st2.hasMoreTokens()){
						param = st2.nextToken();
						if(st2.hasMoreTokens()){
							parameters.add(new Parameter(param));
						}
						else{
							recipient = param;
						}
					}
					event = new Event(performative, parameters);
					if(transition.getSendEvents() == null){
						transition.sendEvents = new Vector();
					}
					transition.sendEvents.add(new SendEvent(event, recipient));
				}
				else{
					// there is an internal send Event
					performative = s;
					parameters = new Vector();
					while(st2.hasMoreTokens()){
						param = st2.nextToken();
						parameters.add(new Parameter(param));
					}
					event = new Event(performative, parameters);
					if(transition.getSends() == null){
						transition.setSends(new Vector());
					}
					transition.getSends().add(event);
				}
			}			
		}
	}

	// parse actions
	
	if(actions != null){
		Vector newActions = new Vector();
		for(int i = 0; i < actions.size(); i++){
			String currentAction = (String)actions.elementAt(i);
			newActions.add(Action.translateIn(currentAction));
		}
		transition.actions = newActions;
	}
	
	// This just matches up the names.  
	// The actual pointers to the states still need to be set!
	transition.setFromName(current);
	transition.setToName(next);
	
	return transition;
}
/**
 * This method will change the appropriate element in convNames from
 * the old name to the new name
 *
 * Creation date: (11/22/00 8:39:31 PM)
 */
public void updateConvName(String old, String newConvName) {
	if(convNames != null){
		for(int i = 0; i < convNames.size(); i++){
			String cname = (String)convNames.elementAt(i);
			if(cname.equals(old)){
				convNames.setElementAt(newConvName, i);
			}
		}
	}
}
}
