package Verify;

/**
 * A Class for verifying conversations created in agentTool or input from a file,
 * <tt>statetbl.txt</tt>
 * When run from in side Visual Age, the properties tab must contain the statement,
 * <tt>user.dir=c:\ibmvjava\ide\program</tt> where the user directory points to the
 * directory where visual age runs it's programs.  The path to the program directory
 * may be different on your machine.
 * @version 1.0 6 Oct 1999
 * @author Capt Tim Lacey
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import java.io.*;
import java.util.*;
import java.awt.*;
import system.*;
public class VCRun
{
	public static TextMessages spinFrame = new TextMessages();
	public static Vector stateTable = null;
	public static Vector panTable = null;
	public static Vector traceTable = null;
	public static Vector errorTable = null;
	public static Vector verifyTable = null;
	public static Vector errorVector = null;
	public static String workingDirectory = "";
	public static File tableFile;
	public static File panTextFile;
	public static File panTextFile2;
	public static File traceTextFile;
	public static File panFile;
	public static File verifyFile;
	public static File panCFile;
	public static File panTraceFile;
	public static File errorFile;
	public static boolean readyToRunSpin = false;
	public static boolean foundError = false;
	//New line	
	public static final String LF = "\n"; 
	//Unicode Page break
	public static final String formFeed = "\u000C"; 
	//Unicode Tab
	public static final String Tab = "\u0009"; 
/**
 * If the class is executed by itself, the main method gets run automatically
 * @param nothing
 * @return nothing
 * @throws nothing
 */
public static void main(String args[])
{
	// Set system properties used in this program
	setProperties();
	// Show the message window where all messages are displayed
	spinFrame.show();
	// Write the first message to the window.
	spinFrame.ta.append("PLEASE STAND BY... TESTING CONVERSATIONS...\n");
	// Write status messages to the console so you can verify the process is running
	System.out.println("Reading State Table...\n");
	// Read in the input file, statetbl.txt
	stateTable = readStateFile();
	System.out.println("Creating Promela Source Code...\n");
	// Create the output file, verify, from the input file, statetbl.txt
	writeOutput(stateTable,5);
	// Run Spin against the Promela code and analyze it
	errorVector = new Vector();
	runSpin();
	// Testing is completed
	spinFrame.ta.append("\n");
	spinFrame.ta.append("********** TESTING COMPLETED **********\n\n");
}
/**
 * Read the error file generated when running Spin, one line at a time, and store in
 * a vector for use later
 * @param nothing
 * @return the vector that contains all the lines of the error file <tt>error</tt>
 * @throws IOException
 */
public static Vector readErrorFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line = null;
	// Create a vector that will hold every line of the input file
	Vector errorTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	ErrorTable errortbl = null;
	try
	{
		// Open an input stream from the error file
		inStream = new BufferedReader(new FileReader(errorFile));
		// read a line of text
		line = inStream.readLine();
		// if the line is not equal to null then an error has been generated
		if (line != null)
		{
			// set a global flag that will be used in the runSpin method
			foundError = true;
		}
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// if the actual length of the file is greater than 0, add the line to the
			// vector
			if (line.trim().length() > 0)
			{
				// get a new line
				errortbl = new ErrorTable(line);
				// add the line to the vector
				errorTable.addElement(errortbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector of error messages
	return errorTable;
}
/**
 * Read the file generated by analyzing the progress state checks, <tt>newpan2.txt</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>newpan2.txt</tt>
 * @throws IOException
 */
public static Vector readPan2File()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line = null;
	// Create a vector that will hold every line of the input file
	Vector panTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	PanTable pantbl = null;
	try
	{
		// Open an input stream from the error file
		inStream = new BufferedReader(new FileReader(panTextFile2));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// if the actual length of the file is greater than 0, add the line to the
			// vector
			if (line.trim().length() > 0)
			{
				// get a new line
				pantbl = new PanTable(line);
				// add the line to the vector
				panTable.addElement(pantbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return panTable;
}
/**
 * Read the file generated by analyzing the deadlock checks, <tt>newpan.txt</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>newpan.txt</tt>
 * @throws IOException
 */
public static Vector readPanFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line = null;
	// Create a vector that will hold every line of the input file
	Vector panTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	PanTable pantbl = null;
	try
	{
		// Open an input stream from the error file
		inStream = new BufferedReader(new FileReader(panTextFile));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// if the actual length of the file is greater than 0, add the line to the
			// vector
			if (line.trim().length() > 0)
			{
				// get a new line
				pantbl = new PanTable(line);
				// add the line to the vector
				panTable.addElement(pantbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return panTable;
}
/**
 * Read the file that contains the state table information, <tt>statetbl.txt</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>statetbl.txt</tt>
 * @throws IOException
 */
public static Vector readStateFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line;
	// Create a vector that will hold every line of the input file
	Vector stateTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	StateTable statetbl = null;
	try
	{
		// Open an input stream from the error file
		inStream = new BufferedReader(new FileReader(tableFile));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// if the actual length of the file is greater than 0, add the line to the
			// vector
			if (line.trim().length() > 0)
			{
				// get a new line
				statetbl = new StateTable(line);
				// add the line to the vector
				stateTable.addElement(statetbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return stateTable;
}
/**
 * Read the vector that contains the state table information that is passed by
 * agentTool.  The vector contains lines of strings that must be parsed into
 * a vector of StateTable types
 * @param The vector containing the conversation state information needed
 * to verify a conversation
 * @return the vector that contains all the lines of the state vector
 * @throws nothing
 */
public static Vector readStateTable(Vector v)
{
	// Create a string variable that contains each line of the input vector
	String line = null;
	// Create a vector that will hold every line of the input vector
	Vector stateTable = new Vector();
	// Create a variable of the type that will be pulled from the vector
	StateTable statetbl = null;
	// while there are more elements in the input vector, write to the new vector
	for (int k = 0; k < v.size(); k++)
	{
		// get the line at the kth element
		line = (String) v.elementAt(k);
		// read the line into the proper format
		statetbl = new StateTable(line);
		// add the formated line to the new vector
		stateTable.addElement(statetbl);
	}
	// return the filled vector
	return stateTable;
}
/**
 * Read the file generated by creating a trace of an error, <tt>trace.txt</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>trace.txt</tt>
 * @throws IOException
 */
public static Vector readTraceFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line = null;
	// Create a vector that will hold every line of the input file
	Vector traceTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	TraceTable tracetbl = null;
	try
	{
		// Open an input stream from the error file
		inStream = new BufferedReader(new FileReader(traceTextFile));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			if (line.trim().length() > 0)
			{
				// get a new line
				tracetbl = new TraceTable(line);
				// add the line to the vector
				traceTable.addElement(tracetbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return traceTable;
}
/**
 * Read the file created by this program from the state table, <tt>verify</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>verify</tt>
 * @throws IOException
 */
public static Vector readVerifyFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line = null;
	// Create a vector that will hold every line of the input file
	Vector verifyTable = new Vector();
	try
	{
		// Open an input stream from the error file
		inStream = new BufferedReader(new FileReader(verifyFile));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// add the line to the vector
			verifyTable.addElement(line);
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return verifyTable;
}
/**
 * Run 8 batch files that will determine if conversations are valid or not
 * @param nothing
 * @return nothing
 * @throws IOException, InterruptedException
 */
public static void runSpin()
{
	// Set the text color to black as default, change to red if an error found
	spinFrame.ta.setForeground(Color.black);
	// if any of the files used in the previous analysis exists, delete them
	// so each analysis is a clean run
	if (panFile.exists())
	{
		boolean deleted = panFile.delete();
	}
	if (panTextFile.exists())
	{
		boolean deleted = panTextFile.delete();
	}
	if (panTextFile2.exists())
	{
		boolean deleted = panTextFile2.delete();
	}
	if (traceTextFile.exists())
	{
		boolean deleted = traceTextFile.delete();
	}
	if (panCFile.exists())
	{
		boolean deleted = panCFile.delete();
	}
	if (panTraceFile.exists())
	{
		boolean deleted = panTraceFile.delete();
	}
	if (errorFile.exists())
	{
		boolean deleted = errorFile.delete();
	}
	// do not run Spin until the Verify file has been created
	// just sit here in this loop until the flag becomes true
	while (!readyToRunSpin)
	{
	}
	try
	{
		System.out.println("Running Spin...");
		// Run the batch file gospin.bat
		// The command is (redir -o error spin -a verify)
		// This command executes spin on the file called verify and redirects the
		// output to a file called error
		Process tempObject = Runtime.getRuntime().exec(workingDirectory + "gospin.bat");
		// Wait until the process has completed before moving on
		tempObject.waitFor();
		// Force garbage collection to free up memory
		System.runFinalization();
	}
	catch (IOException exc)
	{
		System.out.println("Error running Spin");
	}
	catch (InterruptedException exc)
	{
		System.out.println("Error running Spin");
	}
	spinFrame.ta.append("\n");
	spinFrame.ta.append(Tab + "!!!!!!!!!! OUTPUT OF SPIN ANALYSIS !!!!!!!!!!\n\n");
	// If an error file exists, then something went wrong when running Spin
	// This would happen if the Promela code was constructed incorrectly
	if (errorFile.exists())
	{
		// Read the error file, error
		errorTable = readErrorFile();
	}
	// If there is indeed an error the loop through the vector of error messages and
	// print them in the message window
	if (foundError)
	{
		for (int k = 0; k < errorTable.size(); k++)
		{
			ErrorTable errorTemp = (ErrorTable) errorTable.elementAt(k);
			String errorMessage = errorTemp.getErrorMessage();
			// When an error has been discoverd, change the text color to red
			spinFrame.ta.setForeground(Color.red);
			spinFrame.ta.append(errorMessage + "\n");
		}
		// Then loop through the Promela Code and display it for reference
		verifyTable = readVerifyFile();
		spinFrame.ta.append("\n");
		for (int l = 0; l< verifyTable.size(); l++)
		{
			String verifyTemp = (String) verifyTable.elementAt(l);
			spinFrame.ta.append((l+1)+" "+verifyTemp + "\n");
		}
	}
	// If no errors have been detected while running Spin, then proceed with the
	// analysis
	else
	{
		spinFrame.ta.append(Tab + "Analysis Completed... Evaluating Analysis...\n");
	}
	if (!foundError)
	{
		// Run the batch file gogcc.bat
		// The command is (gcc -DEBITSTATE -DSAFETY -o pan pan.c)
		// This will create an executable file, pan.exe from the file pan.c by
		// invoking the C compiler gcc.  This is the command that checks for deadlocks.
		try
		{
			System.out.println("Running GCC...");
			Process tempObject = Runtime.getRuntime().exec(workingDirectory + "gogcc.bat");
			// Wait for the process to complete
			tempObject.waitFor();
			// Force garbage collection to free up memory
			System.runFinalization();
		}
		catch (IOException exc)
		{
			System.out.println("Error running GCC");
		}
		catch (InterruptedException exc)
		{
			System.out.println("Error running GCC");
		}
	}
	if (!foundError)
	{
		// Run the batch file pantext.bat
		// The command is (redir -o newpan.txt pan.exe)
		// This will execute the file pan.exe and redirect the output to the file
		// newpan.txt.  This file will contain any information from the analysis
		try
		{
			System.out.println("Getting Output of Analysis...");
			Process tempObject = Runtime.getRuntime().exec(workingDirectory + "pantext.bat");
			// Wait for the process to complete
			tempObject.waitFor();
			// Force garbage collection to free up memory
			System.runFinalization();
		}
		catch (IOException exc)
		{
			System.out.println("Error Getting Output of Analysis");
		}
		catch (InterruptedException exc)
		{
			System.out.println("Error Getting Output of Analysis");
		}
	}
	// Display the output from the analysis
	spinFrame.ta.append("\n");
	spinFrame.ta.append(Tab + "********** OUTPUT FROM DEADLOCK CHECK **********\n\n");
	// If the file exists...
	if (panTextFile.exists())
	{
		System.out.println("Reading newpan.txt");
		// Read the file into a vector so it can be displayed
		Vector panTable = readPanFile();
		// A variable used to detect an error condition exists
		boolean conversationError = false;
		String panMessage = null;
		String deadlockMessage = null;
		// Loop through the vector of messages looking for an invalid end state message
		for (int k = 0; k < panTable.size(); k++)
		{
			// Read the text at the kth element
			PanTable panTemp = (PanTable) panTable.elementAt(k);
			String spinMessage = panTemp.getSpinMessage();
			// If the message is long enough to be a possible invalid end state...
			if (spinMessage.length() > 20)
			{
				// Cut out the part of the message that would have the correct error msg
				panMessage = spinMessage.substring(0, 21);
			}
			// If the error message equals this...
			if (panMessage.equals("pan: invalid endstate"))
			{
				// Then an invalid endstate has been detected, and thus deadlock exists
				conversationError = true;
				deadlockMessage = spinMessage;
			}
		}
		// If we have a deadlock error
		if (conversationError)
		{
			// Change the text color to red and display the error message
			spinFrame.ta.setForeground(Color.red);
			spinFrame.ta.append(Tab + Tab + "CONVERSATION IS DEADLOCKED!!!\n");
		}
		// If no error, display the appropriate message
		else
		{
			spinFrame.ta.append(Tab + Tab + "CONVERSATION IS NOT DEADLOCKED!!!\n");
		}
	}
	// Something went wrong executing the check.
	// A likely problem is the system Path statement is not correct, so
	// Spin and GCC cannot run properly
	else
	{
		spinFrame.ta.append(Tab + "Error executing Deadlock Check\n\n");
	}
	if (!foundError)
	{
		// Run the batch file gogcc2.bat
		// The command is (gcc -DNP -DBITSTATE -o pan2 pan.c)
		// This will create an executable file pan2.exe by invoking the gcc compiler
		// on the pan.c file.  This is the command that checks for non-progress states.
		try
		{
			System.out.println("Running GCC2...");
			Process tempObject = Runtime.getRuntime().exec(workingDirectory + "gogcc2.bat");
			// Wait for the process to complete
			tempObject.waitFor();
			// Force garbage collection to free up memory
			System.runFinalization();
		}
		catch (IOException exc)
		{
			System.out.println("Error running GCC2");
		}
		catch (InterruptedException exc)
		{
			System.out.println("Error running GCC2");
		}
	}
	if (!foundError)
	{
		// Run the batch file pantext2.bat
		// The command is (redir -o newpan2.txt pan2.exe -l)
		// This will execute the file pan2 and redirect the output to the file
		// newpan2.txt.  This is the analysis of non-progress states
		try
		{
			System.out.println("Getting Output of Analysis2...");
			Process tempObject = Runtime.getRuntime().exec(workingDirectory + "pantext2.bat");
			// Wait for the process to complete
			tempObject.waitFor();
			// Force garbage collection to free up memory
			System.runFinalization();
		}
		catch (IOException exc)
		{
			System.out.println("Error Getting Output of Analysis2");
		}
		catch (InterruptedException exc)
		{
			System.out.println("Error Getting Output of Analysis2");
		}
	}
	// Display the output from the non progress state check
	spinFrame.ta.append("\n");
	spinFrame.ta.append(Tab + "********** OUTPUT FROM PROGRESS CHECK **********\n\n");
	Vector tempTable = new Vector();
	StateTable temp = null;
	boolean messageWritten = false;
	if (panTextFile2.exists())
	{
		System.out.println("Reading newpan2.txt");
		// Read the text file that contains the messages from the non progress state
		// analysis
		Vector panTable2 = readPan2File();
		// While messages exist in the vector...
		for (int k = 0; k < panTable2.size(); k++)
		{
			// Variables used to detect the error message
			boolean progressError = false;
			String panMessage = null;
			String progressMessage = null;
			// Read the message at the kth element of the vector
			PanTable panTemp = (PanTable) panTable2.elementAt(k);
			String spinMessage = panTemp.getSpinMessage();
			// If the message is long enough to be a candidate error message...
			if (spinMessage.length() > 7)
			{
				// Cut out the first 5 characters of the message
				panMessage = spinMessage.substring(0, 5);
			}
			// If the cut out message equals a Tab and the word line...
			if (panMessage.equals(Tab + "line"))
			{
				// Then Spin has detected a non progress state and we must get the rest
				// of the error message
				progressError = true;
				progressMessage = spinMessage;
			}

			// If Spin detected a non progress state
			if (progressError)
			{
				if (!messageWritten)
				{
					// Change the text color to red and write an appropriate message
					spinFrame.ta.setForeground(Color.red);
					spinFrame.ta.append(Tab + Tab + "CONVERSATION HAS AN UNUSED STATE!!!\n");
					messageWritten = true;
				}
				// Parse every word of the error message to find the conversation and state
				// where the error occured
				StringTokenizer t = new StringTokenizer(progressMessage, " ");
				String t1 = t.nextToken();
				String t2 = t.nextToken();
				// This word of the message contains the line number of the Promela code
				// where the error occurred.  We must go to the Promela code to get some
				// of the information needed to pinpoint the location of the error.
				int lineNum = Format.atoi(t2);
				// Read in the verify file, which is the Promela code
				verifyTable = readVerifyFile();
				// Get the line of code where the error occurred
				String code = (String) verifyTable.elementAt(lineNum);
				// Variables used to find the needed information
				String state = null;
				boolean convFound = false;
				boolean participantFound = false;
				// Look at the line above where the error occurred in the Promela code
				int j = 1;
				// Keep moving upwards in the code until the state information is found
				while (convFound == false)
				{
					// Get the line of code pointed to here
					String code2 = (String) verifyTable.elementAt(lineNum - j);
					// Parse each word of the line of code
					StringTokenizer token = new StringTokenizer(code2, " ");
					String code3 = token.nextToken();
					// If the first word of the line is do
					// Then the line above this line contains the information we need
					if (code3.equals(Tab + Tab + "do"))
					{
						// Get the line above the line we are pointing to
						String code4 = (String) verifyTable.elementAt(lineNum - j - 1);
						convFound = true;
						// Cut out of this line the name of the conversation state
						state = code4.substring(9, code4.length() - 1);
					}
					// Keep moving up the code until the state information is found
					j++;
				}
				// Variable used to find the conversation name
				int m = 1;
				// Keep looping up through the code until the conversation name is found
				while (participantFound == false)
				{
					// Get the line of code at the line of Promela code pointed to
					String code2 = (String) verifyTable.elementAt(lineNum - m);
					String code3 = null;
					// Parse each word looking for the conversation name
					StringTokenizer token3 = new StringTokenizer(code2, " ");
					// If we are not on a blank line...
					if (token3.hasMoreTokens())
					{
						// Look at the first word of the line
						code3 = token3.nextToken();
						// If the word equals the word proctype, then the conversation
						// info we need is on this line
						if (code3.equals("proctype"))
						{
							// Get the next word from the code
							String code4 = token3.nextToken();
							// Cut off the closing () on the word
							String temp1 = code4.substring(0, code4.length() - 2);
							// Separate the conversation name from the participant name
							// by cuttin off the last 9 characters.  The participant name
							// will either be Initiator or Responder.
							String participant = temp1.substring(temp1.length() - 9, temp1.length());
							String conversation = temp1.substring(0, temp1.length() - 9);
							// Stop looping, we've found the right line of code
							participantFound = true;
							// Create an error message that will be passed to agentTool
							// for highlighting the exact location of the error
							ATerror errorMsg = new ATerror(ATerror.PROGRESS, conversation, participant, state, "null");
							// Create a new instance of a state that is not reached
							temp = new StateTable(conversation + ";" + participant + ";" + state + ";" + "null;null;null;null");
							// Variable used to detect if the bad state is already in
							// the vector of bad states
							boolean foundEntry = false;
							// Search the bad state vector to see if the current bad
							// state is already in there
							for (int i = 0; i < tempTable.size(); i++)
							{
								StateTable workTable = (StateTable) tempTable.elementAt(i);
								if (workTable.getConvName().equals(conversation) && workTable.getPartName().equals(participant) && workTable.getCurrentState().equals(state))
								{
									// The entry is already in the bad state table
									foundEntry = true;
								}
							}
							// If we did not find the bad state in the table...
							if (!foundEntry)
							{
								// Then add it to the bad state table
								tempTable.addElement(temp);
								// ErrorVector is the vector of error messages that gets passed
								// back to agentTool
								errorVector.addElement(errorMsg);
							}
						}
					}
					// Keep moving up through the code
					m++;
				}
			}
		}
		// If any unused states were detected...
		if (tempTable.size() > 0)
		{
			// Change the text color to red and display the appropriate messages
			//spinFrame.ta.setForeground(Color.red);
			for (int i = 0; i < tempTable.size(); i++)
			{
				StateTable temp2 = (StateTable) tempTable.elementAt(i);
				spinFrame.ta.append("\n" + Tab + Tab + "THE FOLLOWING CONVERSATION STATE IS NOT ENTERED INTO: " + "\n");
				spinFrame.ta.append(Tab + Tab + "Conversation Name = " + temp2.getConvName() + "\n");
				spinFrame.ta.append(Tab + Tab + "Participant Name = " + temp2.getPartName() + "\n");
				spinFrame.ta.append(Tab + Tab + "State = " + temp2.getCurrentState() + "\n");
			}
		}
		// No unused states were detected
		else
		{
			spinFrame.ta.append(Tab + Tab + "CONVERSATION DOES NOT HAVE UNUSED STATES!!!\n");
		}
	}

	// Something happened preventing Spin from running
	// Check the path statement
	else
	{
		spinFrame.ta.append(Tab + "Error executing Progress State Check\n\n");
	}
	if (!foundError)
	{
		// If Spin detected an error, a trace file is created so the error can be traced
		if (panTraceFile.exists())
		{
			// Run the batch file trace.bat
			// The command is (redir -o trace.txt spin -t -c verify)
			// This will run Spin against the trace file created by Spin while
			// analyzing the file verify.  The trace is then redirected to the file
			// trace.txt for later display
			try
			{
				System.out.println("Getting Simulation Trace...");
				Process tempObject = Runtime.getRuntime().exec(workingDirectory + "trace.bat");
				// Wait for the process to complete
				tempObject.waitFor();
				// Force garbage collection to free up memory
				System.runFinalization();
			}
			catch (IOException exc)
			{
				System.out.println("Error Getting Simulation Trace");
			}
			catch (InterruptedException exc)
			{
				System.out.println("Error Getting Simulation Trace");
			}
		}
	}
	// Display the output from the simulation run
	spinFrame.ta.append("\n");
	spinFrame.ta.append(Tab + "********** OUTPUT FROM SIMULATED RUN **********\n\n");
	// Variables used to find deadlock error messages
	String traceMessage = null;
	String detector = null;
	String conversationMessage = null;
	// A vector to hold all the deadlock error messages
	Vector conversationErrors = new Vector();
	if (panTraceFile.exists())
	{
		if (traceTextFile.exists())
		{
			System.out.println("Reading trace.txt");
			// Read the trace file into a vector
			traceTable = readTraceFile();
			// Loop through the vector looking for error messages. The key is that any
			// error message will end with the line number of the error in the Promela
			// code surrounded by parenthesis.  That is what we are looking for.
			for (int k = 0; k < traceTable.size(); k++)
			{
				// Get the error message at the kth element in the vector
				TraceTable traceTemp = (TraceTable) traceTable.elementAt(k);
				traceMessage = traceTemp.getTraceMessage();
				// Observe the last character and see if it is the closing parenthesis
				detector = traceMessage.substring(traceMessage.length() - 1, traceMessage.length());
				// If it is a closing parenthesis...
				if (detector.equals(")"))
				{
					// Grab this line of the messages and add it to the error vector
					conversationMessage = traceMessage;
					conversationErrors.addElement(conversationMessage);
				}
				// Display the entire trace in the text window so it can be evaluated
				spinFrame.ta.append(Tab + traceMessage + "\n");
			}
		}
		// Something happened with Spin
		// Check the path statement
		else
		{
			spinFrame.ta.append(Tab + "Error executing Simulation Run\n\n");
		}
		// Display the deadlock information that contains the conversation info 
		spinFrame.ta.append("\n" + Tab + "********** DETAILED DEADLOCK INFORMATION **********\n");
		// While there are error messages in the error vector...
		for (int i = 0; i < conversationErrors.size(); i++)
		{
			// Get the error message at the ith element of the vector
			String line = (String) conversationErrors.elementAt(i);
			// Parse each word of the message
			StringTokenizer t = new StringTokenizer(line, " ");
			// Skip over to the 5th word and get the line number of the line of code
			// that generated the error
			String t1 = t.nextToken();
			String t2 = t.nextToken();
			String t3 = t.nextToken();
			String t4 = t.nextToken();
			String t5 = t.nextToken();
			// Convert the string to int
			int lineNum = Format.atoi(t5);
			// Read in the source code 
			verifyTable = readVerifyFile();
			// Get the line of source code that generated the error
			String code = (String) verifyTable.elementAt(lineNum);
			// Break the line up based on the receive mark, ?
			// A conversation can only deadlock while waiting to receive a message
			StringTokenizer token1 = new StringTokenizer(code, "?");
			// Get the first section of the line of code, all the way up to the ?
			String trash1 = token1.nextToken();
			// Get the second section of the line of code, minus the ?
			String transition = "null";
			if (trash1.length() < code.length())
			{
				String trash2 = code.substring(trash1.length() + 1, code.length());
				// Parse each word of the second section of code
				StringTokenizer token2 = new StringTokenizer(trash2, " ");
				// Get the first word in the second section of code
				// This is the transition name
				transition = token2.nextToken();
				// Cut out excess spaces and colons before the conversation name
				//String conversation = trash1.substring(5, trash1.length());
			}
			// Variables used to get the participant's name
			String state = null;
			boolean convFound = false;
			boolean participantFound = false;
			int j = 1;
			// Loop through the Promela code until the conversation name is found
			while (convFound == false)
			{
				// Get the line of code above the line of code that caused the error
				String code2 = (String) verifyTable.elementAt(lineNum - j);
				// Parse every word of the line
				StringTokenizer token = new StringTokenizer(code2, " ");
				// Get the first word of the line
				String code3 = token.nextToken();
				// If the first word is do, then the next line above this is the line
				// we want
				if (code3.equals(Tab + Tab + "do"))
				{
					// Get the line of code we want
					String code4 = (String) verifyTable.elementAt(lineNum - j - 1);
					convFound = true;
					// Every state begins with the word progress
					// Cut off the word progress and you're left with the state name
					// Every state ends with a colon that must also be cut off
					state = code4.substring(9, code4.length() - 1);
				}
				// Keep looping until the state name is found
				j++;
			}
			// Loop until the participant name is found
			int m = 1;
			while (participantFound == false)
			{
				// Get the line of code from the source code above the line that caused
				// the error
				String code2 = (String) verifyTable.elementAt(lineNum - m);
				String code3 = null;
				// Parse every word
				StringTokenizer token3 = new StringTokenizer(code2, " ");
				// If the line is not blank...
				if (token3.hasMoreTokens())
				{
					// Get the first word
					code3 = token3.nextToken();
					// If the word equals proctype...
					if (code3.equals("proctype"))
					{
						// Get the next word
						String code4 = token3.nextToken();
						// We've found the line we need
						participantFound = true;
						// Cut out the conversation name from this word and we'll have
						// the participant name
						String conversation = code4.substring(0, code4.length() - 11);
						String participant = code4.substring(conversation.length(), code4.length() - 2);
						// Create an error message that will be passed back to agentTool
						ATerror errorMsg = new ATerror(ATerror.DEADLOCK, conversation, participant, state, transition);
						// Add the error message to the vector that will be passed
						// back to agentTool
						errorVector.addElement(errorMsg);
						// Change the text color to red and display the appropriate
						// information
						spinFrame.ta.setForeground(Color.red);
						spinFrame.ta.append("\n" + Tab + Tab + "DEADLOCK CONDITION EXISTS IN THE FOLLOWING CONVERSATION: " + "\n");
						spinFrame.ta.append(Tab + Tab + "Conversation Name = " + conversation + "\n");
						spinFrame.ta.append(Tab + Tab + "Participant Name = " + participant + "\n");
						spinFrame.ta.append(Tab + Tab + "Current State = " + state + "\n");
						spinFrame.ta.append(Tab + Tab + "State Transition = " + transition + "\n");
					}
				}
				// Keep looping until the right line of code is found
				m++;
			}
		}
	}
	// No trace file exists because there was no reason to have one.
	// This is not an error condition
	else
	{
		spinFrame.ta.append(Tab + "No trace available\n");
	}
}
/**
 * Check the operating system and set the user directory and file names based on the OS
 * @param nothing
 * @return nothing
 * @throws nothing
 */
public static void setProperties()
{
	// Get the user directory, which is the directory the application is installed in
	String userDir = System.getProperty("user.dir");
	// Get the operating system name for use in naming files and executing commands
	String osName = System.getProperty("os.name");
	// Java requires two slashes together to get one slash into a string
	String slash = "\\";
	// The working directory for all files is the user directory
	workingDirectory = userDir + slash;
	// This is the input state table file.  It is only used if input is through a file.
	tableFile = new File(workingDirectory + "statetbl.txt");
	// This file captures the messages from deadlock analysis
	panTextFile = new File(workingDirectory + "newpan.txt");
	// This file captures the messages from non progress state analysis
	panTextFile2 = new File(workingDirectory + "newpan2.txt");
	// This file contains the error trace if an error exists
	traceTextFile = new File(workingDirectory + "trace.txt");
	// This file is the executable created when running GCC against Spin generated C files
	panFile = new File(workingDirectory + "pan.exe");
	// This is the Promela source code created by this program automatically
	verifyFile = new File(workingDirectory + "goverify");
	// This is the C file created by Spin when running Spin against the Promela file verify
	panCFile = new File(workingDirectory + "pan.c");
	// This is the file that captures any errors while executing Spin
	errorFile = new File(workingDirectory + "error");
	System.out.println("OS Name = " + osName);
	// If the current OS is Windows NT...
	if (osName.equals("Windows NT"))
	{
		// The file created by Spin for a trace will be called verify.tra
		panTraceFile = new File(workingDirectory + "goverify.tra");
	}
	// If the current OS is anything else...
	else
	{
		// The file created by Spin for a trace will  be called verify.trail
		panTraceFile = new File(workingDirectory + "goverify.trail");
	}
}
/**
 * Sort the vector of conversation transitions passed by agentTool
 * The conversation transitions must be in contiguous order for the automatic build routine to work
 * @param The unsorted vector of conversation transitions
 * @return The sorted vector of conversation transitions
 * @throws nothing
 */
public static Vector sortVector(Vector v)
{
	// Variables used to sort the vector
	String line = null;
	String tempLine = null;
	Vector stateTable = new Vector();
	StateTable statetbl = null;
	// While there are more elements in the vector...
	while (v.size() > 0)
	{
		// Create a temporary working vector
		Vector tempTable = new Vector();
		// Read the first element in the unsorted vector
		line = (String) v.elementAt(0);
		// Convert the string into a StateTable object
		statetbl = new StateTable(line);
		// Get the conversation name and the participant name
		String tempConvName = statetbl.getConvName();
		String tempPartName = statetbl.getPartName();
		// Get the process name by concatinating the conversation name and the participant name
		String tempProcessName = statetbl.getConvName()+statetbl.getPartName();
		// Now start at the beginning of the unsorted vector and pull out all the matching entries
		// that have the same process name (conversation name + participant name)
		for (int j = 0; j < v.size(); j++)
		{
			// Get the current line of the unsorted vector
			tempLine = (String) v.elementAt(j);
			// Convert the string to a State Table object
			statetbl = new StateTable(tempLine);
			// Get the process name by concatinating the conversation name and the participant name
			String tmpProcessName = statetbl.getConvName()+statetbl.getPartName();
			// If the two process names are equal, add found object to the temporary table
			if (tempProcessName.equals(tmpProcessName))
			{
				tempTable.addElement(statetbl);
			}
		}
		// Start back at the beginning of the unsorted vector and pull out all the matching entries
		for (int l = 0; l < v.size(); l++)
		{
			// Get the entry pointed to
			line = (String) v.elementAt(l);
			// Convert the string to a State Table object
			statetbl = new StateTable(line);
			// Get the process name by concatinating the conversation and participant names
			String tmpProcessName = statetbl.getConvName()+statetbl.getPartName();
			// If the names are equal, delete the entry from the unsorted vector
			if (tempProcessName.equals(tmpProcessName))
			{
				v.removeElementAt(l);
				l--;
			}
		}
		// Pull out all Start State transitions and put them in a separate working vector
		// Go through the temp vector...
		for (int m = 0; m < tempTable.size(); m++)
		{
			// Pull out the entry pointed to 
			statetbl = (StateTable) tempTable.elementAt(m);
			// Get the current state name
			String tmpCurrentState = statetbl.getCurrentState();
			// If it is the Start State...
			if (tmpCurrentState.equals("StartState"))
			{
				// Add it to the temp state table vector first
				stateTable.addElement(statetbl);
				// Then remove it from the temp vector
				tempTable.removeElementAt(m);
				// There could be more than one transition out of the Start State
				// So decrement the counter and look for more transitions out of the Start State
				m--;
			}
		}
		// Now go through the temp vector and the temp start state vector and copy the contents 
		// into the permanent state table vector and delete the entries from the start state temp vector
		while (tempTable.size() > 0)
		{
			// Get the first entry of the temporary vector
			statetbl = (StateTable) tempTable.elementAt(0);
			// Get the current state
			String tempState = statetbl.getCurrentState();
			// Add the element to the state table temporary vector
			stateTable.addElement(statetbl);
			// Remove the copied entry from the temp vector
			tempTable.removeElementAt(0);
			// Continue through the temp vector pulling out any more entries that match the current
			// state.  We want all state entries to be contiguous in the new vector
			for (int o = 0; o < tempTable.size(); o++)
			{
				// Get the first entry in the temp vector
				statetbl = (StateTable) tempTable.elementAt(o);
				// Get the current state
				String tempState2 = statetbl.getCurrentState();
				// If this is a matching state...
				if (tempState2.equals(tempState))
				{
					// Add the element to the state table temp vector
					stateTable.addElement(statetbl);
					// And remove the entry from the temp vector
					tempTable.removeElementAt(o);
					// Since we removed the entry, the counter must be decremented
					o--;
				}
			}
		}
		statetbl = new StateTable("null;null;null;null;null;null;null");
		// The conversation name is the same as the other transitions
		statetbl.setConvName(tempConvName);
		// The participant name is the same as the other transitions
		statetbl.setPartName(tempPartName);
		// This is an End State
		statetbl.setCurrentState("EndState");
		// An End State does not have any transitions out of it, so set all the other values to null
		statetbl.setGuardCondition("null");
		statetbl.setNextState("null");
		statetbl.setRecMessage("null");
		statetbl.setTransMessage("null");
		stateTable.addElement(statetbl);
	}
	// Return the sorted state table vector
	return stateTable;
}
/**
 * This is the method called by agentTool to start verifying conversations
 * @param The unsorted vector of conversations and conversation transitions
 * @return A vector of ATerror entries that pinpoint where deadlock and non progress errors occured
 * @throws nothing
 */
public static Vector verify(Vector v, int buffer)
{
	// Set system properties used in this program
	setProperties();
	// The unsorted vector must be sorted according to conversation name, participant name, and current
	// state
	Vector sortedVector = sortVector(v);
	// Show the message window
	spinFrame.show();
	// Display that we are beginning to verify conversations
	spinFrame.ta.setText("");
	spinFrame.ta.append("PLEASE STAND BY... TESTING CONVERSATIONS...\n");
	// Write status messages to the console so you can verify the process is running
	System.out.println("Reading State Table...\n");
	// Create the output file, verify, from the sorted state table vector
	writeOutput(sortedVector,buffer);
	System.out.println("Running Spin...\n");
	errorVector = new Vector();
	// Run Spin against the Promela code and analyze it
	runSpin();
	// Testing is completed
	spinFrame.ta.append("\n");
	spinFrame.ta.append("********** TESTING COMPLETED **********\n\n");
	// Return to agentTool a vector of ATerror messages that pinpoint the exact location
	// of deadlock conditions and non progress states
	for (int i = 0; i < errorVector.size(); i++)
	{
		ATerror temp = (ATerror)errorVector.elementAt(i);
		System.out.println("ATError # " + i);
		System.out.println(temp.getConvName()+" "+temp.getParticipantName()+" "+
			temp.getType()+" "+temp.getCurrentState()+" "+temp.getStateTransition());
		System.out.println(" ");
	}
	return errorVector;
}
/**
 * Create the Promela source code from the state table
 * @param The vector of sorted conversations and transitions
 * @return nothing
 * @throws IOException
 */
public static void writeOutput(Vector stateTble, int buffer)
{
	// Loop variables
	int i, j;
	// Process variables
	String tempProcessName = null;
	String tempCurrentState = null;
	StateTable stateTemp = null;
	Vector mtypeVector = new Vector();
	Vector channelVector = new Vector();
	String channelName = "bus";
	int processNameCntr = 0;
	int channelIndex = 0;
	boolean timeToWrite = true;
	boolean makingDo = false;
	boolean foundRecMessage = false;
	boolean foundGuardCondition = false;
	boolean foundTransMessage = false;
	boolean wroteTransmit = false;
	boolean first = true;
	// If the verify file, verify exists, delete it
	// This way we are ensured to start over fresh with each verification
	if (verifyFile.exists())
	{
		boolean deleted = verifyFile.delete();
	}
	// File stream object
	DataOutputStream output = null;
	// Output formatting variables
	String processName, currentState, recMessage, guardCondition, transMessage, nextState;
	// Constants
	final String outputFile = workingDirectory + "Goverify";
	// Open the Verify output file
	try
	{
		output = new DataOutputStream(new FileOutputStream(outputFile));
	}
	// Catch any IO exceptions
	catch (IOException e)
	{
		System.err.println("File error.\n" + e.toString());
		System.out.println("OutputFile = "+outputFile);
	}
	// Build vector of mtypes and channels from statetble 
	for (int k = 0; k < stateTble.size(); k++)
	{
		// Variables used
		foundRecMessage = false;
		foundGuardCondition = false;
		foundTransMessage = false;
		// Get the first entry in the state table vector
		stateTemp = (StateTable) stateTble.elementAt(k);
		// Get the process name by concatinating the conversation name and participant name
		processName = stateTemp.getConvName() + stateTemp.getPartName();
		// Get the received message... equals null if no entry
		recMessage = stateTemp.getRecMessage();
		// Get the guard condition... equals null if no entry
		guardCondition = stateTemp.getGuardCondition();
		// Get the transmitted message... equals null if no entry
		transMessage = stateTemp.getTransMessage();
		// If this is the first time we have encountered this process name...
		if (!processName.equals(tempProcessName))
		{
			// We create channels for every pair of conversations
			// This is why we require conversations to be matched.
			// Time to write becomes true for every other process name
			if (timeToWrite)
			{
				// Create a channel name that is the same as the conversation name
				channelName = stateTemp.getConvName();
				// Add the channel name to a vector of channel names for later declaration
				channelVector.addElement(channelName);
				// Reset the time to write flag
				timeToWrite = false;
			}
			else
			{
				// Set the time to write flag
				timeToWrite = true;
			}
		}
		// Create the message type, mtype declarations
		// Look at every entry in the mtype vector and do not add any repeat entries
		// Any received message, guard condition, or transmitted message is an mtype declaration
		tempProcessName = processName;
		for (int l = 0; l < mtypeVector.size(); l++)
		{
			// Get the pointed to mtype entry
			String tempMType = (String) mtypeVector.elementAt(l);
			// If the received message is already in the vector, take note
			if (recMessage.equals(tempMType))
			{
				foundRecMessage = true;
			}
			// If the guard condition is already in the vector, take note
			if (guardCondition.equals(tempMType))
			{
				foundGuardCondition = true;
			}
			// If the transmitted message is already in the vector, take note
			if (transMessage.equals(tempMType))
			{
				foundTransMessage = true;
			}
		}
		// If the received message is not in the vector already, and not null...
		if (!foundRecMessage && !recMessage.equals("null"))
		{
			// add the received message to the vector
			mtypeVector.addElement(recMessage);
		}
		// If the guard condition message is not in the vector already, and not null...
		if (!foundGuardCondition && !guardCondition.equals("null"))
		{
			// add the guard condition message to the vector
			mtypeVector.addElement(guardCondition);
		}
		// If the transmitted message is not in the vector already, and not null...
		if (!foundTransMessage && !transMessage.equals("null"))
		{
			// add the transmitted message to the vector
			mtypeVector.addElement(transMessage);
		}
		try
		{
			// Flush the output buffer to ensure nothing is in it
			output.flush();
		}
		// Catch any IO errors
		catch (IOException io)
		{
			System.out.println("OUTPUT FLUSH ERROR");
		}
	}
	tempProcessName = null;
	// Start writing the mtype declarations
	Format.print(output, "%s", "mtype = { ");
	// iterate through the mtypeVector and write the mtypes
	for (int k = 0; k < mtypeVector.size(); k++)
	{
		String tempMType = (String) mtypeVector.elementAt(k);
		// Write the mtype definitions
		Format.print(output, "%s", tempMType);
		// Mtype declarations are separated by commas, but not after the last entry
		if (k < mtypeVector.size() - 1)
		{
			Format.print(output, "%s", ", ");
		}
		// Flush the buffer again to ensure it is empty
		try
		{
			output.flush();
		}
		// Catch any IO errors
		catch (IOException io)
		{
			System.out.println("OUTPUT FLUSH ERROR");
		}
	}
	// Write the closing brace of the mtype declarations
	Format.print(output, "%s", " };\n\n");
	// Write the channel declaration
	for (int m = 0; m < channelVector.size(); m++)
	{
		Format.print(output, "%s", "chan " + (String) channelVector.elementAt(m) + "1 = ["+buffer+"] of {mtype};\n");
		Format.print(output, "%s", "chan " + (String) channelVector.elementAt(m) + "2 = ["+buffer+"] of {mtype};\n");
	}
	Format.print(output, "%s", "\n");
	// Time to write must be true the first time around 
	timeToWrite = true;
	// Append each channel name with an index
	channelIndex = 0;
	//Iterate through the State table and write the info
	for (int k = 0; k < stateTble.size(); k++)
	{
		// Get the state table information at the kth element
		stateTemp = (StateTable) stateTble.elementAt(k);
		// Get the process name by concatenating the conversation name and participant name
		processName = stateTemp.getConvName() + stateTemp.getPartName();
		if (stateTemp.getPartName().equals("Initiator"))
		{
			first = true;
		}
		else
		{
			first = false;
		}
		// Get the current state
		currentState = stateTemp.getCurrentState();
		// Get the received message
		recMessage = stateTemp.getRecMessage();
		// Get the guard condition
		guardCondition = stateTemp.getGuardCondition();
		// Get the transmitted message
		transMessage = stateTemp.getTransMessage();
		// Get the next state
		nextState = stateTemp.getNextState();
		// If this is a new current state and we're not making a Do loop, then close off the
		// current do loop
		if ((!currentState.equals(tempCurrentState)) && (makingDo))
		{
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", "od;\n");
			// No longer making a do loop
			makingDo = false;
		}
		// If this is a new process, close off the current proctype declaration
		if (!processName.equals(tempProcessName) && tempProcessName != null)
		{
			Format.print(output, "%s", "}\n\n");
		}
		// If this is a new process, start a new proctype declaration
		if (!processName.equals(tempProcessName))
		{
			Format.print(output, "%s", "proctype ");
			Format.print(output, "%s", processName);
			Format.print(output, "%s", "()\n");
			Format.print(output, "%s", "{\n");
			// Set channel variable to be used in this proctype
			if (timeToWrite)
			{
				channelName = (String) channelVector.elementAt(channelIndex);
				channelIndex++;
				timeToWrite = false;
			}
			else
			{
				timeToWrite = true;
			}
		}
		// If this is a new current state and we're not making a do loop make a new state declaration
		if ((!currentState.equals(tempCurrentState)) && (!makingDo))
		{
			Format.print(output, "%s", Tab);
			// Every state begins with the key word progress so we can check for non progress states
			Format.print(output, "%s", "progress" + currentState);
			Format.print(output, "%s", ":\n");
			Format.print(output, "%s", Tab + Tab);
			// The beginning of the do loop
			Format.print(output, "%s", "do\n");
			// Now we are making a do loop
			makingDo = true;
		}
		// If the received message is not equal to null, it contains something of value
		if (!recMessage.equals("null"))
		{
			if (!guardCondition.equals("null"))
			{
				Format.print(output, "%s", Tab + Tab);
				Format.print(output, "%s", ":: " + guardCondition + " -> ");
			}
			// Write the received message statement
			if (guardCondition.equals("null"))
			{
				Format.print(output, "%s", Tab + Tab);
				// Received messages are annotated by a ?
				Format.print(output, "%s", ":: ");
			}
			if (first)
			{
				Format.print(output, "%s", channelName + "1?");
			}
			else
			{
				Format.print(output, "%s", channelName + "2?");
			}				
			Format.print(output, "%s", recMessage);
			// A received message event could be followed by a send message event
			if (!transMessage.equals("null"))
			{
				// Write the transmit message line
				// Transmit messages are annotated by a !
				if (first)
				{
					Format.print(output, "%s", " -> " + channelName + "2!");
				}
				else
				{
					Format.print(output, "%s", " -> " + channelName + "1!");
				}
				Format.print(output, "%s", transMessage);
				Format.print(output, "%s", "; goto ");
				// Every state is preceded by the word progress
				Format.print(output, "%s", "progress" + nextState);
				Format.print(output, "%s", "\n");
				wroteTransmit = true;
			}
			// In this case, no transmit message is associated with a receive message
			else
			{
				// Simply goto the next state
				Format.print(output, "%s", " -> goto ");
				Format.print(output, "%s", "progress" + nextState);
				Format.print(output, "%s", "\n");
			}
		}
		// If a guard condition exists...
		if (!guardCondition.equals("null") && recMessage.equals("null"))
		{
			// Write the guard condition line
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", ":: ");
			Format.print(output, "%s", guardCondition);
			// If there is a transmit message associated with a guard condition...
			if (!transMessage.equals("null"))
			{
				// Write the transmit message line
				if (first)
				{
					Format.print(output, "%s", " -> " + channelName + "2!");
				}
				else
				{
					Format.print(output, "%s", " -> " + channelName + "1!");
				}
				Format.print(output, "%s", transMessage);
				// Then goto the next state
				Format.print(output, "%s", "; goto ");
				Format.print(output, "%s", "progress" + nextState);
				Format.print(output, "%s", "\n");
			}
			///*********** Changes  *********
			// Guard condition exists with no other messages
			else
			{
				Format.print(output, "%s", " -> goto ");
				Format.print(output, "%s", "progress" + nextState);
				Format.print(output, "%s", "\n");
			}
		}
		// If there is a transmit message without a guard condition and we didn't already
		// write this transmit message
		if ((!transMessage.equals("null")) && (guardCondition.equals("null")) && !wroteTransmit)
		{
			// Write the transmit message line
			Format.print(output, "%s", Tab + Tab);
			if (first)
			{
				Format.print(output, "%s", ":: " + channelName + "2!");
			}
			else
			{
				Format.print(output, "%s", ":: " + channelName + "1!");
			}
			Format.print(output, "%s", transMessage);
			// And goto the next state
			Format.print(output, "%s", " -> goto ");
			Format.print(output, "%s", "progress" + nextState);
			Format.print(output, "%s", "\n");
		}
		// If there is no guard condition with no received message and no transmit message
		if ((guardCondition.equals("null")) && (transMessage.equals("null")) && (recMessage.equals("null")) && (!nextState.equals("null")))
		{
			// Write a goto the next state
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", ":: goto ");
			Format.print(output, "%s", "progress" + nextState);
			Format.print(output, "%s", "\n");
		}
		// If we are at the End State...
		if (currentState.equals("EndState"))
		{
			// Exit the do loop with a break, which will fall to the closing brace of the conversation
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", ":: break");
			Format.print(output, "%s", "\n");
		}
		// If we are the last entry in the state table vector...
		if ((k == (stateTble.size() - 1)))
		{
			// Close off the last do loop and the closing brace for the proctype declarations
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", "od;\n");
			Format.print(output, "%s", "}\n\n");
		}
		// Temp variables to set up Process states
		tempProcessName = processName;
		tempCurrentState = currentState;
		wroteTransmit = false;
		// Flush the output buffer to ensure it is empty
		try
		{
			output.flush();
		}
		// Catch any IO errors
		catch (IOException io)
		{
			System.out.println("OUTPUT FLUSH ERROR");
		}
	}
	// ***End of State Table output ***//
	// Create the init procedure
	Format.print(output, "%s", "init\n");
	Format.print(output, "%s", "{\n");
	// Write the atomic statements
	Format.print(output, "%s", Tab);
	Format.print(output, "%s", "atomic\n");
	Format.print(output, "%s", Tab);
	Format.print(output, "%s", "{\n");
	// Loop through the state table and generate the run statements
	tempProcessName = "null";
	// While entries exist in the state table...
	for (int k = 0; k < stateTble.size(); k++)
	{
		// Get the entry at the kth element
		stateTemp = (StateTable) stateTble.elementAt(k);
		// Get the process name by concatenating the conversation and participant names
		processName = stateTemp.getConvName() + stateTemp.getPartName();
		// Get the current state
		currentState = stateTemp.getCurrentState();
		// Write the run statements
		// If the process name is a new one and this is not the first process name,
		// separate the run statements by semicolons
		if (!processName.equals(tempProcessName) && (!tempProcessName.equals("null")))
		{
			Format.print(output, "%s", ";\n");
		}
		// If this is a new process name...
		if (!processName.equals(tempProcessName))
		{
			// Write a run statement for this process name
			Format.print(output, "%s", Tab);
			Format.print(output, "%s", Tab);
			Format.print(output, "%s", "run ");
			Format.print(output, "%s", processName);
			Format.print(output, "%s", "()");
		}
		// Variable used to keep track of the process name we are working with
		tempProcessName = processName;
	}
	// All the run statements have been written, so write the closing braces
	Format.print(output, "%s", "\n");
	Format.print(output, "%s", Tab);
	Format.print(output, "%s", "}\n");
	Format.print(output, "%s", "}\n");
	// Flush the output buffer to ensure it is empty
	try
	{
		output.flush();
	}
	// Catch any IO errors
	catch (IOException io)
	{
		System.out.println("OUTPUT FLUSH ERROR");
	}
	// Close the Verify Output file
	try
	{
		output.flush();
		output.close();
		// Now we are ready to run Spin, so set this flag to true
		readyToRunSpin = true;
	}
	// Catch any IO errors
	catch (IOException io)
	{
		System.err.println("File not closed properly\n" + io.toString());
	}
}
}
