package Verify;

/**
 * A Class for verifying message sequence charts in the context of conversations
 * @version 1.0 6 Oct 1999
 * @author Capt Tim Lacey
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import java.io.*;
import java.util.*;
import java.awt.*;
import system.*;
public class VMSCRun
{
	public static Vector stateTable = null;
	public static Vector messageTable = null;
	public static Vector panTable = null;
	public static Vector traceTable = null;
	public static Vector errorTable = null;
	public static String workingDirectory = "";
	public static File tableFile;
	public static File MSCFile;
	public static File panTextFile;
	public static File panTextFile2;
	public static File traceTextFile;
	public static File panFile;
	public static File verifyFile;
	public static File panCFile;
	public static File panTraceFile;
	public static File errorFile;
	public static boolean readyToRunSpin = false;
	public static TextMessages spinFrame = new TextMessages();
	public static boolean foundError = false;
	public static final String Tab = "\u0009"; //Unicode Tab
/**
 * If the class is executed by itself, the main method gets run automatically
 * @param nothing
 * @return nothing
 * @throws nothing
 */
public static void main(String args[])
{
	// Set system properties used in this program
	setProperties();
	// Show the message window where all messages are displayed
	spinFrame.show();
	// Write the first message to the window.
	spinFrame.ta.append("PLEASE STAND BY... TESTING MESSAGE SEQUENCE...\n");
	// Write status messages to the console so you can verify the process is running
	System.out.println("Reading State Table...\n");
	// Read in the input file, statetbl.txt
	stateTable = readStateFile();
	// Read the message Table file, msctable.txt
	messageTable = readMSCFile();
	System.out.println("Creating Promela Source Code...\n");
	// Create the output file, verify, from the input files, statetbl.txt and msctable.txt
	writeOutput(stateTable,messageTable);
	// Run Spin against the Promela code and analyze it
	runSpin();
	// Testing is completed
	spinFrame.ta.append("\n");
	spinFrame.ta.append("********** TESTING COMPLETED **********\n\n");
}
/**
 * Read the error file generated when running Spin, one line at a time, and store in
 * a vector for use later
 * @param nothing
 * @return the vector that contains all the lines of the error file <tt>error</tt>
 * @throws IOException
 */
public static Vector readErrorFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line = null;
	// Create a vector that will hold every line of the input file
	Vector errorTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	ErrorTable errortbl = null;
	try
	{
		// Open an input stream from the error file
		inStream = new BufferedReader(new FileReader(errorFile));
		// read a line of text
		line = inStream.readLine();
		// if the line is not equal to null then an error has been generated
		if (line != null)
		{
			// set a global flag that will be used in the runSpin method
			foundError = true;
		}
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// if the actual length of the file is greater than 0, add the line to the
			// vector
			if (line.trim().length() > 0)
			{
				// get a new line
				errortbl = new ErrorTable(line);
				// add the line to the vector
				errorTable.addElement(errortbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector of error messages
	return errorTable;
}
/**
 * Read the file that contains the message sequence table information, <tt>msctable.txt</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>msctable.txt</tt>
 * @throws IOException
 */
public static Vector readMSCFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line;
	// Create a vector that will hold every line of the input file
	Vector mscTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	MSCTable msctbl = null;
	try
	{
		// Open an input stream from the message file
		inStream = new BufferedReader(new FileReader(MSCFile));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// if the actual length of the file is greater than 0, add the line to the
			// vector
			if (line.trim().length() > 0)
			{
				// get a new line
				msctbl = new MSCTable(line);
				// add the line to the vector
				mscTable.addElement(msctbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return mscTable;
}
/**
 * Read the file generated by analyzing the progress state checks, <tt>newpan2.txt</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>newpan2.txt</tt>
 * @throws IOException
 */
public static Vector readPan2File()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line = null;
	// Create a vector that will hold every line of the input file
	Vector panTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	PanTable pantbl = null;
	try
	{
		// Open an input stream from the text file
		inStream = new BufferedReader(new FileReader(panTextFile2));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// if the actual length of the file is greater than 0, add the line to the
			// vector
			if (line.trim().length() > 0)
			{
				// get a new line
				pantbl = new PanTable(line);
				// add the line to the vector
				panTable.addElement(pantbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return panTable;
}
/**
 * Read the file generated by analyzing the deadlock checks, <tt>newpan.txt</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>newpan.txt</tt>
 * @throws IOException
 */
public static Vector readPanFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line = null;
	// Create a vector that will hold every line of the input file
	Vector panTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	PanTable pantbl = null;
	try
	{
		// Open an input stream from the text file
		inStream = new BufferedReader(new FileReader(panTextFile));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// if the actual length of the file is greater than 0, add the line to the
			// vector
			if (line.trim().length() > 0)
			{
				// get a new line
				pantbl = new PanTable(line);
				// add the line to the vector
				panTable.addElement(pantbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return panTable;
}
/**
 * Read the file that contains the state table information, <tt>statetbl.txt</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>statetbl.txt</tt>
 * @throws IOException
 */
public static Vector readStateFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line;
	// Create a vector that will hold every line of the input file
	Vector stateTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	StateTable statetbl = null;
	try
	{
		// Open an input stream from the table file
		inStream = new BufferedReader(new FileReader(tableFile));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			// if the actual length of the file is greater than 0, add the line to the
			// vector
			if (line.trim().length() > 0)
			{
				// get a new line
				statetbl = new StateTable(line);
				// add the line to the vector
				stateTable.addElement(statetbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return stateTable;
}
/**
 * Read the vector that contains the state table information that is passed by
 * agentTool.  The vector contains lines of strings that must be parsed into
 * a vector of StateTable types
 * @param The vector containing the conversation state information needed
 * to verify a conversation
 * @return the vector that contains all the lines of the state vector
 * @throws nothing
 */
public static Vector readStateTable(Vector v)
{
	// Create a string variable that contains each line of the input vector
	String line = null;
	// Create a vector that will hold every line of the input vector
	Vector stateTable = new Vector();
	// Create a variable of the type that will be pulled from the vector
	StateTable statetbl = null;
	// while there are more elements in the input vector, write to the new vector
	for (int k = 0; k < v.size(); k++)
	{
		// get the line at the kth element
		line = (String) v.elementAt(k);
		// read the line into the proper format
		statetbl = new StateTable(line);
		// add the formated line to the new vector
		stateTable.addElement(statetbl);
	}
	// return the filled vector
	return stateTable;
}
/**
 * Read the file generated by creating a trace of an error, <tt>trace.txt</tt>
 * @param nothing
 * @return the vector that contains all the lines of the file <tt>trace.txt</tt>
 * @throws IOException
 */
public static Vector readTraceFile()
{
	// Create a buffer for reading in a file
	BufferedReader inStream;
	// Create a string variable that contains each line of the input file
	String line = null;
	// Create a vector that will hold every line of the input file
	Vector traceTable = new Vector();
	// Create a variable of the type that will be pulled from the file
	TraceTable tracetbl = null;
	try
	{
		// Open an input stream from the text file
		inStream = new BufferedReader(new FileReader(traceTextFile));
		// read a line of text
		line = inStream.readLine();
		// while the line is not equal to null, write the line to the vector
		// and read the next line of the input file
		while (line != null)
		{
			if (line.trim().length() > 0)
			{
				// get a new line
				tracetbl = new TraceTable(line);
				// add the line to the vector
				traceTable.addElement(tracetbl);
			}
			// read the next line from the input file
			line = inStream.readLine();
		}
		// close the input stream
		inStream.close();
	}
	// catch any IO errors
	catch (IOException e)
	{
		System.out.print("Error " + e);
	}
	// return the filled vector
	return traceTable;
}
/**
 * Run 8 batch files that will determine if conversations are valid or not
 * @param nothing
 * @return nothing
 * @throws IOException, InterruptedException
 */
public static void runSpin()
{
	// Set the text color to black as default, change to red if an error found
	spinFrame.ta.setForeground(Color.black);
	// if any of the files used in the previous analysis exists, delete them
	// so each analysis is a clean run
	if (panFile.exists())
	{
		boolean deleted = panFile.delete();
	}
	if (panTextFile.exists())
	{
		boolean deleted = panTextFile.delete();
	}
	if (panTextFile2.exists())
	{
		boolean deleted = panTextFile2.delete();
	}
	if (traceTextFile.exists())
	{
		boolean deleted = traceTextFile.delete();
	}
	if (panCFile.exists())
	{
		boolean deleted = panCFile.delete();
	}
	if (panTraceFile.exists())
	{
		boolean deleted = panTraceFile.delete();
	}
	if (errorFile.exists())
	{
		boolean deleted = errorFile.delete();
	}
	// do not run Spin until the Verify file has been created
	// just sit here in this loop until the flag becomes true
	while (!readyToRunSpin)
	{
	}
	try
	{
		System.out.println("Running Spin...");
		// Run the batch file gospin.bat
		// The command is (redir -o error spin -a verify)
		// This command executes spin on the file called verify and redirects the
		// output to a file called error
		Process tempObject = Runtime.getRuntime().exec(workingDirectory + "gospin.bat");
		// Wait until the process has completed before moving on
		tempObject.waitFor();
		// Force garbage collection to free up memory
		System.runFinalization();
	}
	catch (IOException exc)
	{
		System.out.println("Error running Spin");
	}
	catch (InterruptedException exc)
	{
		System.out.println("Error running Spin");
	}
	spinFrame.ta.append("\n");
	spinFrame.ta.append(Tab + "!!!!!!!!!! OUTPUT OF SPIN ANALYSIS !!!!!!!!!!\n\n");
	// If an error file exists, then something went wrong when running Spin
	// This would happen if the Promela code was constructed incorrectly
	if (errorFile.exists())
	{
		// Read the error file, error
		errorTable = readErrorFile();
	}
	// If there is indeed an error the loop through the vector of error messages and
	// print them in the message window
	if (foundError)
	{
		for (int k = 0; k < errorTable.size(); k++)
		{
			ErrorTable errorTemp = (ErrorTable) errorTable.elementAt(k);
			String errorMessage = errorTemp.getErrorMessage();
			// When an error has been discoverd, change the text color to red
			spinFrame.ta.setForeground(Color.red);
			spinFrame.ta.append(errorMessage + "\n");
		}
	}
	// If no errors have been detected while running Spin, then proceed with the
	// analysis
	else
	{
		spinFrame.ta.append(Tab + "Analysis Completed... Evaluating Analysis...\n");
	}
	if (!foundError)
	{
		// Run the batch file gogcc.bat
		// The command is (gcc -DEBITSTATE -o pan pan.c)
		// This will create an executable file, pan.exe from the file pan.c by
		// invoking the C compiler gcc.  This is the command that checks for deadlocks.
		try
		{
			System.out.println("Running GCC...");
			Process tempObject = Runtime.getRuntime().exec(workingDirectory + "gogcc3.bat");
			// Wait for the process to complete
			tempObject.waitFor();
			// Force garbage collection to free up memory
			System.runFinalization();
		}
		catch (IOException exc)
		{
			System.out.println("Error running GCC");
		}
		catch (InterruptedException exc)
		{
			System.out.println("Error running GCC");
		}
	}
	if (!foundError)
	{
		// Run the batch file pantext.bat
		// The command is (redir -o newpan.txt pan.exe -a)
		// This will execute the file pan.exe and redirect the output to the file
		// newpan.txt.  This file will contain any information from the analysis
		try
		{
			System.out.println("Getting Output of Analysis...");
			Process tempObject = Runtime.getRuntime().exec(workingDirectory + "pantext3.bat");
			// Wait for the process to complete
			tempObject.waitFor();
			// Force garbage collection to free up memory
			System.runFinalization();
		}
		catch (IOException exc)
		{
			System.out.println("Error Getting Output of Analysis");
		}
		catch (InterruptedException exc)
		{
			System.out.println("Error Getting Output of Analysis");
		}
	}
	if (!foundError)
	{
		// If Spin detected an error, a trace file is created so the error can be traced
		if (panTraceFile.exists())
		{
			// Run the batch file trace.bat
			// The command is (redir -o trace.txt spin -t -c verify)
			// This will run Spin against the trace file created by Spin while
			// analyzing the file verify.  The trace is then redirected to the file
			// trace.txt for later display
			try
			{
				System.out.println("Getting Simulation Trace...");
				Process tempObject = Runtime.getRuntime().exec(workingDirectory + "trace.bat");
				// Wait for the process to complete
				tempObject.waitFor();
				// Force garbage collection to free up memory
				System.runFinalization();
			}
			catch (IOException exc)
			{
				System.out.println("Error Getting Simulation Trace");
			}
			catch (InterruptedException exc)
			{
				System.out.println("Error Getting Simulation Trace");
			}
		}
	}
	// Display output from message sequence check
	spinFrame.ta.append("\n");
	spinFrame.ta.append(Tab + "********** OUTPUT FROM MESSAGE SEQUENCE CHECK **********\n\n");
	// If a text file was created...
	if (panTextFile.exists())
	{
		System.out.println("Reading newpan.txt");
		// Read into a vector the text file
		Vector panTable = readPanFile();
		// Variables used to search for an error condition (which is the trace we want)
		boolean validMSC = false;
		String panMessage = null;
		// While there are entries in the table...
		for (int k = 0; k < panTable.size(); k++)
		{
			// Get the kth entry
			PanTable panTemp = (PanTable) panTable.elementAt(k);
			// Get the message from this object
			String spinMessage = panTemp.getSpinMessage();
			// The message we are looking for is longer than 19 characters
			if (spinMessage.length() > 19)
			{
				// Get this subset of the message string
				panMessage = spinMessage.substring(0, 20);
			}
			// If this is the message we cut out...
			if (panMessage.equals("pan: claim violated!"))
			{
				// Then we verified the message trace exists (This is good!)
				validMSC = true;
			}
		}
		// If we have an error condition, meaning we found the trace...
		if (validMSC)
		{
			// Display appropriate message
			spinFrame.ta.append(Tab + Tab + "MESSAGE SEQUENCE IS VALID!!!\n");
		}
		// If we did NOT find the trace, then we really do have an error
		else
		{
			// Set text color to red and display appropriate message
			spinFrame.ta.setForeground(Color.red);
			spinFrame.ta.append(Tab + Tab + "MESSAGE SEQUENCE IS INVALID!!!\n");
		}
	}
	// There was an error checking for message sequence
	// Check out the system PATH variable
	else
	{
		spinFrame.ta.append(Tab + "Error executing Message Sequence Check\n\n");
	}
	// If the trace was found, display the trace in its entireity
	spinFrame.ta.append("\n");
	spinFrame.ta.append(Tab + "********** SEQUENCE TRACE IS AS FOLLOWS **********\n\n");
	// If a trail file exists...
	if (panTraceFile.exists())
	{
		// If a trace text file exists...
		if (traceTextFile.exists())
		{
			System.out.println("Reading trace.txt");
			// Read each line of the trace into a vector
			traceTable = readTraceFile();
			// Get the messages from the vector and dislay them in the text box
			for (int k = 0; k < traceTable.size(); k++)
			{
				TraceTable traceTemp = (TraceTable) traceTable.elementAt(k);
				String traceMessage = traceTemp.getTraceMessage();
				spinFrame.ta.append(Tab + traceMessage + "\n");
			}
		}
		// An error condition occured because the trace was not to be found
		else
		{
			spinFrame.ta.append(Tab + "Error executing Simulation Run\n\n");
		}
	}
	// No trace created, so in this case we have an error.  We should have a trail file
	// if Spin found the message sequence.
	else
	{
		spinFrame.ta.append(Tab + "Message Sequence Invalid... - No trace available\n");
	}
}
/**
 * Check the operating system and set the user directory and file names based on the OS
 * @param nothing
 * @return nothing
 * @throws nothing
 */
public static void setProperties()
{
	// Get the user directory, which is the directory the application is installed in
	String userDir = System.getProperty("user.dir");
	// Get the operating system name for use in naming files and executing commands
	String osName = System.getProperty("os.name");
	// Java requires two slashes together to get one slash into a string
	String slash = "\\";
	// The working directory for all files is the user directory
	workingDirectory = userDir + slash;
	// This is the input state table file.  It is only used if input is through a file.
	tableFile = new File(workingDirectory + "statetbl.txt");
	// This file captures the messages from deadlock analysis
	panTextFile = new File(workingDirectory + "newpan.txt");
	// This file captures the messages from non progress state analysis
	panTextFile2 = new File(workingDirectory + "newpan2.txt");
	// This file contains the error trace if an error exists
	traceTextFile = new File(workingDirectory + "trace.txt");
	// This file is the executable created when running GCC against Spin generated C files
	panFile = new File(workingDirectory + "pan.exe");
	// This is the Promela source code created by this program automatically
	verifyFile = new File(workingDirectory + "Goverify");
	// This is the C file created by Spin when running Spin against the Promela file verify
	panCFile = new File(workingDirectory + "pan.c");
	// This is the file that captures any errors while executing Spin
	errorFile = new File(workingDirectory + "error");
	// This is the file that is used for file input of message sequence charts (MSC)
	MSCFile = new File(workingDirectory + "msctable.txt");
	System.out.println("OS Name = "+osName);
	// If the current OS is Windows NT...
	if (osName.equals("Windows NT"))
	{
		// The file created by Spin for a trace will be called verify.tra
		panTraceFile = new File(workingDirectory + "Goverify.tra");
	}
	// If the current OS is anything else...
	else
	{
		// The file created by Spin for a trace will  be called verify.trail
		panTraceFile = new File(workingDirectory + "Goverify.trail");
	}
}
/**
 * Sort the vector of conversation transitions passed by agentTool
 * The conversation transitions must be in contiguous order for the automatic build routine to work
 * @param The unsorted vector of conversation transitions
 * @return The sorted vector of conversation transitions
 * @throws nothing
 */
public static Vector sortVector(Vector v)
{
	// Variables used to sort the vector
	String line = null;
	String tempLine = null;
	Vector stateTable = new Vector();
	StateTable statetbl = null;
	// While there are more elements in the vector...
	while (v.size() > 0)
	{
		// Create a temporary working vector
		Vector tempTable = new Vector();
		// Read the first element in the unsorted vector
		line = (String) v.elementAt(0);
		// Convert the string into a StateTable object
		statetbl = new StateTable(line);
		// Get the conversation name and the participant name
		String tempConvName = statetbl.getConvName();
		String tempPartName = statetbl.getPartName();
		// Get the process name by concatinating the conversation name and the participant name
		String tempProcessName = statetbl.getConvName()+statetbl.getPartName();
		// Now start at the beginning of the unsorted vector and pull out all the matching entries
		// that have the same process name (conversation name + participant name)
		for (int j = 0; j < v.size(); j++)
		{
			// Get the current line of the unsorted vector
			tempLine = (String) v.elementAt(j);
			// Convert the string to a State Table object
			statetbl = new StateTable(tempLine);
			// Get the process name by concatinating the conversation name and the participant name
			String tmpProcessName = statetbl.getConvName()+statetbl.getPartName();
			// If the two process names are equal, add found object to the temporary table
			if (tempProcessName.equals(tmpProcessName))
			{
				tempTable.addElement(statetbl);
			}
		}
		// Start back at the beginning of the unsorted vector and pull out all the matching entries
		for (int l = 0; l < v.size(); l++)
		{
			// Get the entry pointed to
			line = (String) v.elementAt(l);
			// Convert the string to a State Table object
			statetbl = new StateTable(line);
			// Get the process name by concatinating the conversation and participant names
			String tmpProcessName = statetbl.getConvName()+statetbl.getPartName();
			// If the names are equal, delete the entry from the unsorted vector
			if (tempProcessName.equals(tmpProcessName))
			{
				v.removeElementAt(l);
				l--;
			}
		}
		// Pull out all Start State transitions and put them in a separate working vector
		// Go through the temp vector...
		for (int m = 0; m < tempTable.size(); m++)
		{
			// Pull out the entry pointed to 
			statetbl = (StateTable) tempTable.elementAt(m);
			// Get the current state name
			String tmpCurrentState = statetbl.getCurrentState();
			// If it is the Start State...
			if (tmpCurrentState.equals("StartState"))
			{
				// Add it to the temp state table vector first
				stateTable.addElement(statetbl);
				// Then remove it from the temp vector
				tempTable.removeElementAt(m);
				// There could be more than one transition out of the Start State
				// So decrement the counter and look for more transitions out of the Start State
				m--;
			}
		}
		// Now go through the temp vector and the temp start state vector and copy the contents 
		// into the permanent state table vector and delete the entries from the start state temp vector
		while (tempTable.size() > 0)
		{
			// Get the first entry of the temporary vector
			statetbl = (StateTable) tempTable.elementAt(0);
			// Get the current state
			String tempState = statetbl.getCurrentState();
			// Add the element to the state table temporary vector
			stateTable.addElement(statetbl);
			// Remove the copied entry from the temp vector
			tempTable.removeElementAt(0);
			// Continue through the temp vector pulling out any more entries that match the current
			// state.  We want all state entries to be contiguous in the new vector
			for (int o = 0; o < tempTable.size(); o++)
			{
				// Get the first entry in the temp vector
				statetbl = (StateTable) tempTable.elementAt(o);
				// Get the current state
				String tempState2 = statetbl.getCurrentState();
				// If this is a matching state...
				if (tempState2.equals(tempState))
				{
					// Add the element to the state table temp vector
					stateTable.addElement(statetbl);
					// And remove the entry from the temp vector
					tempTable.removeElementAt(o);
					// Since we removed the entry, the counter must be decremented
					o--;
				}
			}
		}
		// We must create an End State entry for the automatic code generation
		statetbl = new StateTable("");
		// The conversation name is the same as the other transitions
		statetbl.setConvName(tempConvName);
		// The participant name is the same as the other transitions
		statetbl.setPartName(tempPartName);
		// This is an End State
		statetbl.setCurrentState("EndState");
		// An End State does not have any transitions out of it, so set all the other values to null
		statetbl.setGuardCondition("null");
		statetbl.setNextState("null");
		statetbl.setRecMessage("null");
		statetbl.setTransMessage("null");
		stateTable.addElement(statetbl);
	}
	// Return the sorted state table vector
	return stateTable;
}
/**
 * This method was created in VisualAge.
 * @param v java.util.Vector
 */
public static void verify(Vector v, Vector mscv)
{
	// Set system properties used in this program
	setProperties();
	// The unsorted vector must be sorted according to conversation name, participant name, and current
	// state
	Vector sortedVector = sortVector(v);
	// Check to see what Mark passes to me here... May not need sorting...
	Vector sortedMSCVector = null;
	// *****   Need to do something with sortedMSCVector  **********************
	// Show the message window
	spinFrame.show();
	// Display that we are beginning to verify the message sequence
	spinFrame.ta.setText("");
	spinFrame.ta.append("PLEASE STAND BY... TESTING MESSAGE SEQUENCE...\n");
	// Write status messages to the console so you can verify the process is running
	System.out.println("Reading Message Sequence Table...\n");
	// Create the output file, verify, from the sorted state table vector
	writeOutput(sortedVector,sortedMSCVector);
	System.out.println("Running Spin...\n");
	// Run Spin against the Promela code and analyze it
	runSpin();
	// Testing is completed
	spinFrame.ta.append("\n");
	spinFrame.ta.append("********** TESTING COMPLETED **********\n\n");
}
/**
 * Create the Promela source code from the state table
 * @param The vector of sorted conversations and transitions
 * @return nothing
 * @throws IOException
 */
public static void writeOutput(Vector stateTble, Vector mscTble)
{
	// Loop variables
	int i, j;
	// Process variables
	String tempProcessName = null;
	String tempCurrentState = null;
	StateTable stateTemp = null;
	Vector mtypeVector = new Vector();
	Vector channelVector = new Vector();
	String channelName = "bus";
	int processNameCntr = 0;
	int channelIndex = 0;
	boolean timeToWrite = true;
	boolean makingDo = false;
	boolean foundRecMessage = false;
	boolean foundGuardCondition = false;
	boolean foundTransMessage = false;
	boolean wroteTransmit = false;
	// If the verify file, verify exists, delete it
	// This way we are ensured to start over fresh with each verification
	if (verifyFile.exists())
	{
		boolean deleted = verifyFile.delete();
	}
	// File stream object
	DataOutputStream output = null;
	// Output formatting variables
	String processName, currentState, recMessage, guardCondition, transMessage, nextState;
	// Constants
	final String outputFile = workingDirectory + "Goverify";
	// Open the Verify output file
	try
	{
		output = new DataOutputStream(new FileOutputStream(outputFile));
	}
	// Catch any IO exceptions
	catch (IOException e)
	{
		System.err.println("File error.\n" + e.toString());
	}
	// Build vector of mtypes and channels from statetble 
	for (int k = 0; k < stateTble.size(); k++)
	{
		// Variables used
		foundRecMessage = false;
		foundGuardCondition = false;
		foundTransMessage = false;
		// Get the first entry in the state table vector
		stateTemp = (StateTable) stateTble.elementAt(k);
		// Get the process name by concatinating the conversation name and participant name
		processName = stateTemp.getConvName() + stateTemp.getPartName();
		// Get the received message... equals null if no entry
		recMessage = stateTemp.getRecMessage();
		// Get the guard condition... equals null if no entry
		guardCondition = stateTemp.getGuardCondition();
		// Get the transmitted message... equals null if no entry
		transMessage = stateTemp.getTransMessage();
		// If this is the first time we have encountered this process name...
		if (!processName.equals(tempProcessName))
		{
			// We create channels for every pair of conversations
			// This is why we require conversations to be matched.
			// Time to write becomes true for every other process name
			if (timeToWrite)
			{
				// Create a channel name that is the same as the conversation name
				channelName = stateTemp.getConvName();
				// Add the channel name to a vector of channel names for later declaration
				channelVector.addElement(channelName);
				// Reset the time to write flag
				timeToWrite = false;
			}
			else
			{
				// Set the time to write flag
				timeToWrite = true;
			}
		}
		// Create the message type, mtype declarations
		// Look at every entry in the mtype vector and do not add any repeat entries
		// Any received message, guard condition, or transmitted message is an mtype declaration
		tempProcessName = processName;
		for (int l = 0; l < mtypeVector.size(); l++)
		{
			// Get the pointed to mtype entry
			String tempMType = (String) mtypeVector.elementAt(l);
			// If the received message is already in the vector, take note
			if (recMessage.equals(tempMType))
			{
				foundRecMessage = true;
			}
			// If the guard condition is already in the vector, take note
			if (guardCondition.equals(tempMType))
			{
				foundGuardCondition = true;
			}
			// If the transmitted message is already in the vector, take note
			if (transMessage.equals(tempMType))
			{
				foundTransMessage = true;
			}
		}
		// If the received message is not in the vector already, and not null...
		if (!foundRecMessage && !recMessage.equals("null"))
		{
			// add the received message to the vector
			mtypeVector.addElement(recMessage);
		}
		// If the guard condition message is not in the vector already, and not null...
		if (!foundGuardCondition && !guardCondition.equals("null"))
		{
			// add the guard condition message to the vector
			mtypeVector.addElement(guardCondition);
		}
		// If the transmitted message is not in the vector already, and not null...
		if (!foundTransMessage && !transMessage.equals("null"))
		{
			// add the transmitted message to the vector
			mtypeVector.addElement(transMessage);
		}
		try
		{
			// Flush the output buffer to ensure nothing is in it
			output.flush();
		}
		// Catch any IO errors
		catch (IOException io)
		{
			System.out.println("OUTPUT FLUSH ERROR");
		}
	}
	tempProcessName = null;
	// Start writing the mtype declarations
	Format.print(output, "%s", "mtype = { ");
	// iterate through the mtypeVector and write the mtypes
	for (int k = 0; k < mtypeVector.size(); k++)
	{
		String tempMType = (String) mtypeVector.elementAt(k);
		// Write the mtype definitions
		Format.print(output, "%s", tempMType);
		// Mtype declarations are separated by commas, but not after the last entry
		if (k < mtypeVector.size() - 1)
		{
			Format.print(output, "%s", ", ");
		}
		// Flush the buffer again to ensure it is empty
		try
		{
			output.flush();
		}
		// Catch any IO errors
		catch (IOException io)
		{
			System.out.println("OUTPUT FLUSH ERROR");
		}
	}
	// Write the closing brace of the mtype declarations
	Format.print(output, "%s", " };\n\n");
	// Write the channel declaration
	for (int m = 0; m < channelVector.size(); m++)
	{
		Format.print(output, "%s", "chan " + (String) channelVector.elementAt(m) + " = [1] of {mtype};\n");
	}
	Format.print(output, "%s", "\n");
	// Time to write must be true the first time around 
	timeToWrite = true;
	// Append each channel name with an index
	channelIndex = 0;
	//Iterate through the State table and write the info
	for (int k = 0; k < stateTble.size(); k++)
	{
		// Get the state table information at the kth element
		stateTemp = (StateTable) stateTble.elementAt(k);
		// Get the process name by concatenating the conversation name and participant name
		processName = stateTemp.getConvName() + stateTemp.getPartName();
		// Get the current state
		currentState = stateTemp.getCurrentState();
		// Get the received message
		recMessage = stateTemp.getRecMessage();
		// Get the guard condition
		guardCondition = stateTemp.getGuardCondition();
		// Get the transmitted message
		transMessage = stateTemp.getTransMessage();
		// Get the next state
		nextState = stateTemp.getNextState();
		// If this is a new current state and we're not making a Do loop, then close off the
		// current do loop
		if ((!currentState.equals(tempCurrentState)) && (makingDo))
		{
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", "od;\n");
			// No longer making a do loop
			makingDo = false;
		}
		// If this is a new process, close off the current proctype declaration
		if (!processName.equals(tempProcessName) && tempProcessName != null)
		{
			Format.print(output, "%s", "}\n\n");
		}
		// If this is a new process, start a new proctype declaration
		if (!processName.equals(tempProcessName))
		{
			Format.print(output, "%s", "proctype ");
			Format.print(output, "%s", processName);
			Format.print(output, "%s", "()\n");
			Format.print(output, "%s", "{\n");
			// Set channel variable to be used in this proctype
			if (timeToWrite)
			{
				channelName = (String) channelVector.elementAt(channelIndex);
				channelIndex++;
				timeToWrite = false;
			}
			else
			{
				timeToWrite = true;
			}
		}
		// If this is a new current state and we're not making a do loop make a new state declaration
		if ((!currentState.equals(tempCurrentState)) && (!makingDo))
		{
			Format.print(output, "%s", Tab);
			// Every state begins with the key word progress so we can check for non progress states
			Format.print(output, "%s", "progress" + currentState);
			Format.print(output, "%s", ":\n");
			Format.print(output, "%s", Tab + Tab);
			// The beginning of the do loop
			Format.print(output, "%s", "do\n");
			// Now we are making a do loop
			makingDo = true;
		}
		// If the received message is not equal to null, it contains something of value
		if (!recMessage.equals("null"))
		{
			if (!guardCondition.equals("null"))
			{
				Format.print(output, "%s", Tab + Tab);
				Format.print(output, "%s", ":: " + guardCondition + " -> ");
			}
			// Write the received message statement
			if (guardCondition.equals("null"))
			{
				Format.print(output, "%s", Tab + Tab);
				// Received messages are annotated by a ?
				Format.print(output, "%s", ":: ");
			}
			Format.print(output, "%s", channelName + "?");
			Format.print(output, "%s", recMessage);
			// A received message event could be followed by a send message event
			if (!transMessage.equals("null"))
			{
				// Write the transmit message line
				// Transmit messages are annotated by a !
				Format.print(output, "%s", " -> " + channelName + "!");
				Format.print(output, "%s", transMessage);
				Format.print(output, "%s", "; goto ");
				// Every state is preceded by the word progress
				Format.print(output, "%s", "progress" + nextState);
				Format.print(output, "%s", "\n");
				wroteTransmit = true;
			}
			// In this case, no transmit message is associated with a receive message
			else
			{
				// Simply goto the next state
				Format.print(output, "%s", " -> goto ");
				Format.print(output, "%s", "progress" + nextState);
				Format.print(output, "%s", "\n");
			}
		}
		// If a guard condition exists...
		if (!guardCondition.equals("null") && recMessage.equals("null"))
		{
			// Write the guard condition line
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", ":: ");
			Format.print(output, "%s", guardCondition);
			// If there is a transmit message associated with a guard condition...
			if (!transMessage.equals("null"))
			{
				// Write the transmit message line
				Format.print(output, "%s", " -> " + channelName + "!");
				Format.print(output, "%s", transMessage);
				// Then goto the next state
				Format.print(output, "%s", "; goto ");
				Format.print(output, "%s", "progress" + nextState);
				Format.print(output, "%s", "\n");
			}
			///*********** Changes  *********
			// Guard condition exists with no other messages
			else
			{
				Format.print(output, "%s", " -> goto ");
				Format.print(output, "%s", "progress" + nextState);
				Format.print(output, "%s", "\n");
			}
		}
		// If there is a transmit message without a guard condition and we didn't already
		// write this transmit message
		if ((!transMessage.equals("null")) && (guardCondition.equals("null")) && !wroteTransmit)
		{
			// Write the transmit message line
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", ":: " + channelName + "!");
			Format.print(output, "%s", transMessage);
			// And goto the next state
			Format.print(output, "%s", " -> goto ");
			Format.print(output, "%s", "progress" + nextState);
			Format.print(output, "%s", "\n");
		}
		// If there is no guard condition with no received message and no transmit message
		if ((guardCondition.equals("null")) && (transMessage.equals("null")) && (recMessage.equals("null")) && (!nextState.equals("null")))
		{
			// Write a goto the next state
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", ":: goto ");
			Format.print(output, "%s", "progress" + nextState);
			Format.print(output, "%s", "\n");
		}
		// If we are at the End State...
		if (currentState.equals("EndState"))
		{
			// Exit the do loop with a break, which will fall to the closing brace of the conversation
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", ":: break");
			Format.print(output, "%s", "\n");
		}
		// If we are the last entry in the state table vector...
		if ((k == (stateTble.size() - 1)))
		{
			// Close off the last do loop and the closing brace for the proctype declarations
			Format.print(output, "%s", Tab + Tab);
			Format.print(output, "%s", "od;\n");
			Format.print(output, "%s", "}\n\n");
		}
		// Temp variables to set up Process states
		tempProcessName = processName;
		tempCurrentState = currentState;
		wroteTransmit = false;
		// Flush the output buffer to ensure it is empty
		try
		{
			output.flush();
		}
		// Catch any IO errors
		catch (IOException io)
		{
			System.out.println("OUTPUT FLUSH ERROR");
		}
	}
	// ***End of State Table output ***//

	//Create the init procedure
	Format.print(output, "%s", "init\n");
	Format.print(output, "%s", "{\n");

	//Write the atomic statements
	Format.print(output, "%s", Tab);
	Format.print(output, "%s", "atomic\n");
	Format.print(output, "%s", Tab);
	Format.print(output, "%s", "{\n");

	// ************************************************************************
	//Loop through the MSC table and generate the run statements
	String tempConvName = "null";
	String convName = null;
	String convInit = null;
	String convResp = null;
	String message = null;
	for (int k = 0; k < mscTble.size(); k++)
	{
		MSCTable mscTemp = (MSCTable)mscTble.elementAt(k);
		convName = mscTemp.getConvName();
		convInit = mscTemp.getConvInit();
		convResp = mscTemp.getConvResp();

		//Write the run statements
		if (!tempConvName.equals("null"))
		{
			Format.print(output, "%s", ";\n");
		}
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", "run ");
		Format.print(output, "%s", convName+convInit);
		Format.print(output, "%s", "();\n");
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", "run ");
		Format.print(output, "%s", convName+convResp);
		Format.print(output, "%s", "()");
		tempConvName = convName;
	}
	Format.print(output, "%s", "\n");
	Format.print(output, "%s", Tab);
	Format.print(output, "%s", "}\n");
	Format.print(output, "%s", "}\n\n");
	Format.print(output, "%s", "never\n");
	Format.print(output, "%s", "{\n");
	for (int k = 0; k < mscTble.size(); k++)
	{
		MSCTable mscTemp = (MSCTable) mscTble.elementAt(k);
		convName = mscTemp.getConvName();
		message = mscTemp.getMessage();
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", "State"+k+":\n");
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", "do\n");
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", Tab);
		if (k==(mscTble.size()-1))
		{
			Format.print(output, "%s", ":: "+convName+"?["+message+"] -> goto accept\n");
		}
		else
		{
			Format.print(output, "%s", ":: "+convName+"?["+message+"] -> goto State"+(k+1)+"\n");
		}
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", ":: skip\n");
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", Tab);
		Format.print(output, "%s", "od;\n");
	}
	Format.print(output, "%s", Tab);
	Format.print(output, "%s", "accept:\n");
	Format.print(output, "%s", Tab);
	Format.print(output, "%s", Tab);
	Format.print(output, "%s", "skip\n");		
	Format.print(output, "%s", "}\n");
	try
	{
		output.flush();
	}
	catch (IOException io)
	{
		System.out.println("OUTPUT FLUSH ERROR");
	}


	//Close the Verify Output file
	try
	{
		output.flush();
		output.close();
		readyToRunSpin = true;
	}
	catch (IOException io)
	{
		System.err.println("File not closed properly\n" + io.toString());
		System.exit(0);
	}
}
}
