package system;

/**
 * task in role diagram
 * has one role
 * This type was created in VisualAge.
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import java.util.*;
public class ATTask extends ATobject{

	private ATRole role;
	private ATstatetable table;
	///--graphics objects stuff.

	//actual graphics object
	private ATTaskGRFX grfxObj;
	private ATTaskConnectorGRFX grfxCObj;
	//keeps track of whether agent is selected or not 
	public boolean Selected = false;
	private static final String ukh = "task@";
/**
 * ATagent constructor comment.
 */
public ATTask(double ver, String[] tokens,ATsystem sys) {
	this(ver,0);
	 //*** Version 1 of ATagent ***
	int loc = ATtokenOps.findnextinstanceof("version",tokens,0);
	double v=Double.parseDouble(tokens[loc+3]);
	//setVersion(v);
	//table = new ATstatetable(v);

  //Code for constructing object model from MAML tokens
	if (v>=1) constructFromV1(tokens,sys);
	else {
		System.out.println("Loading a version other than 1.");	
	}
 	//	System.out.println("Loaded an agent");
}
	//keeps track of whether an agent's corner is selected
	
/**
 * ATagent constructor comment.
 */
public ATTask(double v, int keynum) {
	super(v);
	setKey(ukh + keynum);
	MAMLheader+="<!DOCTYPE agent SYSTEM \"atagent"+Double.toString(getVersion())+".dtd\">\n";
	grfxObj = new ATTaskGRFX();
	grfxCObj = new ATTaskConnectorGRFX();
	table = new ATstatetable(v,keynum);
}
/**
 * ATagent constructor comment.
 */
public ATTask(double ver, String x,ATsystem sys) {
	this(ver,ATtokenOps.parseLine(x),sys);
}
	//keeps track of whether an agent's corner is selected
	
/**
 * ATagent constructor comment.
 */
public ATTask(ATTask cp) {
	super(cp);
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public boolean checkSelected(int x, int y) {
	//check to see if object has been selected
	if(grfxObj.dist(x,y)) {
		Selected = true;
		return true;
	} else {
		Selected = false;
		return false;
	}
}
/**
 * Insert the method's description here.
 * Creation date: (10/25/99 11:28:36 PM)
 */
public void constructFromV1(String[] tokens,ATsystem sys) {
	
	 	int end=0;
		
		int loc = ATtokenOps.findnextinstanceof("version",tokens,0);
  		double v=Double.parseDouble(tokens[loc+3]);
		loc = ATtokenOps.findnextinstanceof("name",tokens,0);
		setName(tokens[loc+3]);

		boolean keybased = false;
		loc  = ATtokenOps.findnextinstanceof("key", tokens, loc);
		if(loc != -1) {
			setKey(tokens[loc+3]);
			keybased = true;
		} else {
			setKey(ukh + sys.getNextKey());
		}
	
		int start=ATtokenOps.findnextinstanceof("description", tokens, loc+3);
	    loc  = ATtokenOps.findnextinstanceof("/description", tokens, start+2);
		description=ATtokenOps.extract(tokens,(start+2), (loc-2));
		//Get agent coordinates for graphics display
		loc = ATtokenOps.findnextinstanceof("rName",tokens,loc+2);
		String rName = (String)tokens[loc+3];
		Enumeration roles = sys.returnRoles();
		ATRole role;
		while(roles.hasMoreElements()){
			role = (ATRole)roles.nextElement();
			if(keybased) {
				if(role.getKey().equals(rName)){
					setRole(role);
				}
			} else {
				if(role.getName().equals(rName)){
					setRole(role);
				}
			}
		}
		loc  = ATtokenOps.findnextinstanceof("view", tokens, loc+2);
		if (loc>0){ //KB-stored agents do not have view information
					//because they appear at a location the operator selects
			int x = Integer.parseInt(tokens[loc+5]);
			int y = Integer.parseInt(tokens[loc+11]);
			int h = Integer.parseInt(tokens[loc+17]);
			int w = Integer.parseInt(tokens[loc+23]);
	//		loc+=25;
			setCoordinates(x,y,h,w);
		}
		loc = ATtokenOps.findnextinstanceof("n_pts",tokens,loc+2);
		int j = 0;
		if(loc>0){
			j = Integer.parseInt(tokens[loc+3]);
			loc= loc +3;
		}
//		setCoordinates(component1,component2);
		
		int[] xpos = new int[j];
		int[] ypos = new int[j];
		
		for(int i=0; i<j; i++){
			xpos[i] = Integer.parseInt(tokens[loc+6]);
			loc+=6;
			ypos[i] = Integer.parseInt(tokens[loc+6]);
			loc+=6;
		}
		grfxCObj.initializePoints(xpos,ypos);
		
/*		for(int i=0; i<j; i++){
			int k = Integer.parseInt(tokens[loc+6]);
			loc+=6;
			int l = Integer.parseInt(tokens[loc+6]);
			loc+=6;
			grfxCObj.Xs[i]=k;
			grfxCObj.Ys[i]=l;
		}
*/
		start   = ATtokenOps.findnextinstanceof("statetable", tokens, end);
		//Need to locate "match" for this architecture, not for possible sub-architectures.
		end     = ATtokenOps.findnextinstanceof("/statetable", tokens, start);
		String[] submaml = ATtokenOps.subTokens(tokens, (start-1), (end+1));
		table = new ATstatetable(v,submaml,sys);
}
public int conversationCount() {
	//return the number of conversations connected to this agent
	return grfxObj.lineCount();
}
/**
 * Insert the method's description here.
 * Creation date: (6/8/00 3:33:33 PM)
 */
public void deleteMe() {
	role.deleteTask(this);
	grfxCObj.deleteMe();
	}
/**
 * This method was created in VisualAge.
 */
public void draw(java.awt.Graphics g) {
	//draw the object
	grfxCObj.drawLine(g);
	grfxObj.drawObject(g);
//	grfxCObj.drawLine(g);
	//if the object is selected, draw the selection box
	if(Selected==true){
		grfxObj.drawSelected(g);
	}
}
public void drawDiag(java.awt.Graphics g) {
	//draw the state diagram for this component
	table.draw(g);
}
/**
 * This method was created in VisualAge.
 * @return system.ATagentGRFX
 */
public ATTaskGRFX getGrfx() {
	return grfxObj;
}
/**
 * This method was created in VisualAge.
 */
public String getMAML() throws Exception {
	//Opening line for an Component element
	MAML="";
	MAML+="\t<task";
	MAML+=" version=\"" + Double.toString(version) + "\"";
	MAML+=" name=\"" + name + "\"";
	//New Line for Description element
	
	MAML += " key=\"" + getKey() + "\"";
	
	MAML+="\t\t<description>\n\t\t";
	MAML+=getDescription();
	MAML+="\n\t\t</description>\n";
	MAML+="\t\t<rName=\"" + role.getKey() +"\"\n";
	//MAML+="\t\t<rName=\"" + role.getName() +"\"\n";
	int[] c = grfxObj.getCoords();
	MAML+="\t\t<view";
	MAML+=" x=\""+ c[0] + "\"";
	MAML+=" y=\""+ c[1] + "\"";
	MAML+=" w=\""+ c[2] + "\"";
	MAML+=" h=\""+ c[3] + "\"/>\n";

	MAML+=" n_pts=\""+grfxCObj.n()+"\"";//>\n";
	for(int j = 0; j<grfxCObj.n(); j++){
		MAML+=" x" +j+ "=\"" +grfxCObj.x(j) +"\"";
		MAML+=" y" +j+ "=\"" +grfxCObj.y(j) +"\"";
	}
	//StateTable
	MAML+=table.getMAML();
	//End of encode sequence	
	MAML+="\t\t\t</task>\n";

	return MAML;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getName() {
	return name;
}
/**
 * Insert the method's description here.
 * Creation date: (5/31/00 8:38:49 AM)
 * @return system.ATRole
 */
public ATRole getRole() {
	return role;
}
/**
 * This method was created in VisualAge.
 * @return system.ATstatetable
 */
public ATstatetable getStateTable() {
	return table;
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public void move(int x, int y) {
	//move the graphics object
	grfxObj.move(x,y);
}
/**
 * Insert the method's description here.
 * Creation date: (6/8/00 3:17:14 PM)
 * @return boolean
 */
public boolean nothingAttached() {
	if((grfxObj.LinesX1).size()==1){
		return true;
	}
	return false;
}
/**
 * This method was created in VisualAge.
 */
public void print(java.awt.Graphics g) {
	grfxObj.paint(g);
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public void resize(int x, int y) {
	//resize the graphics object
	grfxObj.resize(x,y);
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 1:07:33 PM)
 * @param role system.ATRole
 */
public void resizeLine(ATRole role) {
	grfxCObj.resize(role.getGrfx());
	}
/**
 * This method was created in VisualAge.
 */
public void setCoordinates(int x, int y, int width, int height) {
	grfxObj.place(x,y,width,height);
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setName(String s) {
	name = ATtokenOps.validName(s);
	grfxObj.name = name;
	grfxObj.resetMax();
}
/**
 * Insert the method's description here.
 * Creation date: (5/2/00 12:22:46 PM)
 * @param r system.ATRole
 */
public void setRole(ATRole r) {
	role = r;
	r.addTask(this);
	setCoordinates(r.getGrfx().center_x,r.getGrfx().center_y+75,50,30);
	grfxCObj.place(r.getGrfx(),grfxObj);
//	grfxCObj.Arrow();
}
/**
 * Insert the method's description here.
 * Creation date: (7/17/00 2:17:09 PM)
 */
public void writeDATA(Vector protocols) {
	super.writeDATA();
	MAML+= "\n\tRole = " + getRole().getName();
	MAML+= "\n\tConnected to:\n";
	Enumeration prots = protocols.elements();
	ATProtocol prot;
	while(prots.hasMoreElements()){
		prot = (ATProtocol)prots.nextElement();
		if(prot.getInitiator() == this){
			MAML+= "\t"+ prot.getResponder().getName() + "\tvia " + prot.getName() +"\n";
		} else if(prot.getResponder() == this){
			MAML+= "\t"+ prot.getInitiator().getName() + "\tvia " + prot.getName() +"\n";
		}
	}
}
}
