package system;

/**
 * the architecture of an agent or component
 * made up of components and connections
 * Creation date: (12/2/99 11:21:36 AM)
 * @author: 
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import java.util.Vector;
import java.util.Enumeration;
public class ATarchitecture extends ATobject {
	private Vector components;
	private Vector connections;
	private static final String ukh = "arch@";
/**
 * AgentArch constructor comment.
 */
public ATarchitecture(double ver, String[] tokens, ATsystem sys) {
	this(ver,0); //*** Version 1 of ATagent ***
	
	int loc = ATtokenOps.findnextinstanceof("version",tokens,0);
  	double v=Double.parseDouble(tokens[loc+3]);
	//setVersion(v);
  //Code for constructing object model from MAML tokens
	if (v>=1) constructFromV1(tokens, sys);
	else {
		System.out.println("Loading a version other than 1.");	
	}
 	//	System.out.println("Loaded an agent");
}
/**
 * AgentArch constructor comment.
 */
public ATarchitecture(double v, int keynum) {
	super(v);
	MAMLheader+="<!DOCTYPE architecture SYSTEM \"atarch"+Double.toString(getVersion())+".dtd\">\n";
	
	// architecture will have the same keynum as the Agent or Component it was created
	// from.
	setKey(ukh + keynum);  
	
	components  = new java.util.Vector();
	connections = new java.util.Vector();
}
/**
 * AgentArch constructor comment.
 */
public ATarchitecture(double ver, String s, ATsystem sys) {
	this(ver,ATtokenOps.parseLine(s),sys);
}
/**
 * AgentArch constructor comment.
 */
public ATarchitecture(ATarchitecture cp) {
	super(cp);
}
public void addComponent(ATagentComponent c) {
	//set all components to not be selected
	ATagentComponent j;
	java.util.Enumeration ComponentsEnum = components.elements();
	while(ComponentsEnum.hasMoreElements()){
		j=(ATagentComponent)ComponentsEnum.nextElement();
		j.Selected = false;
	}
	//add component c

	String n = c.getName();
	String name = n;
	int i=2;
	while(repeatName(name)) {
		name = n + Integer.toString(i);
		i++;
	}
	c.setName(name);
	components.addElement(c);
	c.Selected = true;
}
public void addConnection(ATconnection c) {
	//add c to connections
	connections.addElement(c);
}
/**
 * Insert the method's description here.
 * Creation date: (2/1/00 10:14:17 AM)
 */
public void clear() {
	//remove all connections
	connections.removeAllElements();
	//remove all components
	components.removeAllElements();

}
/**
 * Insert the method's description here.
 * Creation date: (10/25/99 11:28:36 PM)
 */
public void constructFromV1(String[] tokens, ATsystem sys) {
	
	    int end=0;
		int loc = ATtokenOps.findnextinstanceof("name",tokens,0);
  		setName(tokens[loc+3]);
		
		loc  = ATtokenOps.findnextinstanceof("key", tokens, loc);
		if(loc != -1) {
			setKey(tokens[loc+3]);
		} else {
			setKey(ukh + sys.getNextKey());
		}
		
		int start=ATtokenOps.findnextinstanceof("description", tokens, loc+3);
	    loc  = ATtokenOps.findnextinstanceof("/description", tokens, start+2);
		description=ATtokenOps.extract(tokens,(start+2), (loc-2));

	   for(int i=loc;i<tokens.length;++i) {
	     if (tokens[i].equals("component")) {
		    loc = ATtokenOps.findmatch("component", tokens, i);
		    ATagentComponent c = new ATagentComponent(version, ATtokenOps.extract(tokens,(i-1), (loc+1)), sys);
		    addComponent(c);
		    i=loc;
	     }
	   }
	   for(int i=(loc+2);i<tokens.length;++i) {
	     if (tokens[i].equals("connection")) {
		    loc=ATtokenOps.findnextinstanceof("/", tokens, i);
		    ATconnection c = new ATconnection(version, ATtokenOps.extract(tokens,(i-1), (loc+1)), this, sys);
		    addConnection(c);
	     }
	   }
}
/**
 * Insert the method's description here.
 * Creation date: (10/22/99 3:10:54 PM)
 */
public ATagentComponent findComponentbykey(String key) {

	ATagentComponent c = null;
	Enumeration comps = returnComponents();
	if(comps != null) {
		while(comps.hasMoreElements()){
		   c = (ATagentComponent)comps.nextElement();
		   if (c.getKey().equals(key)) return (ATagentComponent)c;
		}
	}
	return null;	
}
/**
 * Insert the method's description here.
 * Creation date: (10/22/99 3:10:54 PM)
 */
public ATagentComponent findComponentbyname(String name) {

	ATagentComponent c = null;
	Enumeration comps = returnComponents();
	if(comps != null) {
		while(comps.hasMoreElements()){
		   c = (ATagentComponent)comps.nextElement();
		   if (c.getName().equals(name)) return (ATagentComponent)c;
		}
	}
	return null;	
}
/**
 * Insert the method's description here.
 * Creation date: (11/29/00 11:49:28 AM)
 */
public void getAllNames() {
	Enumeration enum = components.elements();
	while(enum.hasMoreElements()) {
		ATagentComponent comp = (ATagentComponent)enum.nextElement();
		comp.getAllNames();
	}	
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public int getComponentCount() {
	int count = 0;
	Enumeration enum = components.elements();
	while(enum.hasMoreElements()) {
		ATagentComponent comp = (ATagentComponent)enum.nextElement();
		count += (comp.getSubComponentCount() + 1);
	}
	return count;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public int getConnectionCount() {
	int count = 0;
	Enumeration enum = components.elements();
	while(enum.hasMoreElements()) {
		ATagentComponent comp = (ATagentComponent)enum.nextElement();
		count += comp.getSubConnectionCount();
	}
	count += connections.size();
	return count;
}
/**
 * Insert the method's description here.
 * Creation date: (10/20/99 1:16:45 PM)
 */
public String getMAML() throws Exception {
	
	//Opening line for an Agent element
	MAML="";
	MAML+="\t\t<architecture";
	MAML+="\t version=\"" + Double.toString(version) + "\"";
	MAML+="\t name=\"" + name + "\">\n";
	//New Line for Description element
	
	MAML += " key=\"" + getKey() + "\"";
	
	MAML+="\t\t\t<description>\n\t\t\t";
	MAML+=getDescription();
	MAML+="\n\t\t\t</description>\n";	
	//Opening line for an Property element
	
	//Closing line for an Property element	
	//Opening line for an Property element
	
	//Closing line for an Property element
	//Opening line for an Component elements
	Enumeration comps  = returnComponents();
	if(components != null){
	 while(comps.hasMoreElements()){
		MAML+=((ATagentComponent)comps.nextElement()).getMAML();
	 }
	}
	//Closing line for an Component elements	
	//Opening line for an Connector elements
	Enumeration conns  = returnConnections();
	if(conns != null){
	 while(conns.hasMoreElements()){
		MAML+=((ATconnection)conns.nextElement()).getMAML();
	 }
	}	
	//Closing line for an Connector elements
	//Closing line for a architecture element		
	MAML+="\n\t\t</architecture>\n";

	return MAML;
}
/**
 * Insert the method's description here.
 * Creation date: (10/28/99 10:31:27 AM)
 */
public void integrateKBInfo(String maml, ATsystem sys) {
  String[] tokens = ATtokenOps.parseLine(maml);
  int v=Integer.parseInt(tokens[6]);
	
  //Code for constructing object model from MAML tokens
  if (v==version) {
	constructFromV1(tokens, sys);
  }
  else {
	System.out.println("Loading from KB a version other than 1");	
  }	
}
/**
 * Insert the method's description here.
 * Creation date: (12/28/00 2:42:24 PM)
 */
public void removeAllComponents() {
	components.removeAllElements();
	}
/**
 * Insert the method's description here.
 * Creation date: (12/28/00 2:42:38 PM)
 */
public void removeAllConnections() {
	connections.removeAllElements();
	}
public void removeComponent(ATagentComponent c) {
	components.removeElement(c);
}
public void removeConnection(ATconnection c) {
	connections.removeElement(c);
	c.deleteMe();
}
/**
 * Insert the method's description here.
 * Creation date: (11/28/00 1:13:12 PM)
 * @return boolean
 * @param n java.lang.String
 */
public boolean repeatName(String n) {
	Enumeration enum = components.elements();
	while(enum.hasMoreElements()) {
		ATagentComponent comp = (ATagentComponent)enum.nextElement();
		if(comp.repeatName(n)) return true;
	}
	return false;
}
public Enumeration returnComponents() {
	if (components == null) {
		return null;
	}
	Enumeration ComponentsEnum = components.elements();
	return ComponentsEnum;
}
public java.util.Enumeration returnConnections() {
	if (connections == null) {
		return null;
	}
	java.util.Enumeration ConnectionsEnum = connections.elements();
	return ConnectionsEnum;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setName(String s) {
	name = ATtokenOps.validName(s);
}
}
