package system;

/**
 * conversation between two agents
 * has two state tables (initiator and responder)
 * This type was created in VisualAge.
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
// import java.util.Vector;
import java.util.*;

public class ATconversation extends ATobject {

	private String mult;	//multiplicity of conversation
	private ATstatetable initiator;
	private ATstatetable responder;
	
// graphics info
	private ATconvGRFX grfxObj;
	public boolean Selected;
	private static final String ukh = "conv@";
/**
 * ATconversation constructor comment.
 */
public ATconversation(double ver, String[] tokens, ATsystem sys) {
	this(ver,0); //*** Version 1 of ATconversation ***

	int loc = ATtokenOps.findnextinstanceof("version",tokens,0);
  	double v=Double.parseDouble(tokens[loc+3]);
  	
  //setVersion(v);
  //Code for constructing object model from MAML tokens
  if (v>=1) constructFromV1(tokens, sys);
  else {
	System.out.println("Loading a version other than 1");	
  }
 }   
/**
 * ATconversation constructor comment.
 */
public ATconversation(double v, int keynum) {
	super(v);
	setKey(ukh + keynum);
	initiator = null;
	responder = null;
	MAMLheader+="<!DOCTYPE conversation SYSTEM \"atconversation"+Double.toString(getVersion())+".dtd\">\n";
	grfxObj = new ATconvGRFX();
	Selected = false;
}
/**
 * ATconversation constructor comment.
 */
public ATconversation(double ver, String maml, ATsystem sys) {
	this(ver,ATtokenOps.parseLine(maml),sys);
 }  
/**
 * ATconversation constructor comment.
 */
public ATconversation(ATconversation cp) {
	super(cp);
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public void addPoint(int x, int y) {
	grfxObj.addPoint(x,y);
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public void arrange_pts() {
	grfxObj.arrange_pts();
}
/**
 * This method was created in VisualAge.
 * @return java.util.Vector
 */
public java.util.Vector buildVerificationTable() {
	java.util.Vector tempVec;

	//tempVec = initiator.clone();
	return null;
}
/**
 * This method was created in VisualAge.
 * @return boolean
 * @param agent afit.system.ATagent
 */
public boolean checkAgent(ATagent agent) {
	boolean match;
	match = grfxObj.checkMatch(agent.getGrfx());
	return match;
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public boolean checkSelected(int x, int y) {
	if(grfxObj.dist(x,y)) {
		Selected = true;
		return false;
	} else {
		Selected = false;
		return true;
	}
}
/**
 * Insert the method's description here.
 * Creation date: (10/25/99 11:33:58 PM)
 * @param tokens java.lang.String[]
 */
public void constructFromV1(String[] tokens, ATsystem sys) {

 	int start=0;
	int end=0;
	
	int loc = ATtokenOps.findnextinstanceof("version",tokens,0);
  	double v = Double.parseDouble(tokens[loc+3]);
	
  	loc = ATtokenOps.findnextinstanceof("name",tokens,0);
  	setName(tokens[loc+3]);
	description=tokens[loc+9];
	boolean keybased = false;

	loc  = ATtokenOps.findnextinstanceof("key", tokens, 18);
	if(loc != -1) {
		setKey(tokens[loc+3]);
		keybased = true;
	} else {
		setKey(ukh + sys.getNextKey());
	}
	
	start = ATtokenOps.findnextinstanceof("description", tokens, 15);
	end   = ATtokenOps.findnextinstanceof("/description", tokens, start+2);
	description=ATtokenOps.extract(tokens,(start+2), (end-2));
	//Load initiator
		  start = ATtokenOps.findnextinstanceof("statetable", tokens, end);
		  end   = ATtokenOps.findnextinstanceof("/statetable", tokens, start);
		  String[] submaml= ATtokenOps.subTokens(tokens, (start-1), (end+1));
		  //ATagent i = getInitiator();		  
		  initiator = new ATstatetable(v,submaml,sys); //pass conversation model version down
	//Load responder
		  start = ATtokenOps.findnextinstanceof("statetable", tokens, (end+1));
		  end   = ATtokenOps.findnextinstanceof("/statetable", tokens, start);
		  submaml= ATtokenOps.subTokens(tokens, (start-1), (end+1));
		  //ATagent r = getResponder();
		  responder = new ATstatetable(version,submaml,sys); //pass conversation model version down
		  loc = ATtokenOps.findnextinstanceof("view",tokens,end+2);
	int j = 0;
	if(loc>0){
		j = Integer.parseInt(tokens[loc+5]);
		loc= loc +5;
	}
	ATagent temp,init = null,resp = null;
	if(sys!=null){
		Enumeration agnts = sys.returnAgents();
		while(agnts.hasMoreElements()){
		     temp = (ATagent)agnts.nextElement();
		     if(keybased) {
				if ((temp.getKey()!=null)&&(temp.getKey().equals(getInitiatorKey())))   init=temp;
				if ((temp.getKey()!=null)&&(temp.getKey().equals(getResponderKey())))   resp=temp;
		     } else {
				if ((temp.getName()!=null)&&(temp.getName().equals(getInitiator().getName())))   init=temp;
				if ((temp.getName()!=null)&&(temp.getName().equals(getResponder().getName())))   resp=temp;
		     }
		}
		if(init == null) {
			System.out.println("Error!!!  Initiator not found.");
		} else if(resp == null) {
			System.out.println("Error!!!  Responder not found.");
		} else {
			setCoordinates2(init,resp);
		}
	}			
	//grfxObj.place(initiator.getParticipant().getGrfx(),responder.getParticipant().getGrfx());
	if(v>1){
/*		for(int i=0; i<j; i++){
			int k = Integer.parseInt(tokens[loc+6]);
			loc+=6;
			int l = Integer.parseInt(tokens[loc+6]);
			loc+=6;
			grfxObj.Xs[i]=k;
			grfxObj.Ys[i]=l;
		}
*/
		int[] xpos = new int[j];
		int[] ypos = new int[j];
		
		for(int i=0; i<j; i++){
			loc+=12;
			xpos[i] = Integer.parseInt(tokens[loc-6]);
			ypos[i] = Integer.parseInt(tokens[loc]);
		}
		grfxObj.initializePoints(xpos,ypos);
		
/*		grfxObj.setType(tokens[loc+7]);
		loc+=7;
		char p1 = (tokens[loc+7]).charAt(0);
		loc+=7;
		int n1 = Integer.parseInt(tokens[loc+4]);
		loc+=4;
		char p2 = (tokens[loc+7]).charAt(0);
		loc+=7;
		int n2 = Integer.parseInt(tokens[loc+4]);
*/		
		grfxObj.setType(tokens[loc+7]);
		char p1 = (tokens[loc+14]).charAt(0);
		int n1 = Integer.parseInt(tokens[loc+18]);
		char p2 = (tokens[loc+25]).charAt(0);
		int n2 = Integer.parseInt(tokens[loc+29]);
		loc+=25;

//		grfxObj.n_pts=j;
		grfxObj.Arrow();
		grfxObj.setPos(p1,n1,p2,n2);
	}	
//	System.out.println("End of conversation parsing at token: "+(end+7)+" of "+tokens.length);	
//	System.out.println("Last 7 tokens were: "+ATtokenOps.extract(tokens,(end+1), (end+7)));
}
public void deleteMe() {
	grfxObj.deleteMe();
}
/**
 * This method was created in VisualAge.
 */
public void draw(java.awt.Graphics g) {
	if (grfxObj != null) {
		grfxObj.drawLine(g);
		if(Selected){
			grfxObj.drawSelected(g);
		}
	}
}
/**
 * This method was created in VisualAge.
 * @param g Graphics
 * @param InitOrResp int
 */
public void draw(java.awt.Graphics g, int InitOrResp) {
	if(InitOrResp == 1){
		initiator.draw(g);
	} else if(InitOrResp == 2){
		responder.draw(g);
	}
}
/**
 * This method was created in VisualAge.
 * @return system.ATagentGRFX
 */
public ATconvGRFX getGrfx() {
	return grfxObj;
}
/**
 * This method was created in VisualAge.
 * @param a system.ATagent
 */
public ATagent getInitiator() {
	try{
	if (initiator==null) return null;
	}
	catch (NullPointerException x) {
		return null;
	}
	return initiator.getParticipant();
}
/**
 * This method was created in VisualAge.
 * @return system.ATstatetable
 * @param s java.lang.String
 */
public String getInitiatorKey() {
	return initiator.getParticipantKey();
}
/**
 * Insert the method's description here.
 * Creation date: (10/20/99 1:15:09 PM)
 */
public String getMAML() throws Exception {
	MAML="";
	//Opening line for an Conversation element			
	MAML+="\n\t<conversation";
	MAML+=" version=\"" + Double.toString(version) + "\"";
	MAML+=" name=\"" + name + "\">\n";
	//New Line for Description element
	
	MAML += " key=\"" + getKey() + "\"";
	
	MAML+="\t\t<description>\n\t\t";
	MAML+=getDescription();
	MAML+="\n\t\t</description>\n";
	//Opening line for an Participant element
	MAML+="\t\t<participant";
	MAML+=" name=\"initiator\">\n";
	MAML+=initiator.getMAML();
	//Closing line for an Participant element
	MAML+="\t\t</participant>\n";	
	//Opening line for an Participant element
	MAML+="\t\t<participant";
	MAML+=" name=\"responder\">\n";
	MAML+=responder.getMAML();
	MAML+="\n\t\t<view";
	MAML+=" n_pts=\""+grfxObj.n()+"\"";//>\n";
	for(int j = 0; j<grfxObj.n(); j++){
		MAML+=" x" +j+ "=\"" +grfxObj.x(j) +"\"";
		MAML+=" y" +j+ "=\"" +grfxObj.y(j) +"\"";
	}
	MAML+=" linetype =\"" + grfxObj.type() + "\"";///>\n";
	MAML+=" POS1 =\"" + grfxObj.pos1 + "\" \"" + grfxObj.num1 + "\"";
	MAML+=" POS2 =\"" + grfxObj.pos2 + "\" \"" + grfxObj.num2 + "\"/>\n";

	//Closing line for an Participant element
	MAML+="\t\t</participant>\n";
	MAML+="\t</conversation>\n";

	return MAML;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getMultiplicity() {
	return mult;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getName() {
	return name;
}
/**
 * This method was created in VisualAge.
 * @return system.ATstatetable
 * @param s java.lang.String
 */
public ATstatetable getParticipant(ATagent a) {
	if (initiator.getParticipantKey().equals(a.getKey()))
	{
		return initiator;
	} else
	if (responder.getParticipantKey().equals(a.getKey()))
	{
		return responder;
	}
	return null;
}
/**
 * This method was created in VisualAge.
 * @param a system.ATagent
 */
public ATagent getResponder() {
	try{
	if (responder==null) return null;
	}
	catch (NullPointerException x) {
		return null;
	}
	return responder.getParticipant();
}
/**
 * This method was created in VisualAge.
 * @return system.ATstatetable
 * @param s java.lang.String
 */
public String getResponderKey() {
	return responder.getParticipantKey();
}
/**
 * This method was created in VisualAge.
 * @return system.ATstatetable
 */
public ATstatetable getStateTable(int InitOrResp) {
	if(InitOrResp == 1) {
		return initiator;
	} else if(InitOrResp == 2) {
		return responder;
	}
	return null;
}
/**
 * Insert the method's description here.
 * Creation date: (10/28/99 10:36:53 AM)
 */
public void integrateKBInfo(String maml, ATsystem sys) {
	
  String[] tokens = ATtokenOps.parseLine(maml);
  int v=Integer.parseInt(tokens[6]);
	
  //Code for constructing object model from MAML tokens
  if (v==version) {
	constructFromV1(tokens, sys);
  }
  else {
	System.out.println("Loading a version other than 1");	
  }
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public void movePoint(int x, int y) {
	grfxObj.movePoint(x,y);
}
/**
 * This method was created in VisualAge.
 */
public java.awt.Graphics print(java.awt.Graphics g) {
	grfxObj.paint(g);
	return g;
}
/**
 * This method was created in VisualAge.
 * @param agent system.ATagent
 */
public void resize(ATagent agent) {
	grfxObj.resize(agent.getGrfx());
}
/**
 * Insert the method's description here.
 * Creation date: (3/28/00 3:01:00 PM)
 */
public void reverse() {
	ATstatetable hold = initiator;
	initiator = responder;
	responder = hold;
	grfxObj.reverse();
}
/**
 * This method was created in VisualAge.
 */
public void setCoordinates(ATagent first, ATagent second) {

	String key = getKey();
	int pos = key.indexOf('@');
	int keynum = 0;
	try{
		keynum = Integer.parseInt(key.substring(pos+1, key.length()));
	} catch (NumberFormatException e) {}
	
	grfxObj.place(first.getGrfx(),second.getGrfx());
	initiator = new ATstatetable(first,version,keynum);
	responder = new ATstatetable(second, version,keynum);
}
/**
 * This method was created in VisualAge.
 */
public void setCoordinates2(ATagent first, ATagent second) {

	grfxObj.place(first.getGrfx(),second.getGrfx());
	initiator.setParticipant(first);
	responder.setParticipant(second);
}
/**
 * This method was created in VisualAge.
 * @param a system.ATagent
 */
public void setInitiator(ATagent a) {
	grfxObj.place(a.getGrfx(),getResponder().getGrfx());
	initiator.setAgent(a);
//	initiator = new ATstatetable(a);
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setMultiplicity(String s) {
	mult = s;
	grfxObj.setInitiator(s);
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setName(String s) {
	name = ATtokenOps.validName(s);
	grfxObj.setName(name);
}
/**
 * This method was created in VisualAge.
 * @param a system.ATagent
 */
public void setResponder(ATagent a) {
	grfxObj.place(getInitiator().getGrfx(),a.getGrfx());
	responder.setAgent(a);
}
/**
 * Insert the method's description here.
 * Creation date: (7/17/00 2:17:09 PM)
 */
public void writeDATA() {
	super.writeDATA();
	MAML+= "\tAgent1 = " + getInitiatorKey() +"\n";
	MAML+= "\tAgent2 = " + getResponderKey() +"\n";
}
}
