package system;

/**
 * transition between two states
 * This type was created in VisualAge.
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 import java.util.*;
public class ATtransition extends ATobject{
	private ATtableElement currentState;
	private ATtableElement nextState;
		
	private String recMessage;
	private String guardCondition;
	private String transMessage;
	//private String csname;
	//private String nsname;
	private Vector actions = new Vector();
	// graphics info
	private ATtransitionGRFX grfxObj;
	public boolean Selected;
	public boolean errorSelected = false;
	private static final String ukh = "trans@";
/**
 * ATtransition constructor comment.
 */
public ATtransition(String[] tokens, double v, ATstatetable s, ATsystem sys) {
	this(v,0);

	//initializing variables used in parsing
	String currenttoken=null;
	String submaml=null;
	
	boolean keybased = false;
	
	int loc  = ATtokenOps.findnextinstanceof("key", tokens, 0);
	if(loc != -1) {
		setKey(tokens[loc+3]);
		keybased = true;
	} else {
		setKey(ukh + sys.getNextKey());
	}
	
  //Code for constructing object model from MAML tokens
  //Since some of these atributes may be empty, we use conditional assignments
  //For instance, a name="" means no name and no token between the quotes
  //this effects future token locations by 1
  //This leads us to use referencial rather than absolute location for token extraction
  //the 'loc' variable provides this moving reference
  
  if (v>=1) {
	
	loc  = ATtokenOps.findnextinstanceof("rMessage", tokens, loc) + 3;
	recMessage    =  (tokens[loc].equals("\"") ? "" : tokens[loc]);
	
	loc  = ATtokenOps.findnextinstanceof("tMessage", tokens, loc) + 3;
	transMessage  =  (tokens[loc].equals("\"") ? "" : tokens[loc]);
	
	loc  = ATtokenOps.findnextinstanceof("guard", tokens, loc) + 3;
	guardCondition=  (tokens[loc].equals("\"") ? "" : tokens[loc]);
	
	loc  = ATtokenOps.findnextinstanceof("cstate", tokens, loc) + 3;
	String csname        =  (tokens[loc].equals("\"") ? "" : tokens[loc]);
	
	loc  = ATtokenOps.findnextinstanceof("nstate", tokens, loc) + 3;
	String nsname        =  (tokens[loc].equals("\"") ? "" : tokens[loc]);
	
	int loc2  = ATtokenOps.findnextinstanceof("action", tokens, loc);
	while(loc2 != -1){
		loc2 +=3;
		currenttoken = (tokens[loc2].equals("\"") ? "" : tokens[loc2]);
		addAction(currenttoken);
		loc2  = ATtokenOps.findnextinstanceof("action", tokens, loc2);
	}

	if(keybased) {
		currentState = s.findStatebykey(csname);
		nextState = s.findStatebykey(nsname);
	} else {
		currentState = s.findStatebyname(csname);
		nextState = s.findStatebyname(nsname);
	}
	if(currentState == null) {
		System.out.println("Error!!!  State " + csname + " not found.");
	} else if(nextState == null) {
		System.out.println("Error!!!  State " + nsname + " not found.");
	} else {
		setCoordinates(currentState,nextState);
	}
	
	if(v>1){

		loc = ATtokenOps.findnextinstanceof("view",tokens,loc);
		int j = 0;
		if(loc>0){
			j = Integer.parseInt(tokens[loc+5]);
			loc= loc +5;
		}	

		int[] xpos = new int[j];
		int[] ypos = new int[j];
		
		for(int i=0; i<j; i++){
			loc+=12;
			xpos[i] = Integer.parseInt(tokens[loc-6]);
			ypos[i] = Integer.parseInt(tokens[loc]);
		}

		grfxObj.initializePoints(xpos,ypos);
		grfxObj.Arrow();
	
		grfxObj.setType(tokens[loc+7]);
		char p1 = (tokens[loc+14]).charAt(0);
		int n1 = Integer.parseInt(tokens[loc+18]);
		char p2 = (tokens[loc+25]).charAt(0);
		int n2 = Integer.parseInt(tokens[loc+29]);
		loc+=25;
		
		grfxObj.setPos(p1,n1,p2,n2);
	}
	
	loc = ATtokenOps.findnextinstanceof("se",tokens,loc-3);
	if(loc >= 0) {
		grfxObj.start = (tokens[loc+4].equalsIgnoreCase("t"));
		grfxObj.end = (tokens[loc+8].equalsIgnoreCase("t"));
	}	
		
	grfxObj.Arrow();
	updateText();
  }
}
//	private ATtableElement first;
//	private ATtableElement second;
/**
 * ATtransition constructor comment.
 */
public ATtransition(double v, int keynum) {
	super(v);
	setKey(ukh + keynum);
	grfxObj = new ATtransitionGRFX();
	currentState   = null;
	nextState      = null;
	recMessage     = "";
	guardCondition = "";
	transMessage   = "";
//	action         = "";
}
/**
 * ATtransition constructor comment.
 */
public ATtransition(String maml, double v, ATstatetable s, ATsystem sys) {
	this(ATtokenOps.parseLine(maml),v,s,sys);
}
//	private ATtableElement first;
//	private ATtableElement second;
/**
 * ATtransition constructor comment.
 */
public ATtransition(ATtransition cp) {
	super(cp);
}
/**
 * This method was created in VisualAge.
 * @param a afit.system.ATagent
 */
public void addAction(String a) {
	actions.addElement(a);
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public void addPoint(int x, int y) {
	grfxObj.addPoint(x,y);
	grfxObj.Arrow();
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public void arrange_pts() {
	grfxObj.arrange_pts();
	grfxObj.Arrow();
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public boolean checkSelected(int x, int y) {
	if(grfxObj.dist(x,y)) {
		Selected = true;
		return false;
	} else {
		Selected = false;
		return true;
	}
}
/**
 * This method was created in VisualAge.
 * @return boolean
 * @param agent afit.system.ATagent
 */
public boolean checkState(ATtableElement state) {
	boolean match;
	match = grfxObj.checkMatch(state.getGrfx());
	return match;
}
/**
 * This method was created in VisualAge.
 */
public void deleteMe() {
	grfxObj.deleteMe();
}
/**
 * This method was created in VisualAge.
 * @param g java.awt.Graphics
 */
public void draw(java.awt.Graphics g) {
	if(errorSelected) {
		grfxObj.drawError(g);
	}
	
	grfxObj.drawLine(g);
	
	if(Selected){
		grfxObj.drawSelected(g);
	}
}
//	MAML+="/>\n";	
//}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public Vector getActions() {
	return actions;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public ATtableElement getCurrentState() {
	return currentState;
}
/**
 * Insert the method's description here.
 * Creation date: (11/28/00 2:13:34 PM)
 * @return boolean
 */
public boolean getEnd() {
	return grfxObj.end;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getGuardCondition() {
	return guardCondition;
}
/**
 * Insert the method's description here.
 * Creation date: (10/20/99 2:44:02 PM)
 */
public String getMAML() throws NullPointerException {
	Enumeration actions = returnActions();
	MAML="";
	String hold=null;
	//Opening line for an Transisiton element
	MAML+="\t\t\t\t<transition";
	
	MAML += " key=\"" + getKey() + "\"";
	
	MAML+=" rMessage=\"" + recMessage + "\"";
	MAML+=" tMessage=\"" + transMessage + "\"";
	MAML+=" guard=\"" + guardCondition + "\"";
	
	MAML+=" cstate=\"" + ((currentState==null) ? "none" : currentState.getKey()) + "\"";	
	MAML+=" nstate=\"" + ((nextState==null) ? "none" : nextState.getKey()) + "\"";
	//MAML+=" cstate=\"" + ((currentState==null) ? "none" : currentState.getName()) + "\"";	
	//MAML+=" nstate=\"" + ((nextState==null) ? "none" : nextState.getName()) + "\"";
	
	
	while(actions.hasMoreElements()){
		hold =(String)actions.nextElement();;
		MAML+=" action=\"" + hold + "\"";//Temporary Fix
	}
	//MAML+=" action=\"" + action"\"";
	MAML+="\n\t\t\t\t<view";
	MAML+=" n_pts=\""+grfxObj.n()+"\"";//>\n";
	for(int j = 0; j<grfxObj.n(); j++){
		MAML+=" x" +j+ "=\"" +grfxObj.x(j) +"\"";
		MAML+=" y" +j+ "=\"" +grfxObj.y(j) +"\"";
	}
	MAML+=" linetype =\"" + grfxObj.type() + "\"";///>\n";
	MAML+=" POS1 =\"" + grfxObj.pos1 + "\" \"" + grfxObj.num1 + "\"";
	MAML+=" POS2 =\"" + grfxObj.pos2 + "\" \"" + grfxObj.num2 + "\"";
	
	MAML+=" se =\"" + ((grfxObj.start) ? "t":"f")+ "\" \"" + ((grfxObj.end) ? "t":"f")+ "\"/>\n";

	return MAML;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public ATtableElement getNextState() {
	return nextState;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getRecMessage() {
	return recMessage;
}
/**
 * Insert the method's description here.
 * Creation date: (11/28/00 2:13:34 PM)
 * @return boolean
 */
public boolean getStart() {
	return grfxObj.start;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getTransMessage() {
	return transMessage;
}
/**
 * This method was created in VisualAge.
 * @param x int
 * @param y int
 */
public void movePoint(int x, int y) {
	grfxObj.movePoint(x,y);
	grfxObj.Arrow();
}
/**
 * ATtransition constructor comment.
 */
public void oldATtransition(ATstate scurrentState, String srecMessage,
	String sguardCondition, String stransMessage, ATstate snextState) {
	//this(1);

	currentState   = scurrentState;
	recMessage     = srecMessage;
	guardCondition = sguardCondition;
	transMessage   = stransMessage;
	nextState      = snextState;
}
/**
 * This method was created in VisualAge.
 */
public java.awt.Graphics print(java.awt.Graphics g) {
	grfxObj.paint(g);
	return g;
}
/**
 * This method was created in VisualAge.
 * @param agent system.ATagent
 */
public void resize(ATstate state) {
	grfxObj.resize(state.getGrfx());
	//grfxObj.Arrow();
}
/**
 * Insert the method's description here.
 * Creation date: (3/14/00 3:13:33 PM)
 */
public Enumeration returnActions() {
	return actions.elements();
	}
/**
 * Insert the method's description here.
 * Creation date: (3/28/00 3:01:00 PM)
 */
public void reverse() {
	ATtableElement hold = currentState;
	currentState = nextState;
	nextState = hold;
	grfxObj.reverse();
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setAction(Vector s) {
	actions = s;
	updateText();
}
/**
 * This method was created in VisualAge.
 * @param x1 int
 * @param y1 int
 * @param x2 int
 * @param y2 int
 */
public void setCoordinates(ATtableElement first, ATtableElement second) {
	this.setCurrentState(first);
	this.setNextState(second);
	if(first == second){
		grfxObj.setType("loop");
		grfxObj.placeLoop((ATstateGRFX)first.getGrfx());
	} else {
		grfxObj.place(first.getGrfx(),second.getGrfx());
		grfxObj.checkEnds();
		grfxObj.Arrow();
	}
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setCurrentState(ATstate s) {
	currentState = (ATtableElement)s;
}
/**
 * This method was created in VisualAge.
 * @param a system.ATagent
 */
public void setCurrentState(ATtableElement s) {
	currentState = s;
}
/**
 * Insert the method's description here.
 * Creation date: (11/28/00 2:06:51 PM)
 * @param v boolean
 */
public void setEnd(boolean v) {
	grfxObj.end = v;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setGuardCondition(String s) {
	guardCondition = s;
	updateText();
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setNextState(ATstate s) {
	nextState = (ATtableElement)s;
}
/**
 * This method was created in VisualAge.
 * @param a system.ATagent
 */
public void setNextState(ATtableElement s) {
	nextState = (ATtableElement)s;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setRecMessage(String s) {
	recMessage = s;
	updateText();
}
/**
 * Insert the method's description here.
 * Creation date: (11/28/00 2:06:51 PM)
 * @param v boolean
 */
public void setStart(boolean v) {
	grfxObj.start = v;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setTransMessage(String s) {
	transMessage = s;
	updateText();
}
public void updateText() {
	String t = "";

	if (this.getRecMessage() != null && !this.getRecMessage().equals("")) {
		t = t + this.getRecMessage();
	}

	if (this.getGuardCondition() != null && !this.getGuardCondition().equals("")) {
		t = t + "[" + this.getGuardCondition() +"]";
	}

	if (this.getActions() != null/* && !this.getAction().equals("")*/) {
		Enumeration actions = returnActions();
		String s1,s = "";
		while(actions.hasMoreElements()){
			s1 = (String)actions.nextElement();
			if(s!=""){
				s = s + ";" + s1;
			} else {
				s = s1;
			}
		}
		if(s!=""){
			t = t + "/" + s;//this.getAction();
		}
	}

	if (this.getTransMessage() != null && !this.getTransMessage().equals("")) {
		t = t + "^" + this.getTransMessage();
	}
	
	grfxObj.setText(t);
}
}
